<?
/**
* @defgroup BaseJumper BaseJumper Framework
* @brief BaseJumper is a library that does all sorts of sweet things.
*
* @{
*/

/**
* @brief BaseJumper is the controller for your whole site.
*
* It handles module loading, initialization, template stuff, and much more.
*/
class BaseJumper
{
	/**
	* The name of our default class.
	*/
	public $defaultClass;
	
	/**
	* The name of our default template.
	*/
	public $defaultTemplate;

	/**
	* Our instance of the module object
	*/
	public $object;

	/**
	* our database tables - overview array
	*/
	public static $tables = null;

	/**
	* our list of data for each table.
	*/
	public static $tableData = array();

	/**
	* how long do we cache our table structure data for? (in seconds)
	*/
	public static $tableCacheLife = 3000000;

	/**
	* do we track hits to the site on module / page basis?
	*/
	public static $trackHits = true;

	/**
	* table to track hits to the site on.
	*/
	public static $hitTrackingTable = 'site_hits';

	/**
	* this is our list of module names in string form.
	*/
	private $modules = array();
	
	/**
	* here is our constructor.  you provide the default class and default template, it handles the rest.
	*
	* @param $defaultClass the string name of the default class if none are supplied.  Defaults to 'Main'
	* @param $defaultTemplate the string name of the default template to use.  Defaults to 'Template'
	*/
	public function __construct($defaultTemplate = 'DefaultTemplate', $defaultClass = 'Main')
	{
		//set our stuff.
		$this->defaultClass = $defaultClass;
		$this->defaultTemplate = $defaultTemplate;

		if (!Config::get('basejumper_installed'))
		{
			self::$trackHits = false;
		}
	}

	/**
	* this function adds a module to the list of modules that BaseJumper will serve up.
	*
	* @param $module the string name of the module to add
	*/
	public function addModule($module)
	{
		$module = strtolower($module);
		
		//if we dont already have it, add it in.
		if (!$this->hasModule($module))
			$this->modules[] = $module;
	}

	/**
	* this function will add a whole bunch of modules in at the same time.
	*
	* @param $modules and array of string module names
	*/
	public function addModules($modules)
	{
		foreach ($modules AS $module)
			$this->addModule($module);
	}

	/**
	* this function removes a module from the list of modules that BaseJumper will serve, effectively disabling it.
	*
	* @param $module the string name of the module to remove
	*/
	public function removeModule($module)
	{
		$module = strtolower($module);
		
		//do we have the module loaded?
		if ($this->hasModule($module))
			//yup, go thru each of our modules.
			foreach ($this->modules AS $key => $val)
				//if this is the right one, delete it.
				if ($val == $module)
					unset($this->modules[$key]);
	}

	/**
	* this function tells us if we have a module loaded or not.
	* 
	* @param $module the string name of the module to check
	*/
	public function hasModule($module)
	{
		$module = strtolower($module);
		
		return in_array($module, $this->modules);
	}

	/**
	* get an array of all the tables in our database... used for autodetection stuff
	*/
	public static function getDbTables()
	{
		self::lookupDbTables();

		return self::$tables;
	}

	/**
	* get an array of data about our table... basically the result form DESCRIBE $table
	*
	* @param $table teh name of the table we want info on.
	*
	* @return the array of info on that table.
	*/
	public static function getDbTable($table)
	{
		self::lookupDbTable($table);

		return self::$tableData[$table];
	}

	/**
	* get an array of index information about our table. based on SHOW INDEXES
	*
	* @param $table the name of the table we want info on.
	*
	* @return the array of index info on that table.
	*/
	public static function getTableIndexes($table)
	{
		//init
		$rval = array();
		
		//get our keys from the table.
		$rs = dbQuery("
			SHOW INDEXES FROM $table
		");
		while ($ar = dbFetchAssoc($rs))
			$rval[] = $ar;

		//send her home
		return $rval;
	}

	/**
	* tells us if we have a table in the database or not.
	*
	* @param the string name of a table.
	*
	* @return bool if it exists or not.
	*/
	public static function dbHasTable($table)
	{
		self::lookupDbTables();
	
		return isset(self::$tables[$table]);
	}
	
	/**
	* tells us if a table in the database has a field or not.
	*
	* @param table the string name of a table.
	* @param field the string name of the field.
	*
	* @return bool if it exists or not.
	*/
	public static function tableHasField($table, $field)
	{
		self::lookupDbTable($table);

		return isset(self::$tableData[$table][$field]);
	}

	/**
	* this function looks up all of the tables contained in the database and
	* stores them in our static variable.
	*/
	private static function lookupDbTables()
	{
		//if we already have it, do nothing
		if (self::$tables === null)
		{
			//try our cache.
			$rval = CacheBot::get("BaseJumper:Tables", self::$tableCacheLife);
			
			//no cache?  do it from db.
			if (!$rval)
			{
				//get all the tables.
				$tableRs = dbQuery("SHOW TABLES");
				while ($tableAr = dbFetchArray($tableRs))
					$rval[$tableAr[0]] = true;
				
				//cache it for 4 hours.
				CacheBot::set("BaseJumper:Tables", $rval, self::$tableCacheLife);
			}
			
			//send it home
			self::$tables = $rval;
		}
	}
	
	/**
	* this function looks up detailed information about a table in our
	* database and stores it on our static array.
	*/
	private static function lookupDbTable($table)
	{
		//if we already have it, do nothing
		if (!isset(self::$tableData[$table]))
		{
			//try our cache.
			$rval = CacheBot::get("BaseJumper:Table:$table", self::$tableCacheLife);
			
			//did we get it?
			if (!$rval)
			{
				//save the hassle (and exception) of looking up a nonexistant table.
				if (self::dbHasTable($table))
				{
					//nope have the db tell us.
					$descRs = dbQuery("DESCRIBE $table");
					while ($descAr = dbFetchAssoc($descRs))
					{
						//get our field.
						$field = $descAr['Field'];
						
						//save it.
						$rval[$field] = $descAr;
					}
					
					//cache it for 4 hours.
					CacheBot::set("BaseJumper:Table:$table", $rval, self::$tableCacheLife);
				}
			}

			//save it.
			self::$tableData[$table] = $rval;
		}
	}
	
	/**
	* this function sets up and initializes our global user.
	*/
	protected function checkCredentials()
	{
		//authenticate...
		global $me;
		$me = new User();
		$me->authenticate();
	}

	/**
	* this function checks to make sure we're on the right host and redirects
	* us if we're not.
	*/
	protected function checkHost()
	{
		//first check to see if we're on teh right server...
		if ($_SERVER["HTTP_HOST"] != Config::get('site_hostname'))
			Util::redirect("http://" . Config::get('site_hostname') . $_SERVER["REQUEST_URI"]);
	}

	/**
	* this is the main hook of the library.  it tells the object to draw
	* itself.  hefty work indeed.
	*/
	public function start()
	{
		//we want output buffering so we can add variables in...
		ob_start();

		//attempt to have the object draw itself.
		try
		{
			//make sure we're on the right server.
			$this->checkHost();

			//log our page request
			if (self::$trackHits)
				$this->trackPageHit();
			
			//try to get our page if its cached.
			$page = BaseModule::getCachedPage();
			if ($page)
				echo $page;
			else
			{
				//now do our checking.
				$this->checkCredentials();

				//do we need tables?
				if (Config::get('db_host'))
					self::getDbTables();

				//initialize our module
				$this->init();
				
				//turn output buffering on so we can show error forms and stuff cleanly.
				ob_start();
				$this->object->draw();
				$contents = ob_get_contents();
				ob_end_clean();

				//show the contents.
				echo $contents;

				//save it if we got it.
				if ($this->object->doCachePage)
					BaseModule::setCachedPage($contents);
			}
		}
		//have our error handler clean up any juices.
		catch (Exception $e)
		{
			ErrorHandler::handleException($e);		
		}
		
		//get the output.
		$contents = ob_get_contents();
		ob_end_clean();
		
		//add in our stuff...
		echo $this->filterFinalOutput($contents);
	}

	/**
	* this function adds in stuff liek runtime, queries, etc to the body.
	* %%BJ_RUNTIME%% gets replaces with the time in seconds, &&BJ_CACHE_HITS%%
	* gets replaced with the number of cache hits, and %%BJ_QUERIES%% gets
	* replaces with the number of db queries used.
	*
	* @param $output the whole site output to use
	* @return the data to display to the user =)
	*/
	protected function filterFinalOutput($output)
	{
		global $runtimeStart, $sqlQueryCount;

		//how long did it take?
		$runtimeStop = microtime(true);
		$total = round($runtimeStop - $runtimeStart, 4);
		
		//replace our shit.
		$output = str_replace("%%BJ_RUNTIME%%", $total, $output); 
		$output = str_replace("%%BJ_QUERIES%%", (int)$sqlQueryCount, $output);
		$output = str_replace("%%BJ_CACHE_HITS%%", (int)EasyCache::$hits, $output);

		//finally show our site.
		return $output;
	}

	/**
	 * This function initializes everything.  It creates the object that
	 * does the actual drawing and initializes it.
	 *
	 * @return true on success, false on failure
	 */
	protected function init()
	{
		//we want our default module.
		$this->setModule(new BaseModule());
		
		//make the object init.
		try
		{
			//well first try and load our object.
			$object = $this->loadModule();
			
			//did we get an object?
			if ($object)
			{
				$this->setModule($object);
				
				//get our parameters array..
				$page = $this->getPage();
				$params = $this->getParams();
				
				//did we get a global template passed in?
				$template = $this->getGlobalTemplate($params);
				if ($template instanceOf BaseTemplate)
					$this->object->setTemplate($template);
				
				//check our auth code.
				$this->checkAuthCode($params);
				
				//finially, init!
				$this->object->init($page, $params);
				
				//we may need to reset the template if the module set its own template
				if ($template instanceOf BaseTemplate && get_class($template) != get_class($this->object->template))
					$this->object->setTemplate($template);

				//let them know we were successful
				return true;
			}
			//no object... return false
			else
				return false;
		}
		//did we get any errors?
		catch(PageError $err)
		{
			$this->object->setError($err->getMessage());
		}
	}

	/**
	* this sets our module for the page we're drawing.  it loads the default template and such.
	*
	* @param $module a BaseModule derived object.
	*/
	public function setModule(BaseModule $module)
	{
		//save the module
		$this->object = $module;
				
		//give it a default template.
		$this->object->setTemplate(new $this->defaultTemplate());
	}

	/**
	* check our parameters and figure out what our page is.
	*
	* @return the string name of our page to draw.
	*/
	protected function getPage()
	{
		if (isset($_REQUEST['page']))
			return $_REQUEST['page'];
		else
			return 'main';
	}

	/**
	* check our request vars to get our parameters.  this is unvalidated, just
	* the raw request.
	*
	* @return the array of key to value for parameters
	*/
	protected function getParams()
	{
		//start off with all our request vars.
		$vars = $_REQUEST;
		
		//do our url rewriting stuff
		if (Config::get("use_mod_rewrite"))
		{
			//get our params
			$params = $vars['params'];
			unset($vars['params']);

			//explode our parameters
			$paramA = explode('/', $params);
			if (count($paramA) && $params != '')
			{
				//put them in the array
				if (count($paramA) % 2 == 0)
					for ($i = 0; $i < count($paramA); $i += 2)
						$vars[$paramA[$i]] = $paramA[$i+1];
				else
					throw new PageError('Wrong parameter count.');
			}
		}

		//send them home
		return $vars;
	}
	
	/**
	* this function will check our request to see if they want a special
	* template.  if so, then return that, otherwise give us a default template.
	* the 'template' variable is checked and if found it will be used and then unset.
	*
	* @return a BaseTemplate derived object.
	*/
	protected function getGlobalTemplate($params)
	{
		//did we get one passed in?
		if (isset($params['bjtemplate']))
		{
			//does it exist?
			$class = trim($params['bjtemplate']) . "Template";
			if (class_exists($class))
			{
				//yup, unset it and send it home.
				unset($params['bjtemplate']);
				return new $class();
			}
		}
	}

	/**
	* this fucntion checks to see if an authorization code was passed in.  if
	* it does, it attempts to use it to authenticate our user.  if it fails, ti
	* will trigger an error.  on success, it jsut logs the user in.
	*
	* @param $params an array of parameters to check
	*/
	protected function checkAuthCode($params)
	{
		//did we get one passed in?
		if (isset($params['bjauthcode']))
		{
			//get it..
			$code = trim($params['bjauthcode']);

			//try our auth code!!
			$user = new User();	
			if (!$user->tryAuthCode($code, false))
				throw new PageError('Authorization code failed.');
		}
	}
	
	/**
	 * This function loads the object.  It supports auto loading of modules
	 * so you dont have to include them off the bat.
	 *
	 * @param $module - if given, will attempt to load this module.
	 *
	 * @return teh module object on success, false on failure
	 */
	protected function loadModule($module = null)
	{
		//did we get a module?
		if ($module === null)
		{
			//figure out what our class is.
			if (isset($_REQUEST['module']))
				$module = trim($_REQUEST['module']);
			else
				$module = $this->defaultClass;
		}

		//does it already exist?
		if ($this->hasModule($module))
		{
			//if we can find the class.. do it.
			if (class_exists($module))
				return new $module();
			//otherwise, this is a programmer error, throw an exception for them.
			else
				throw new Exception("Class $module not found.");
		}
		//nope, its not on our list... you cant get in.
		else
			throw new PageError("The module '$module' does not exist in " . Config::get('site_name') . ".");
	
		return false;
	}
	
	/**
	* This function tries to autoload our class files!!
	*
	* @param $class the name of the class to autoload
	*
	* @return true on success false on failure
	*/
	public static function autoload($class)
	{
		//set up some default names...
		$lowerClass = strtolower($class);
		$dashClass = strtolower(preg_replace("/([a-z])([A-Z])/", "$1-$2", $class));
		
		//if it doesnt exist, can we open the file?
		if (file_exists("include/$lowerClass.inc.php"))
		{
			require_once("include/$lowerClass.inc.php");
			return true;
		}
		//if it doesnt exist, can we open the file?
		else if (file_exists("include/$dashClass.inc.php"))
		{
			require_once("include/$dashClass.inc.php");
			return true;
		}
		//if it doesnt exist, can we open the file?
		else if (file_exists("include/{$class}.inc.php"))
		{
			require_once("include/{$class}.inc.php");
			return true;
		}

		return false;
	}

	/**
	*  this function gets the create syntax for the module usage tracking
	*  table.  this will track all the hits your site is getting by module and
	*  page in that module.
	*
	* @return mysql create string
	*/
	public function getCreateHitTrackingTable()
	{
		return 
"CREATE TABLE " . self::$hitTrackingTable . "
(
	id int(11) not null auto_increment,
	module varchar(64) default '' not null,
	page varchar(64) default '' not null,
	access_date datetime not null,
	PRIMARY KEY(id)
) TYPE=MyISAM;";
	}

	/**
	* this will track a hit to a certain page in a certain module in the database
	*
	* @param $module the text name of the module.
	* @param $module the text name of the page in the module.
	*/
	public function trackPageHit($module = null, $page = null)
	{
		//set some defaults.
		if ($module === null)
			$module = $_REQUEST['module'];
		if ($page === null)
			$page = $_REQUEST['page'];

		//safety first
		$module = dbEscape($module);
		$page = dbEscape($page);

		//track it.
		dbExecute("
			INSERT INTO " . self::$hitTrackingTable . "
				(module, page, access_date)
			VALUES
				('$module', '$page', NOW())
		");
	}

	/**
	* remove all the hit tracking records from the db.
	*/
	public function clearHits()
	{
		dbExecute("DELETE FROM " . self::$hitTrackingTable);
	}
	
	/**
	* this returns a sorted array of all the module names this BaseJumper knows how to handle
	*
	* @return an array of the module names (in string format)
	*/
	public function getModulesArray()
	{
		//sort first!
		sort($this->modules);
		
		return $this->modules;
	}

	/**
	* this function gets the xml string describing all the modules in our system.
	*
	* @return an xml string with each module data as generated by each module.
	*/
	public function getModulesXml()
	{
		$xml = '<modules>';

		//get our modules xml data.
		$modules = $this->getModulesArray();
		foreach ($modules AS $module)
		{
			$obj = new $module;
			$xml .= $obj->getModuleXml();
		}

		$xml .= '</modules>';

		return $xml;
	}
}

/*@}*/

/**
* @defgroup BJHelpers Helper Classes
* @ingroup BaseJumper
*
* @brief Helper classes / functions for BaseJumper
*
* These classes and functions provide a variety of help to BaseJumper.  Most
* come as either singleton or static member classes for ease of use and
* autoload support.
*/

/**
* @defgroup BJModules Module System
* @ingroup BaseJumper
*
* @brief Modules are the core of the BaseJumper system.
*
* Modules make up the meat of your system.  They are the ones how handle the
* initialization and drawing of each page.  In addition, there are specialized
* modules that handle objects, searching, tags, users, and much more.  There
* are a variety of pre-built modules you can extend off of, or your can create
* your own.
*/

/**
* @defgroup BJTemplate Template System
* @ingroup BaseJumper
*
* @brief Templates control control how your pages are drawn.
*
* Templates control the drawing of the pages. It has access to the module and
* control how the header / footers are drawn, titles, errors, and status
* messages.  Included are classes you can build off of to do a variety of
* things.  HTML, XML, JSON, and AJAX (blank) are all possibilities that are
* easy and fun.
*/

/**
* @defgroup BaseModules Basic Modules
* @ingroup BJModules
* @brief Basic modules that dont do much or any database stuff.
*
* These include the main modules, admin, etc. 
*/

/**
* @defgroup BaseObjects Base Objects
* @ingroup BJModules
* @brief These modules handle creation, editing, viewing, and deletion of objects.
*
* These modules provide an easy, abstracted way to interface with a database
* backed content object.  With a variety of functions for ease of use, they
* allow you to directly work with the data and not have to worry about SQL.
*/

/**
* @defgroup ObjectManagers Object Managers
* @ingroup BJModules
* @brief These modules handle object searching, sorting, and display.
*
* These modules provide an easy way to interface with your BaseObjects.
* Searching, sorting, pagination, multiple data output formats (html, rss,
* json, php, xml) and much much more.
*/

?>
