<?php
require_once('field.inc.php');

/**
* Abstract field class.  Most (read: all) fields inherit from this.
* @ingroup BasicFields 
* @author Scott Peshak, Zach Smith
*/
abstract class AbstractField implements Field
{
	public $name = ''; ///< Name of the Field, must correspond with a database column name if \c sync is used.
	protected $title = ''; ///< Display title for the form.
	protected $value = ''; ///< Data value.
	protected $attribs = array(); ///< Array of extra attributes.
	protected $required = false; ///< Boolean flag, if \c true then the field will not validate until a vaild value is input.
	protected $regExp = ''; ///< Optional regular expression that is use to validate the input
	protected $errorMessage = ''; ///< Error message to display if an error is encountered.
	protected $hasError = false; ///< Boolean flag set after validation.
	protected $parentForm = ''; ///< Reference to the parent Form object.
	public $definition = '';

	
	/**
	 * Constructor
	 * @param $name Field name
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.  Any non special key values will simply be output in the HTML of the field.
	 *	Special keys include:
	 *		- \c value Sets a default value for the field
	 *		- \c required If \c true then the field is "required" (will not validate with an empty value)
	 *		- \c errorMessage Message to display if an error is encountered during validation. 
	 *		- \c regExp	Regular expression to use for validation.  The special values of \c number & \c float can be used.
	 *	Inheirited fields may define additional special keys as needed.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		$this->name = $name;
		$this->parentForm = $parentForm;

		$this->required = false;
		$this->parseExtras($attribs);
		$this->title = array_key_exists('title', $attribs) ? $attribs['title'] : ucwords(str_replace('_', ' ', $name));
		$this->errorMessage = "The $this->title field had an error.";
		
		if(!isset($this->attribs['id']))
			$this->attribs['id'] = $this->name;

		$this->hasError = null;
	}

	/**
	 * Magic getter
	 * @param $name Name of member that is being attempted
	 */
	public function __get($name)
	{
		if(isset($this->$name) || is_null($this->$name))
			return $this->$name;
		else if(array_key_exists($name, $this->attribs))
			return $this->attribs[$name];

		return false;
	}

	/**
	 * Magic setter
	 * @param $name Name of the member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		if($name == 'value')
			$this->setData($value);
		else if(isset($this->$name))
			$this->$name = $value;
		else
			$this->attribs[$name] = $value;

		return true;
	}

	/**
	 * Validate the feild's contents
	 */
	public function validate()
	{
		$this->setDataFromRequest();

		$this->hasError = false;
		
		if($this->required && $this->value == null)
		{
			$this->hasError = true;
			$this->errorMessage = "The $this->title field is required.";
		}
		
		if($this->regExp && !$this->hasError)
		{
			list($this->regExp, $this->errorMessage) = 
				array_values($this->convertSpecialRegex($this->regExp, $this->errorMessage));

			# Error if the string doesn't match the regex, or if the value is empty (required fields are handled above)
			$this->hasError = !((bool)preg_match($this->regExp, $this->value) || $this->value == null);
		}
	}

	public function getTitle()
	{
		if ($this->definition == '')
			return $this->title;
		else
			return "<acronym title=\"$this->definition\">$this->title</acronym>";
	}

	/**
	 * Return the status of the field after running validate
	 * @return Boolean \c true if there is an error, \c false otherwise
	 */
	public function hasError()
	{
		if($this->hasError === null)
			throw new NotValidatedException($this);

		return (bool)$this->hasError;
	}

	/**
	 * Force the field to have an error.  This is most useful for forcing all fields in a collection 
	 * to display as an error.
	 */
	public function forceError()
	{
		$this->hasError = true;
	}

	/**
	 * Get the SQL for this field.
	 * @return SQL string
	 */
	public function getSqlImpl()
	{
		return "$this->name = '$this->value'";
	}

	/**
	 * Set data
	 * @param $value Value to set
	 */
	public function setData($value)
	{
		$this->value = $value;
	}

	/**
	 * Get the data value
	 */
	public function getData()
	{
		return $this->value;
	}

	/**
	 * Get any JavaScript needed for the field
	 * This is for JavaScript that will be common for all instances of this field type.
	 */
	public function getJs()
	{
		return null; # No-Op
	}

	/**
	 * Get any CSS needed for the field
	 */
	public function getCss()
	{
		return null; # No-Op
	}

	/**
	 * Add more attributes
	 * @param $attr \c array of attributes
	 */
	public function addAttributes(array $attr)
	{
		$this->attribs = array_merge($this->attribs, $attr);
	}


	/**
	 * Parse the extra values
	 * @param $attribs array of extra attributes.
	 */
	protected function parseExtras(array $data)
	{
		$this->attribs = array(); # clear out any existing value

		foreach($data as $key => $value)
			$this->__set($key, $value);
	}


	/**
	 * Convert the extra attributes array as an HTML attribute string
	 * @return attribute string 
	 */
	protected function extrasAsAttributes()
	{
		$ret = array();
			
		foreach(array_change_key_case($this->attribs, CASE_LOWER) as $key => $value)
			$ret[] = "$key=\"$value\"";
		return implode(' ', $ret);
	}

	/**
	 * Set the form's data from the $_POST array
	 */
	protected function setDataFromRequest()
	{
		if(isset($_REQUEST[$this->name]))
			$this->value = $_REQUEST[$this->name];
	}

	
	/**
	 * Convert a special regex identifier (eg: 'number', 'float') into a real
	 * regex.  Also generates an error message appropirate for the resulting regex.
	 * @param $regExp RegEx string to convert (if it not a special identifier it is passed back out unchanged)
	 * @param $defaultError Default error message to use if no conversion takes place
	 * @return Array with keys 'regexp' and 'message'
	 */
	protected function convertSpecialRegex($regExp, $defaultError)
	{
		$error = $defaultError;
		switch($regExp)
		{
			case 'number':
				$regExp = '/^-?[0-9]+$/';
				$error = "The $this->title field must be a number.";
				break;

			case 'float':
				$regExp = '/^-?([0-9]*\.)?[0-9]+$/';
				$error = "The $this->title field must be a number.";
				break;

			case 'alpha':
				$regExp = '/^[A-Za-z]*$/';
				$error = "The $this->title field must consist of only alphabetic characters.";
				break;

			case 'alphanumeric':
				$regExp = '/^[A-Za-z0-9]*$/';
				$error = "The $this->title field must consist of only alpha numeric characters.";
				break;
			
			case 'email':
				$regExp = '/^[a-z0-9]+([_\\.-][a-z0-9]+)*@([a-z0-9]+([\.-][a-z0-9]+)*)+\\.[a-z]{2,}$/i';
				$error = "The $this->title field must be a real email.";
				break;
		}
		
		return array('regexp' => $regExp, 'message' => $error);
	}

	/**
	 * Assert that an attribute value is an array
	 * @param $value Value to check
	 */
	protected static function assertAttributeIsArray(&$value)
	{
		if(!is_array($value))
			throw new InvalidAttributeValueException($value, 'An Array'); 
	}

	/**
	 * Assert that an attribute value is an instance of an object type.
	 * @param $type Object type to check for.
	 * @param $value Value to check.
	 */
	protected static function assertAttributeIsObject($type, &$value)
	{
		if(!($value instanceof $type))
			throw new InvalidAttributeValueException($value, $type); 
	}
}
?>
