<?php
/**
* A field that is a set of multiple fields.
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-04-27 12:25:42 -0500 (Thu, 27 Apr 2006) $
*/
class CollectionField extends AbstractField
{
	protected $fields; ///< Array of Field objects that make up the collection
	protected $autoAdvance; ///< Boolean flag, if \c true then the user will be automatically advanced through the fields
	private static $skipFieldTypes = array('HiddenField', 'LabelField'); ///< Array of field types to skip over when auto advancing
	protected static $jsDrawn; ///< Boolean flag if the drawJs method had been called yet.
	protected $arrangement; ///< Mode to use for arranging the sub fields, either \c horizontal (default) or \c vertical

	/**
	 * Constructor
	 * @param $name Name of collection
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Extra attributes
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array()) 
	{
		$this->fields = array();
		parent::__construct($name, $parentForm, $attribs);
		self::$jsDrawn = false;
		if(!$this->arrangement)
			$this->arrangement = 'horizontal';
	}

	/**
	 * Magic setter
	 * @param $name Name of member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		switch($name)
		{
			case 'parentForm':
				foreach($this->fields as $field)
					$field->parentForm = $value;
				break;

			case 'arrangement':
				$this->arrangement = $value;
				break;

			case 'fields':
				self::assertAttributeIsArray($value);
				foreach($value as $param)
				{
					self::assertAttributeIsArray($param);
					$this->add($param[0], $param[1], is_array($param[2]) ? $param[2] : array($param[2]));
				}
				break;


			default:
				parent::__set($name, $value);
		}
	}


	/**
	 * Add field to collection.
	 * @param $type Field type.
	 * @param $name Name of field.
	 * @param $options Array of extra options.
	 * @return Reference to the added Field.
	 */
	public function &add($type, $name, array $options = array())
	{
		if($this->parentForm->hasField($name))
			throw new Exception("Field name $name is not unique.");

		$field = new $type($name, $this->parentForm, $options);
		$this->fields[] = $field;
		return $field;
	}

	/**
	* Tells us if we have a field or not.
	*
	* @param $name the field name to check for.
	*
	* @return boolean if it has it or not.
	*/
	public function hasField($name)
	{
		foreach ($this->fields AS $field)
			if ($field->name == $name)
				return true;

		return false;
	}

	/**
	* Sets the data on our fields in our collection
	*
	* @param $data the data to set to the fields.
	*/
	public function setFieldsData($data)
	{
		foreach ($this->fields AS $field)
		{
			if(isset($data[$field->name]))
				$field->setData($data[$field->name]);
		}
	}

	/**
	 * Draw the fields
	 */
	public function drawInput()
	{
		for($i = 0; $i < count($this->fields); $i++)
		{
			if($this->autoAdvance)
			{
				$this->fields[$i]->addAttributes(array(
					'onfocus' => 'this.formCollectionLength = this.value.length',
					'onkeydown' => 'formCollectionKD(this)',
					'onkeyup' => sprintf("formCollectionKU(window.event ? window.event : event, this, %s, %s)", $this->getPrevFieldId($i), $this->getNextFieldId($i))
				));
			}
			$this->fields[$i]->drawInput();

			echo ($this->arrangement == 'vertical') ? '<br/>' : " "; 
		}
	}

	/**
	 * \todo implement setData
	 */

	/**
	 * Set the field data from the $_REQUEST array
	 */
	public function setDataFromRequest()
	{
		foreach($this->fields as $field)
			$field->setDataFromRequest();
	}

	/**
	 * Validate all fields in the collection
	 */
	public function validate()
	{
		foreach($this->fields as $field)
			$field->validate();
	}

	/**
	 * Check to see if any fields in the collection have an error
	 * @return Boolean \c true if any fields have an error, \c false otherwise.
	 */
	public function hasError()
	{
		$ret = false;
		foreach($this->fields as $field)
			$ret = ($field->hasError() || $ret);
		
		return $ret || $this->hasError;
	}

	/**
	 * Get SQL for all fields in the collection.
	 * @return SQL string
	 */
	public function getSqlImpl()
	{
		$ret = array();
		foreach($this->fields as $field)
			$ret[] = $field->getSqlImpl();
			
		return implode(',', array_diff($ret, array('')));
	}

	
	/**
	 * Get the ID of the next field
	 * @param $num The number of the field to check for.
	 * @return ID of next field, or the string 'null' if there is no next.
	 */
	private function getNextFieldId($num)
	{
		if($num >= count($this->fields))
			return 'null';

		if(array_search(get_class($this->fields[$num + 1]), self::$skipFieldTypes) !== false)
			return $this->getNextFieldId($num + 1);

		return "'" . $this->fields[$num + 1]->id . "'";
	}

	/**
	 * Get the ID of the previous field for auto advance purposes
	 * @param $num The number of the field to check for.
	 * @return ID of previous field, or the string 'null' if there is no previous.
	 */
	private function getPrevFieldId($num)
	{
		if($num == 0)
			return 'null';

		if(array_search(get_class($this->fields[$num - 1]), self::$skipFieldTypes) !== false)
			return $this->getPrevFieldId($num - 1);

		return "'" . $this->fields[$num - 1]->id . "'";
	}

	/**
	 * Get a string of JavaScript needed for this field
	 * @return string of JavaScript
	 */
	public function getJs()
	{
		$subFields = null;

		foreach($this->fields as $field)
			$subFields .= $field->getJs();

		if(self::$jsDrawn || !$this->autoAdvance)
			return $subFields;

		self::$jsDrawn = true;		
		return <<<EOJAVASCRIPT
var formCollectionSave = null;

function formCollectionKD(field)
{
	field.formCollectionLength = field.value.length;
	formCollectionSave = field;
}

function formCollectionKU(evt, field, prev, next)
{
	var len;

	if (field == formCollectionSave)
	{
		if (evt.keyCode == 8)
		{
			if (prev && !(len = field.value.length) && len < field.formCollectionLength)
			{
				field = document.getElementById(prev);
				field.focus();
				field.value += '';
			}
		}
		else
		{
			if (next && (len = field.value.length) > field.formCollectionLength && len >= field.maxLength)
			{
				field = document.getElementById(next);
				field.focus();
				field.select();
			}
		}
	}

	formCollectionSave = null;
}

$subFields
EOJAVASCRIPT;
	}


	/**
	 * Get the CSS for all subfields.
	 * @return \c string of CSS
	 */
	public function getCss()
	{
		$css = null;

		foreach($this->fields as $field)
			$css .= $field->getCss();

		return $css;
	}


	/**
	 * Check to see if all fields in the collection have a data value
	 * @return Boolean \c true if all fields have some value
	 */
	public function allFieldsHaveData()
	{
		$ret = true;
		foreach($this->fields as $field)
		{
			if(array_search(get_class($field), self::$skipFieldTypes) === false)
				$ret = (($field->getData() != '') && $ret);
		}

		return $ret;
	}


	/**
	 * Check to see if none of the fields in the collection have a non-null value
	 * @return Boolean \c true if none of the fields have a value
	 */
	public function noFieldsHaveData()
	{
		$ret = true;
		foreach($this->fields as $field)
		{
			if(array_search(get_class($field), self::$skipFieldTypes) === false)
				$ret = (($field->getData() == '') && $ret);
		}

		return $ret;
	}

	public function getData()
	{
		$ret = array();
		foreach($this->fields as $field)
		{
			if(array_search(get_class($field), self::$skipFieldTypes) === false)
				$ret[$field->name] = $field->getData();
		}

		return $ret;
	}
}

?>
