<?php
/**
* Password field
* @ingroup BasicFields
* @author Scott Peshak
* @date $Date: 2006-04-11 03:05:52 -0500 (Tue, 11 Apr 2006) $
*
* The PasswordField supports setting the member encCallback to the name of a
* function to use for encypting the password.
* For example:
*
* @code
* // Use the PHP builtin md5
* $field = new PasswordField('passwd', array('encCallback' => 'md5'));
* @endcode
*
* You can also define your own functions for this use too:
*
* @code
* function weakCrypt($val, &$noQuote)
* {
* 	 $noQuote = false;
*	 return ucfirst($val);
* }
*
* $field = new PasswordField('passwd', array('encCallback' => 'weakCrypt'));
* @endcode
*
* The rules for the callback function are as follows:
* -# Must be in global context (must be callable from everywhere)
* -# Must accept two parameters
* 		- Value that was input by user.
*		- Reference to a boolean flag.  Set this to true if getSqlImpl() should not quote the value before inserting.
* -# Must return the encrypted string, ready for use.
*
* The encCallback is especially useful if you want to use some sort of built in
* encryption in your database server.  This example uses the MySQL built in
* AES_ENCRYPT function.
* 
* @code
* function strongCrypt($val, &$noQuote)
* {
*	$noQuote = true;
*	return "AES_ENCRYPT('$val', 'TopS3cr3t')";
* }
* @endcode
*/
class PasswordField extends TextField
{
	protected $encCallback; ///< Name of callback function to use to encrypt the password.

	/**
	 * Draw the field
	 */
	public function drawInput()
	{
		$attr = $this->extrasAsAttributes();
		
		if ($this->width)
			$style = "style=\"width: $this->width;\"";

		if($this->hasError)
			$css = ' class="errorinput" ';

		echo "<input type=\"password\" name=\"$this->name\" value=\"$this->value\" $css $attr $style/>";
	}

	/**
	 * Get the field data, run through the encCallback method if defined.
	 * @return field value
	 */
	public function getData()
	{
		return $this->doEncCallback($noQuote);
	}

	/**
	 * Get field data as an SQL string ready for use in a query
	 * @return SQL string
	 */
	public function getSqlImpl()
	{
		$noQuote = false;
		$val = $this->doEncCallback($noQuote);

		return ( "$this->name = " . ($noQuote ? $val : "'$val'"));
	}

	/**
	 * If an encryption callback function has been defined, call it on the data
	 * @param @noQuote Boolean flag, is set to \c true if the value should not be quoted when used in SQL string.
	 */
	protected function doEncCallback(&$noQuote)
	{
		$noQuote = false;

		if($this->encCallback)
		{
			if(!is_callable($this->encCallback))
				throw new InvalidCallbackException($this->encCallback); 
				
			return call_user_func_array($this->encCallback, array($this->value, &$noQuote)); // _array flavor used to preserve references	
		}

		return $this->value;
	}

}
?>
