<?php
/**
* @defgroup Form Form Abstraction 
* @author Scott Peshak, Zach Smith
*
* The Form system is a collection of classes to make the generation of HTML
* forms very simple.  Using the Form classes ensure that all HTML forms on a
* site behave the same.  Effort was also made to make all output XHTML Strict
* compliant. 
*/

/**
* @defgroup BasicFields Basic Fields
* @ingroup Form
* @author Scott Peshak, Zach Smith
*
* Basic fields like text fields, password, checkboxes, radio buttons, hidden,
* labels, textarea, and upload are all included here.
*/

/**
* @defgroup ListFields List Fields
* @ingroup Form
* @author Scott Peshak, Zach Smith
*
* List fields are fields that consist of a list of data.  Lists of checkboxes,
* radio fields, and select boxes are all included here.  It has state fields,
* gender fields, and more.
*/

/**
* @defgroup CollectionFields Collection Fields
* @ingroup Form
* @author Scott Peshak, Zach Smith
*
* Collection fields are groups of 2 or more other fields that are used to
* create unique fields.  Examples are CityStateZip, Ip Address, Name Field,
* Password Change, Phone Number, and SSN fields.
*/

/**
* @defgroup FormErrors Form Exceptions
* @ingroup Form
* @author Scott Peshak, Zach Smith
*
* These are all the various exceptions that can be thrown at the various parts
* in the form class
*/

//if we dont have an autoload function, add the Form one in.  used for using form as stand alone
if (!function_exists("__autoload"))
{
	function __autoload($class)
	{
		Form::autload($class);
	}
}

/**
* Form
*
* This is the main form class that handles a variety of form tasks like adding
* / removing fields, setting layout, checking submission and validation status,
* etc. 
* @ingroup Form
* @author Scott Peshak, Zach Smith
*/
class Form
{
	private $fields = array(); ///< Array of \c Field objects.
	private $method = 'post'; ///< Request method to use, either \c post or \c get
	private $layout; ///< Layout object for displaying the form.
	private $name; ///< Name of the form. Used with JavaScript.
	private $action; ///< Url to post/get to.
	private $onSubmit; ///< Url to post/get to.

	/**
	 * Constructor
	 * @param $layout Optional Layout object to use
	 */
	public function __construct($layout = null)
	{
		$this->action = $_SERVER['PHP_SELF'];
		$this->layout = $layout === null ? new TableLayout() : $layout;
		$this->name = uniqid('form');
		
		//add our submit field.
		$this->add('HiddenField', 'is_submitted', array(
			'value' => 1
		));

		if(!($this->layout instanceof Layout))
			throw new Exception("Passed layout object does not implement Layout");
	}

	/**
	 * Magic getter
	 * @param $name
	 */
	public function __get($name)
	{
		if(isset($this->$name))
			return $this->$name;
		else
			return false;
	}

	/**
	 * Magic setter
	 * @param $name Name of member
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		switch($name)
		{
			case 'method':
				$value = strtolower($value);
				if($value != 'post' && $value != 'get') 
					throw new InvalidValueException();

				$this->method = $value;
				break;

			case 'layout':
				if(!($value instanceof Layout))
					throw new InvalidValueException();

				$this->layout = $value;
				break;

			case 'action':
			case 'name':
			case 'onSubmit':
			case 'onsubmit':
				$this->$name = $value;
				return true;
				break;

			default:
				return false;
		}
	}

	/**
	* Add a field to the form
	* @param $type Field type (string).
	* @param $name Name of field.
	* @param $options Array of extra options.
	* @return Reference to the added Field.
	*/
	public function add($type, $name, array $options = array())
	{
		if(array_key_exists($name, $this->fields))
			throw new Exception("Field name $name is not unique.");

		$field = new $type($name, $this, $options);
		$this->fields[$name] = $field;
		
		return $field;
	}

	/**
	* add a ButtonField of type submit with a title to the form (shortcut function)
	*
	* @param the title of the submit button
	* @return a reference to the Field
	*/
	public function addSubmit($title = 'Submit')
	{
		$this->add('ButtonField', 'submitme', array('value' => $title, 'type' => 'submit'));

		return $field;
	}

	/**
	* remove a field form the form (useful for modifying existing form objects
	*
	* @param $name the name of the field
	*/
	public function remove($name)
	{
		if ($this->hasField($name))
			unset($this->fields[$name]);
		else
			throw new Exception("Cannot remove field $name, it doesn't exist.");
	}


	/**
	 * Check to see if the form has been submitted
	 */
	public function isSubmitted()
	{
		return isset($_POST['is_submitted']);
	}

	/**
	 * Validate all form fields
	 */
	public function validate()
	{
		foreach($this->fields as $field)
			$field->validate();
	}

	
	/**
	 * Set data
	 * @param $data Either an array of field name => value, or a db result set
	 */
	public function setData($data)
	{
		if(is_string($data))
		{
			$res =& $this->dbConn->query($data);
			
			if(PEAR::isError($res))
				throw new InvalidQueryException($data);
			
			$data = $res->fetchRow(DB_FETCHMODE_ASSOC);
		}

		if(!is_array($data))
			throw new InvalidValueException();

		foreach($this->fields as $field)
		{
			//we may need to set the data on our individual collection fields.
			if ($field instanceOf CollectionField)
				$field->setFieldsData($data);
			
			//nope, just a simple field.. but still try it on collection fields.
			if(isset($data[$field->name]))
				$field->setData($data[$field->name]);			
		}
	}
	

	/**
	 * Get the data
	 * @param $fieldName Name of single field to get data for
	 * @return Array of field name => value if $fieldName is null, value of field named $fieldName otherwise
	 */
	public function getData($fieldName = null)
	{
		if($fieldName != null)
		{
			$this->assertFieldExists($fieldName);
			return $this->fields[$fieldName]->getData();
		}

		$ret = array();
		foreach($this->fields as $field)
		{
			$ret[$field->name] = $field->getData();
		}

		return $ret;
	}

	/**
	* Check all fields for an error
	*
	* @return boolean if the form has an error or not
	*/
	public function hasError()
	{
		$error = false;	
		foreach($this->fields as $field)
			$error = $field->hasError() || $error;

		return $error;
	}

	/**
	* Figure out if we have a field or not....
	*
	* @param $field the name of a field.
	* @return boolean if it exists or not
	*/
	public function hasField($field)
	{
		return is_object($this->fields[$field]);
	}

	/**
	* Determine if we need our form drawn...
	* @return boolean if its not submitted or has error
	*/
	public function needsDrawn()
	{
		return (!$this->isSubmitted() || $this->hasError());
	}

	/**
	* a helper function that checks to see if its submitted, validates it, and
	* then tells us if we have errors. saves 4-5 lines on common repetitive
	* stuff.
	*
	* @return bool if the form is ready for post processing
	*/
	public function isSubmittedAndValid()
	{
		//if its submitted...
		if ($this->isSubmitted())
		{
			//validate it.
			$this->validate();
			return !$this->hasError();
		}

		//no submit?  no problem.
		return false;
	}

	/**
	* shortcut function to draw the form if its needed:  which is either not submitted or on error
	*/
	public function drawIfNeeded()
	{
		if ($this->needsDrawn())
			$this->draw();
	}

	/**
	* Draw the form
	* @param $action value to use for form action.
	* @param $submitText Text to use for the submit button, if \c null then no buttons will be drawn
	*
	* @depreciated we will move to using only draw() in v2.2
	* @todo move all this to draw() in v2.2
	*/
	public function drawAll($action = null, $submitText = null)
	{
		//error handling first
		if(!count($this->fields))
			throw new Exception('Form has no fields.');

		//where we sending to?
		$action = $action === null ? $this->action : $action;

		
		//do we add submit buttons?
		if($submitText !== null)
			$this->addSubmit($submitText);

		if ($this->onSubmit)
			$submit = "onSubmit=\"$this->onSubmit\"";

		//start our form
		echo "<form method='$this->method' action='$action' enctype='multipart/form-data' name='$this->name' id='$this->name' $submit>";

		//draw our layout
		$this->layout->drawBeginning();

		//draw our fields
		foreach($this->fields as $field)
			$this->layout->drawField($field);

		//end our layout
		$this->layout->drawEnd();

		echo "</form>";
	} 

	/**
	* draw the entire form
	*/
	public function draw()
	{
		$this->drawAll();
	}

	/**
	 * Mark a field as having an error.
	 * @param $fieldName name of field to mark
	 * @param $errorString error message to set
	 */
	public function setError($fieldName, $errorString = null)
	{
		$this->assertFieldExists($fieldName);
		$field = $this->fields[$fieldName];
		$field->forceError();
		
		if($errorString)
			$field->errorMessage = $errorString;
	}

	/**
	 * Get any special CSS styles that fields may define
	 * @return string of CSS
	 */
	public function getCss()
	{
		$css = null;

		foreach($this->fields as $field)
			$css .= $field->getCss();

		return $css;
	}

	/**
	 * Assert that a given field name actually exists in the form
	 * @param $fieldName name to check
	 */
	public function assertFieldExists($fieldName)
	{
		if(!array_key_exists($fieldName, $this->fields))
			throw new InvalidFieldNameException();

		return true;
	}

	/**
	 * Magic function to load include files as needed
	 * @param $class Class name for which include is needed
	 */
	public static function autoload($class)
	{
		if(preg_match('/.*Field$/', $class))
		{
			$class = str_replace('field', '-field', strtolower($class));
			require_once("fields/$class.inc.php");
			return true;
		}
		elseif(preg_match('/.*Layout$/', $class))
		{
			$class = str_replace('layout', '-layout', strtolower($class));
			require_once("layouts/$class.inc.php");
			return true;
		}
		elseif(preg_match('/.*Exception$/', $class))
		{
			$class = str_replace('exception', '-exception', strtolower($class));
			require_once("exceptions/$class.inc.php");
			return true;
		}

		return false;
	}
}
?>
