<?php
/**
 * @ingroup BJHelpers
 * @brief Image Manipulation
 *
 * This module is used to manipulate an image.  It uses ImageMagick convert so
 * it can accept a wide range of image formats.
 *
 */

class Img
{
	/**
	 * Rotates the given file @c $degrees clockwise.
	 *
	 * @param $inputFile the full path of image to rotate (ie. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of rotated image (ie. @c "/home/user/gallery/...")
	 * @param $degrees the number of degrees to rotate the file
	 */
	public static function rotate($inputFile, $outputFile, $degrees)
	{
		//error checking.
		if (!is_numeric($degrees))
			throw new Exception("Img::rotate - invalid degrees: $degrees");

		//setup our call.
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		$call = "/usr/bin/convert -rotate $degrees $inputFile $outputFile";
		
		//do it!
		Util::shellExec($call);
	}

	/**
	 * Flips the given file horizontally or vertically.
	 *
	 * @param $inputFile the full path of image to rotate (ie. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of rotated image (ie. @c "/home/user/gallery/...")
	 * @param $type the direction that you wish to flip the image; must be @c "horizontal" or @c "vertical"
	 */
	public static function flip($inputFile, $outputFile, $type)
	{
		if ($type == "horizontal")
			$arg = "flop";
		else if ($type == "vertical")
			$arg = "flip";
		else
			throw new Exception("imageFlip: invalid type: $type");

		//setup our call.
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		$call = "/usr/bin/convert -$arg $inputFile $outputFile";

		//do it!
		Util::shellExec($call);
	}

	/**
	 * Resizes an image so that no dimension is larger than @c $maxDimension 
	 * If the images largest size is smaller than $maxDimension nothing is done
	 *
	 * @param $inputFile the full path of image to rotate (ie. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of rotated image (ie. @c "/home/user/gallery/...")
	 * @param $maxDimension the largest dimension of any side in pixels that you would like to resize to
	 */
	public static function resize($inputFile, $outputFile, $maxDimension, $execute = true)
	{
		if (preg_match("/\d*:\d*/", $maxDimension))
		{
			list ($maxX, $maxY) = explode(':', $maxDimension);
		}
		else
		{
			$maxX = $maxY = $maxDimension;
		}

		if (!is_numeric($maxX) || !is_numeric($maxY))
			throw new Exception("imageResize: invalid maximum dimension: $maxDimension");

		//setup our call.
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		$call = "/usr/bin/convert -size ${maxX}x${maxY} $inputFile -resize '${maxX}x${maxY}>' +profile \"*\" $outputFile";

		//do it!
		if ($execute)
			Util::shellExec($call);
		else
			echo "$call;\n";
	}

	/**
	 * Crops an image.
	 *
	 * @param $inputFile the full path of image to crop (ie. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of cropped image (ie. @c "/home/user/gallery/...")
	 * @param $dimensions the new dimensions for the cropped image
	 */
	public static function crop($inputFile, $outputFile, $dimensions)
	{
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		$call = "/usr/bin/convert -crop $dimensions $inputFile +profile \"*\" $outputFile";
		
		Util::shellExec($call);
	}

	/**
	 * Front end for ImageMagick convert.  Converts @c $inputFile and places the
	 * result in @c $outputFile.  @c $flags should be properly escapped command
	 * line arguments (see escapeshellarg()).
	 *
	 * @param $inputFile the full path of image to convert (eg. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of converted image (eg. @c "/home/user/gallery/...")
	 * @param $flags extra flags to pass on to convert; no checking is done to validate the flags that are passed.
	 */
	public static function convert($inputFile, $outputFile, $flags = "")
	{
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		$call = "/usr/bin/convert $flags $inputFile $outputFile";
		
		Util::shellExec($call);
	}

	/**
	 * Front end for ImageMagick convert.  Converts @c $inputFile and places the
	 * result in @c $outputFile.  @c $flags should be properly escapped command
	 * line arguments (see escapeshellarg()).
	 *
	 * @param $inputFile the full path of image to convert (eg. @c "/home/user/gallery/...")
	 * @param $outputFile the full path of converted image (eg. @c "/home/user/gallery/...")
	 * @param $width the width that the photo should be specified to
	 * @param $height the height that the photo should be specified to
	 * @param $force when set to true will force the size given, ie: not scale the image
	 * @return returns true on success, false otherwise
	 */
	public static function scale($inputFile, $outputFile, $width = null, $height = null, $force = false)
	{
		$inputFile = escapeshellarg($inputFile);
		$outputFile = escapeshellarg($outputFile);
		if ($force)
			$force = "!";
		else
			$force = "";

		if ($width && !$height)
			$call = "/usr/bin/convert -geometry ${width}$force $inputFile $outputFile";
		else if ($height && !$width)
			$call = "/usr/bin/convert -geometry x${height}$force $inputFile $outputFile";
		else if ($height && $width)
			$call = "/usr/bin/convert -geometry ${width}x${height}$force $inputFile $outputFile";
		else
			return false;
		Util::shellExec($call);

		return true;
	}
}
?>
