<?
/**
* @ingroup BaseModules
* @brief Basic Administration area.
*
* Provides cache management, hit tracking, etc.  Automatically protected by default.
*/
class BaseAdmin extends MyModule
{
	/**
	* @see BaseModule::init
	*
	* init is extended to force users to a) be logged in, and b) that they are admins to view any page in the module
	*/
	public function init($page = null, $vars = null)
	{
		//parental init.
		parent::init($page, $vars);

		//gotta be logged in.
		$this->assertLogin();

		//seriously, you cant come in.
		global $me;
		if (!$me->isAdmin())
			throw new PageError('You must be an admin to access the Admin area. ;)');
	}

	/**
	* adds in default admin pages.
	*
	* pages added are:
	*
	* clearcache - a page that clears the entire cache
	* deletecache - deletes a single key from the cache
	* viewcache - view a single key's data from the cache
	* hittracking - track hits across the site.
	* clearhits - clear the hits from the tracking table
	*/
	public function getPagesXml()
	{ 
		$xml = parent::getPagesXml();

		$xml .= <<<XML
			<page name="clearcache"/>
			<page name="deletecache">
				<param name="key" required="1"/>
			</page>
			<page name="viewcache">
				<param name="key" required="1"/>
			</page>
			<page name="hittracking" title="Hit Tracking">
			</page>
			<page name="clearhits"/>
XML;
		return $xml;
	}

	/**
	* inits the main page, sets the title
	*/
	public function initMainPage()
	{
		$this->pageTitle = 'Admin Homepage';
	}

	/**
	* draws the main page, with links for cache, and hit trackingg
	*/
	public function drawMainPage()
	{
?>
	<h3>Cache Management</h3>
	<ul>
		<li><?=$this->getLink(".deletekey", "Delete Key")?></li>
		<li><?=$this->getLink(".clearcache", "Clear Cache")?></li>
	</ul>

	<h3>Hit Tracking</h3>
	<ul>
		<li><?=$this->getLink(".hittracking", "View Stats")?></li>
		<li><?=$this->getLink(".clearhits", "Clear Hits")?></li>
	</ul>
<?
	} 

	/**
	* sets title
	*/
	public function initClearCachePage()
	{
		$this->pageTitle = 'Clear the Cache';
	}

	/**
	* does cache clearing page w/ confirmation form.
	*/
	public function drawClearCachePage()
	{
		$form = $this->createClearCacheForm();

		if ($form->isSubmitted())
		{
			$form->validate();
			if (!$form->hasError())
			{
				CacheBot::flush();
				echo "All entries in the cache have been deleted.  Have a nice day!";
			}
		}

		if ($form->needsDrawn())
			$form->draw();
	}

	/**
	* creates cache clearing confirmation form
	*
	* @return a Form object
	*/
	public function createClearCacheForm()
	{
		$form = new Form();
		$form->action = $this->getUrl(".clearcache");

		$form->add("LabelField", "warning", array(
			"text" => "You are about to clear the entire cache for the whole website.  Are you sure you want to do this?"
		));
		$form->addSubmit("Clear Cache");

		return $form;
	}

	/**
	* sets the title
	*/
	public function initDeleteCachePage()
	{
		$key = base64_decode($this->params('key'));
		
		$this->pageTitle = "Delete Cache - $key";
	}

	/**
	* does the cache deletion w/ confirmation
	*/
	public function drawDeleteCachePage()
	{
		$key = base64_decode($this->params('key'));
		
		$form = $this->createDeleteCacheForm($key);

		if ($form->isSubmitted())
		{
			$form->validate();
			if (!$form->hasError())
			{
				CacheBot::delete($key);
				echo "The key '$key' has been deleted from the cache.";
			}
		}

		if ($form->needsDrawn())
			$form->draw();
	}

	/**
	* creates cache key deletion confirmation form
	*
	* @param $key the key to delete
	* @return a Form class
	*/
	public function createDeleteCacheForm($key)
	{
		$form = new Form();
		$form->action = $this->getUrl(".deletecache?key=" . base64_encode($key));

		$form->add("StaticField", "warning", array(
			"text" => "Are you sure you want to delete the cache for <b>$key</b>?"
		));
		$form->addSubmit("Delete Cache");

		return $form;
	}

	/**
	* sets the title
	*/
	public function initViewCachePage()
	{
		$key = base64_decode($this->params('key'));

		$this->pageTitle = "View Cache - $key";
	}

	/**
	* shows the cache data for the key, and shows delete link
	*/
	public function drawViewCachePage()
	{
		$key = base64_decode($this->params('key'));
		
		//get our data... use a large life so we dont accidentally delete it.
		$data = CacheBot::get($key, 1000000000);
		if ($data)
		{
			//show our delete link
			echo "<p>" . $this->getLink(".deletecache?key=" . base64_encode($key), "Delete Cache for $key") . "</p>";

			//show the data!
			Util::dump($data);
		}
		else
			echo "<p><b>Key $key not found!</b></p>";
	}

	/**
	* draws hit tracking page.  broken down into popular modules and popular modules/pages.
	*/
	public function drawHitTrackingPage()
	{
?>
	<h2>Popular Modules</h2>
	<table width="50%">
		<tr>
			<th>#</th>
			<th>Module</th>
			<th>Hits</th>
		</tr>
<?
	$rs = dbQuery("
		SELECT count(module) AS module_hits, module
		FROM site_hits
		GROUP BY module
		ORDER BY module_hits DESC
		LIMIT 50
	");
	$count = 0;
	while ($ar = dbFetchAssoc($rs))
	{
		$count++;
		echo "<tr><td>$count</td><td>$ar[module]</td><td>$ar[module_hits]</td></tr>";
	}
?>
	</table>
	
	<h2>Popular Pages</h2>
	<table width="50%">
		<tr>
			<th>#</th>
			<th>Module</th>
			<th>Page</th>
			<th>Hits</th>
		</tr>
<?
	$rs = dbQuery("
		SELECT count(page) AS page_hits, module, page
		FROM site_hits
		GROUP BY module, page
		ORDER BY page_hits DESC
		LIMIT 50
	");
	$count = 0;
	while ($ar = dbFetchAssoc($rs))
	{
		$count++;
		echo "<tr><td>$count</td><td>$ar[module]</td><td>$ar[page]</td><td>$ar[page_hits]</td></tr>";
	}
?>
	</table>
<?
	}
	
	/**
	* sets title
	*/
	public function initClearHitsPage()
	{
		$this->pageTitle = 'Clear the Hit Tracking Table';
	}

	/**
	* does cache clearing page w/ confirmation form.
	*/
	public function drawClearHitsPage()
	{
		$form = $this->createClearHitsForm();

		if ($form->isSubmitted())
		{
			$form->validate();
			if (!$form->hasError())
			{
				$site = Site::getSite();
				$site->clearHits();
				
				echo "All entries in the hit tracking table have been deleted.  Have a nice day!";
			}
		}

		if ($form->needsDrawn())
			$form->draw();
	}

	/**
	* creates cache clearing confirmation form
	*
	* @return a Form object
	*/
	public function createClearHitsForm()
	{
		$form = new Form();
		$form->action = $this->getUrl(".clearhits");

		$form->add("LabelField", "warning", array(
			"text" => "You are about to clear the entire hit tracking data for the whole website.  Are you sure you want to do this?"
		));
		$form->addSubmit("Clear Hits");

		return $form;
	}

}
?>
