<?
/**
* @ingroup BaseObjects
* @brief A basic event with dates, guests, etc.
*
* A basic event object for use with the calendar.  It has start/end dates,
* location enabled, etc.
*
* @todo add code for creation of guests table
*/
class BaseEvent extends MyObject
{
	public $guests = null;
	public $guestTable = null;
	
	/**
	* see BaseObject::__construct
	*
	* @param $guestTable the table to store guests data in
	*/
	public function __construct($data = null, $table = 'events', $guestTable = 'event_guests')
	{
		$this->useLatLng = true;
		$this->commentsEnabled = true;

		$this->guestTable = $guestTable;
		
		parent::__construct($data, $table);
		
		$this->fullTextFields[] = 'name';
		$this->fullTextFields[] = 'description';
		$this->fullTextFields[] = 'location';
	}
	
	/**
	* add in event pages: rsvp, attend, leave
	*
	* @todo remove invite / uninvite pages.  we want to move that to an EventInvitation
	*/
	public function getPagesXml()
	{ 
		$xml = parent::getPagesXml();
		$xml .= <<<XML
			<page name="invite">
				<param name="id" required="1" type="int"/>
			</page>
			<page name="uninvite">
				<param name="id" required="1" type="int"/>
				<param name="guest_id" required="1" type="int"/>
			</page>
			<page name="rsvp">
				<param name="id" required="1" type="int"/>
			</page>
			<page name="attend">
				<param name="id" required="1" type="int"/>
			</page>
			<page name="leave">
				<param name="id" required="1" type="int"/>
			</page>
XML;
		return $xml;
	}

	/**
	* extend edit page to allow date to be passed in
	*/
	public function getEditPageParamXml()
	{
		$xml = '<param name="date" regex="/^\d{4}-\d{1,2}-\d{1,2}$/"/>';

		return $xml;
	}
	
	/**
	* delete the event, also delete guests
	*/
	public function delete()
	{
		dbExecute("
			DELETE FROM {$this->guestTable}
			WHERE event_id = '$this->id'
		");
		
		parent::delete();
	}

	/**
	* prep our attend page
	*
	* assert login, make sure they can attend, and set the title
	*/
	public function initAttendPage()
	{
		$this->assertLogin();

		$this->id = $this->params('id');

		if (!$this->canAttend())
			throw new PageError('You do not have access to attend this event.');

		$this->pageTitle = "I'm there!";
	}

	/**
	* draw our attend page... with guest form too
	*/
	public function drawAttendPage()
	{
		global $me;
		
		$form = $this->createAttendForm();
		if ($form->isSubmitted())
		{
			$form->validate();

			if (!$form->hasError())
			{
				$this->addGuest($me->id);
				$this->rsvp($me->id, 'yes', $form->getData('guests'));
				Util::redirect($this->getUrl(".view?id=$this->id"));
			}
		}
		else
			$form->setData(array('guests' => 0));

		if ($form->needsDrawn())
			$form->drawAll();
	}
	
	/**
	* create attend form w/ guest count
	*
	* @return a Form object
	*/
	public function createAttendForm()
	{
		$form = new Form();
		$form->action = $this->getUrl(".attend?id=$this->id");

		$form->add('TextField', 'guests', array(
			'title' => 'How many guests are you bringing?',
			'required' => true,
			'size' => 10
		));
		$form->addSubmit("I'm There");

		return $form;
	}

	/**
	* prep our leave page, for removing yourself from an event.
	*
	* assert login, check if they can leave, etc.
	*/
	public function initLeavePage()
	{
		$this->assertLogin();
		
		$this->id = $this->params('id');

		if (!$this->canLeave())
			throw new PageError('You do not have access to leave this event.');

		$this->pageTitle = 'Remove Me From Guests';
	}
	
	/**
	* draw the leave page w/ confirmation form
	*/
	public function drawLeavePage()
	{
		global $me;
		
		$form = $this->createLeaveForm();
		if ($form->isSubmitted())
		{
			$this->removeGuest($me->id);
			Util::redirect($this->getUrl(".view?id=$this->id"));
		}
		else
			$form->drawAll();
	}

	/**
	* create the leave confirmation form.
	*
	* @return the Form object
	*/
	public function createLeaveForm()
	{
		$form = new Form();
		$form->action = $this->getUrl(".leave?id=$this->id");

		$form->add('StaticField', 'info', array(
			'text' => 'Are you sure you want to remove yourself from the list of guests?'
		));
		$form->addSubmit("Remove me from list");

		return $form;
	}

	/**
	* init the RSVP page
	*
	* assert login, error checking, title, etc.
	*/
	public function initRSVPPage()
	{
		global $me;
	
		$this->assertLogin();

		$this->id = $this->params('id');

		if (!$this->isGuest($me->id))
			throw new PageError('You must be a guest to RSVP.');

		$this->pageTitle = 'RSVP to ' . $this->getName();
	}

	/**
	* draw the RSVP page w/ form, etc
	*/
	public function drawRSVPPage()
	{
		global $me;
		
		$form = $this->createRSVPForm();

		if ($form->isSubmitted())
		{
			$form->validate();

			if (!$form->hasError())
			{
				$this->rsvp($me->id, $form->getData('commitment'), $form->getData('guests'));

				Util::redirect($this->getUrl(".view?id=$this->id"));
			}
		}
		else
			$form->setData($this->getGuest($me->id));

		if ($form->needsDrawn())
			$form->drawAll();
	}

	/**
	* create the RSVP form w/ commitment and guests
	*
	* @return the Form object
	*/
	public function createRSVPForm()
	{
		$form = new Form();
		$form->action = $this->getUrl(".rsvp?id=$this->id");

		$form->add('SelectField', 'commitment', array(
			'title' => 'Are you attending this event?',
			'options' => array(
				'yes' => 'Yes',
				'no' => 'No',
				'maybe' => 'Maybe'
			),
			'required' => true
		));
		$form->add('TextField', 'guests', array(
			'title' => 'How many guests are you bringing?',
			'size' => 10
		));
		$form->addSubmit("RSVP");

		return $form;
	}
	
	/**
	* @todo remove this page
	*/
	public function initUnInvitePage()
	{
		$this->assertLogin();

		$this->id = $this->params('id');

		$this->pageTitle = 'Uninvite Guests';

		if (!$this->canUnInvite())
			throw new PageError('You do not have access to uninvite people to this event.');
	}

	/**
	* @todo remove this page
	*/
	public function drawUnInvitePage()
	{
		//this is who we're uninviting
		$user = new User($this->params('guest_id'));
		
		//get our form
		$form = $this->createUnInviteForm($user);

		//did we submit?
		if ($form->isSubmitted())
		{
			$form->validate();

			//no problems... do it
			if (!$form->hasError())
			{
				$this->removeGuest($user->id);
				Util::redirect($this->getUrl(".view?id=$this->id"));
			}
		}

		//draw it.
		if ($form->needsDrawn())
			$form->drawAll();
	}

	/**
	* @todo remove this page
	*/
	public function createUnInviteForm($user)
	{
		$form = new Form();
		$form->action = $this->getUrl(".uninvite?id=$this->id&guest_id=$user->id");

		$form->add("StaticField", "guest", array(
			'text' => 'Are you sure you want to uninvite ' . $user->getName(true) . '?'
		));
		$form->addSubmit('Uninvite Guest');

		return $form;
	}

	/**
	* @todo remove this page
	*/
	public function canInvite()
	{
		return $this->canEdit();
	}

	/**
	* @todo remove this page
	*/
	public function canUnInvite()
	{
		return $this->canDelete();
	}

	/**
	* can $me attend an event?
	*
	* checks to see if they're not a guest, or if its invite only, etc.
	*
	* @return boolean result.
	*/
	public function canAttend()
	{
		global $me;

		return ((bool)$this->allow_attendees && !$this->invite_only && $this->canView() && !$this->isGuest($me->id));
	}

	/**
	* can $me leave the event?
	*
	* checks to see if they are attending, etc.
	*
	* @return boolean result
	*/
	public function canLeave()
	{
		global $me;
		
		return (bool)$this->isGuest($me->id);
	}
	
	/**
	* @todo remove this page
	*/
	public function initInvitePage()
	{
		$this->assertLogin();

		$this->id = $this->params('id');

		$this->pageTitle = 'Invite Guests';

		if (!$this->canInvite())
			throw new PageError('You do not have access to invite people to this event.');
	}

	/**
	* @todo remove this page
	*/
	public function drawInvitePage()
	{
		$form = $this->createInviteForm();

		//did we get it?
		if ($form->isSubmitted())
		{
			$form->validate();

			if (!$form->hasError())
			{
				$guests = explode(',', $form->getData('guests'));
				if (count($guests))
					foreach ($guests AS $id)
						$this->inviteGuest($id);

				Util::redirect($this->getUrl(".view?id=$this->id"));
			}
		}
		//draw it.
		if ($form->needsDrawn())
			$form->drawAll();
	}
	
	/**
	* @todo remove this page
	*/
	public function createInviteForm()
	{
		global $me;
		
		$form = new Form();
		$form->action = $this->getUrl(".invite?id=$this->id");
		
		//add it in
		$form->add('MultiSelectField', 'guests', array(
			'multiple' => true,
			'size' => 10,
			'options' => $this->getPotentialGuestList(),
			'title' => 'Select the people you want to invite.<br/>
			We will email the people you select <br/>and let them know they
			have been invited.',
		));

		$form->addSubmit('Invite Guests');
	
		return $form;
	}

	/**
	* gets a list of potential guests.
	*
	* by default it gets the users friends and removes current guests
	*
	* @return an array of user ids => user names
	*/
	protected function getPotentialGuestList()
	{
		global $me;

		//create our guestlist
		$items = array();
		$friends = $me->getFriends();
		foreach ($friends AS $member)
			if (!$this->isGuest($member->id))
				$items[$member->id] = $member->getName();

		return $items;
	}

	/**
	* is the user a guest or not?
	*
	* @param $userId the id of the user to check
	* @return boolean result
	*/
	public function isGuest($userId)
	{
		$guests = $this->getGuests();

		return is_array($guests[$userId]);
	}

	/**
	* get a guests information.
	*
	* @param $userId the id of the guest to get
	* @return an array of guest information
	*/
	public function getGuest($userId)
	{
		$guests = $this->getGuests();
		return $guests[$userId];
	}
	
	/**
	* get the guests for the event.
	*
	* @param $all boolean.  do we get all guests, or just 'yes' ones
	* @param $force do we force it to look it up, or use the stored one if we have it
	* @return an array of user ids => guest information
	*/
	public function getGuests($all = true, $force = false)
	{
		if ($this->guests === null || $force)
		{
			$ret = array();
			//make sure we have an id...
			if ($this->id)
			{
				$where = '';
				if ($all)
					$where = "AND commitment = '$yes'";
					
				//query...
				$guestRs = dbQuery("
					SELECT *
					FROM {$this->guestTable}
					WHERE event_id = '$this->id'
				");
			
				while ($guestAr = dbFetchAssoc($guestRs))
					$ret[$guestAr['user_id']] = $guestAr;
			}
			$this->guests = $ret;
		}

		return $this->guests;
	}

	/**
	* get a count of all our guests
	*
	* @param $all do we count all guests, or just confirmed
	*/
	public function getGuestCount($all = true)
	{
		return count($this->getGuests($all));
	}

	/**
	* prep our edit page, add js
	*/
	public function initEditPage()
	{
		parent::initEditPage();

		if (!$this->id)
			$this->pageTitle = 'Create New Event';
		else
			$this->pageTitle = 'Update Event';

		$this->needsJs('lib/js/event.js');
	}
	
	/**
	* add fields to our event form
	*
	* @see BaseObject::editFormAddFields()
	*/
	public function editFormAddFields($form)
	{
		global $me;
		
		$this->addContentFields($form);
		$this->addLocationFields($form);
		$this->addPrivacyFields($form);
		
		if (!$this->id)
			$form->addSubmit('Add Event');
		else
			$form->addSubmit('Save Event');
	}
	
	/**
	* add our content fields to the add/edit form
	*
	* @param the Form object
	*/
	public function addContentFields($form)
	{
		$form->add('TextField', 'name', array(
			"required" => true,
			"width" => "95%"
		));

		$form->add('DateField', 'start_date', array(
			'required' => true,
			'format' => 12
		));
		
		$form->add('DateField', 'end_date', array(
			"definition" => "Leave blank for 1 day events.",
			'format' => 12
		));
		
		$form->add('EditorField', 'description', array(
			"width" => '95%',
			"height" => '300px',
		));

		return $form;
	}

	/**
	* add our location fields to the add/edit form
	*
	* @param the Form object
	*/
	public function addLocationFields($form)
	{
		$form->add('TextField', 'location', array(
			"title" => "Location<br/>",
			"definition" => "My House, Bob's Place, Library, etc.",
			"size" => 50
		));
		
		$form->add('TextField', 'street', array(
			'size' => 45,
			'maxlength' => 255,
			'title' => 'Street Address'
		));
		$form->add('CityStateZipField', 'citystzip', array(
			'title' => 'City / State / Zip'
		));

		return $form;
	}

	/**
	* add our privacy fields to the add/edit form
	*
	* @param the Form object
	*/
	public function addPrivacyFields($form)
	{
		$form->add('CheckboxField', 'allow_attendees', array(
			'value' => 1,
			'title' => 'Attendees',
			'label' => 'Allow people to add themselves to the list of attendees.'
		));
		
		$form->add('CheckboxField', 'invite_only', array(
			'title' => '',
			'label' => 'Only allow people who are on the list of guests to view the event.<br/>You can add guests once you create the event. (They still have to be family or friend.)',
			'id' => 'inviteOnly',
			'onclick' => 'doPrivacyCheck(this.checked)'
		));
	
		return $form;
	}
	
	/**
	* load our data into the form, also load start/end date
	*
	* @param the Form object
	*/
	public function editFormLoad($form)
	{
		parent::editFormLoad($form);

		if (!$this->id)
		{
			if ($this->params('date'))
				$form->setData(array(
					'start_date' => $this->params('date'),
					'end_date' => $this->params('date')
				));
			else
				$form->setData(array(
					'start_date' => date('Y-m-d H:i:s'),
					'end_date' => date('Y-m-d H:i:s')
				));
		}
	}

	/**
	* post-processing, add creator as first guest
	*
	* @param the Form object
	*/
	protected function editPagePostSuccess($form)
	{
		global $me;
		
		if (!$this->isGuest($me->id))
			$this->addGuest($me->id, 'yes');
	
		parent::editPagePostSuccess($form);
	}
	
	/**
	* add user as a guest w/ commitment
	*
	* @param $id the user id to add
	* @param $commitment the commitment the user has.  values: yes / no / maybe
	*/
	public function addGuest($id, $commitment = 'yes')
	{
		dbExecute("
			INSERT INTO {$this->guestTable}
				(user_id, event_id, commitment)
			VALUES
				('$id', '$this->id', '$commitment')
		");

		$this->guests[$id] = array();
	}

	/**
	* @todo remove this function to event invite
	*/
	public function inviteGuest($id)
	{
		//is it valid?
		if ($id)
		{
			$this->addGuest($id, 'maybe');

			$user = new User($id);
			$this->emailInvite($user);
		}
	}
	
	/**
	* remove a guest from the event.
	* 
	* @param $id the id of the user to remove
	*/
	public function removeGuest($id)
	{
		dbExecute("
			DELETE FROM {$this->guestTable}
			WHERE user_id = '$id'
				AND event_id = '$this->id'
		");

		unset($this->guests[$id]);
	}

	/**
	* do the RSVP update
	*
	* @param $id the id of the user to rsvp
	* @param $commitment the commentment of the user... value: yes/no/maybe
	* @param $guests the number of guests they are bringing
	*/
	public function rsvp($id, $commitment, $guests)
	{
		//we want an integer
		$guests = (int)$guests;

		//no's mean 0 guests.
		if ($commitment == 'no')
			$guests = 0;

		dbExecute("
			UPDATE {$this->guestTable} SET
				commitment = '$commitment',
				guests = '$guests'
			WHERE user_id = '$id'
				AND event_id = '$this->id'
		");
	}

	/**
	* @todo move this to event invite module
	*/
	protected function emailInvite($user)
	{
		global $me;
		
		if ($user->email_invites)
		{
			$subject = $me->getName() . " has invited you to an event.";
			$body = $me->getName() . " has recently invited you to an event on " . Config::get('site_name') . ".\n\n";
			$body .= "You can view the event, and RSVP with the link below:\n\n";
			$html .= $user->getTicket(strtolower(get_class($this)) . ".rsvp?id=$this->id") . "\n\n";
			$body .= "Thank you,\nThe Happy " . Config::get('site_name') . " Event Invitation Robot";
			$body .= "\n\nPS. You can turn these emails off at: " . $user->getUrl('.emailprefs', null, true);
			
			//html
			$html = $me->getName() . " has recently invited you to an event on " . Config::get('site_name') . ".<br/><br/>";
			$html .= "You can view the event, and RSVP ";
			$html .= $user->getTicket(strtolower(get_class($this)) . ".rsvp?id=$this->id", 'here.') . "<br/><br/>";
			$html .= "Thank you,<br/>The Happy " . Config::get('site_name') . " Event Invitation Robot";
			$html .= "<br/><br/>PS. You can turn these emails off " . $user->getLink(array('page' => 'emailprefs'), 'here.', null, true);
			$user->mail($subject, $body, $html);
		}
	}

	/**
	* setup our view page
	*/
	public function initViewPage()
	{
		global $me;

		//get our id..
		$this->id = $this->params('id');

		//init our page.
		parent::initViewPage();

		//title!
		$this->pageTitle = $this->getName();

		//guests only?
		if ($this->invite_only && !$this->isGuest($me->id))
			throw new PageError('Only guests are allowed to view this event.');
	}
	
	/**
	* add an info box to the page about event.
	*/
	public function addInfoBox()
	{
		global $me;

		//get our info.	
		$s = "<p><b>Date:</b> " . $me->formatDate($this->start_date) . "</p>";
		if ($this->lat || $this->lon)
		{
			$s .= "<p><b>Location:</b>";
			
			//map link?
			if ($this->lat || $this->lon)
			{
				$s .= " (<a href=\"http://maps.google.com/maps?oi=map&q=";
				$s .= urlencode("$this->street, $this->city, $this->state $this->zip") . "\">map</a>)";
			}
			
			$s .= "<br/>";
			if ($this->location)
				$s .= "$this->location<br/>";
			if ($this->street)
				$s .= "$this->street<br/>";
			if ($this->city)
				$s .= "$this->city, $this->state, $this->zip";
			$s .= "</p>";
		} 
		$this->addBox(new BaseBox('Info', $s));
	}

	/**
	* add a box of guest information to the event
	*/
	public function addGuestBox()
	{
		$s = '';
		$yesCount = 0;
		$maybeCount = 0;
		if (count($this->guests))
		{
			$s .= "<table width=\"100%\">";
			$s .= "<tr><th>Name</th><th>RSVP</th><th>Guests</th></tr>";
			foreach ($this->guests AS $key => $val)
			{
				$guest = new User($key);
				$s .= "<tr><td>";
				$s .= $guest->getName(true);
				
				if ($this->canUnInvite())
					$s .= " " . $this->getLink(".uninvite?id=$this->id&guest_id=$guest->id", "x");
				
				$s .= "</td>";

				if ($val['commitment'] == '')
					$commit = '???';
				else
					$commit = $val['commitment'];

				if ($commit == 'yes')
				{
					$yesCount += $val['guests'];
					$yesCount++;
				}
				else if ($commit == 'maybe')
				{
					$maybeCount += $val['guests'];
					$maybeCount++;
				}
				
				$s .= "<td>$commit</td>";

				$s .= "<td align=\"center\">$val[guests]</td>";
				$s .= "</tr>";
			}

			$totalCount = $yesCount + $maybeCount;
			$s .= "</table>";
			
			$s .= "<p><b>$totalCount</b> total guests.<br/><b>$yesCount</b> yes and <b>$maybeCount</b> maybe.</p>";
		}
		else
			$s .= "No guests yet.<br/>";
		
		$s .= "<p>";
		if ($this->canInvite())
			$nav[] = $this->getLink(".invite?id=$this->id", "Invite more guests.");
		if ($this->canAttend())
			$nav[] = $this->getLink(".attend?id=$this->id", "Add me to the guests.");
		if ($this->isGuest($me->id))
			$nav[] = $this->getLink(".rsvp?id=$this->id", "Update my RSVP.");
		if ($this->canLeave())
			$nav[] = $this->getLink(".leave?id=$this->id", "Remove me from the guests.");
		$s .= implode("<br/>", $nav);
		$s .= "</p>";
		$this->addBox(new BaseBox('Guests', $s));
	}

	/**
	* draw a line of info on the event
	*/
	public function drawLine()
	{
		global $me;
		
		echo $this->getLink(".view?id=$this->id", $this->name) . " on " . $me->formatDate($this->start_date) . ". ";
		echo "Added by " . $this->creator->getName(true) . ".";
	}

	/**
	* draw the location of the event
	*
	* @param $map show a map link or not?
	* @todo make this function do something real.
	*/
	public function drawLocation($map = false)
	{
		if ($map)
			echo 'map';
		else
			echo $this->location;
	}

	/**
	* get the event info RSS item
	*/
	public function getRssItem()
	{
		global $me;
		
		$rss = parent::getRssItem();

		$rss->title = $this->name . " (" . $me->formatDate($this->start_date) . ")";
		$rss->description = $this->description;

		return $rss;
	}
	
	/**
	* get data for export via API's and such
	*
	* @see BaseObject::getPublicData()
	*/
	public function getPublicData()
	{
		global $me;
		
		$data = parent::getPublicData();

		$data['name'] = $this->getName();
		$data['description'] = $this->description;
		$data['start_date'] = $me->formatDate($this->start_date);
		$data['end_date'] = $me->formatDate($this->end_date);
		$data['location'] = $this->location;
		$data['street'] = $this->street;
		$data['city'] = $this->city;
		$data['state'] = $this->state;
		$data['zip'] = $this->zip;
		$data['guests'] = $this->getGuestCount();
		
		return $data;
	}

	/**
	* get the name of our event
	*/
	public function getName($link = false)
	{
		if ($link)
			return $this->getLink(".view?id=$this->id", $this->name);
		return $this->name;
	}

	/**
	* get an array of fields in the table that stores the events
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getCreateFieldsArray()
	{
		$fields = parent::getCreateFieldsArray();

		$fields['name'] = "name varchar(255) default '' not null";
		$fields['description'] = "description text default '' not null";
		$fields['location'] = "location varchar(255) default '' not null";
		$fields['start_date'] = "start_date date default '' not null";
		$fields['end_date'] = "end_date date default '' not null";
		$fields['allow_attendees'] = "allow_attendees tinyint(2) default '' not null";
	
		return $fields;
	}

	/**
	* get an array of indexes on fields in the alert table
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getCreateIndexesArray()
	{
		$fields = parent::getCreateIndexesArray();

		$fields['start_date'] = "key(start_date)";
		$fields['end_date'] = "key(end_date)";

		return $fields;
	}
}
?>
