<?
/**
* @ingroup BaseObjects
* @brief BaseGroup is a module that simplifies groups of users.
*
* A class that handles creating, editing, deleting of groups.  Also handles
* things like adding/removing members, and the various permissions associated
* with that.  It also has pages for joining and leaving.
 */
class BaseGroup extends MyObject
{
	/**
	* the name of our joiner table in teh db.
	*/
	public $joinerTable;

	/**
	* the class name of our joined object
	*/
	public $joinedObject;

	/**
	* an array of our members
	*/
	public $members = array();

	/**
	* our constructor.  create the group!
	*
	* @param $data see BaseObject
	* @param $table see BaseObject
	* @param $joinerTable the table to join object to our group with.
	* @param $joinedObject the sring name of the object joined
	*/
	public function __construct($data = null, $table = 'groups', $joinerTable = 'users_to_groups', $joinedObject = 'User')
	{
		parent::__construct($data, $table);

		$this->joinerTable = $joinerTable;
		$this->joinedObject = $joinedObject;
		
		//add in fullltext
		$this->fullTextFields[] = 'name';
		$this->fullTextFields[] = 'description';
	}

	/**
	* add join / leave pages
	*
	* these are two pages used to let users join or leave our group.
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= <<<XML
			<page name="join">
				<param name="id" type="int" required="1"/>
			</page>
			<page name="leave">
				<param name="id" type="int" required="1"/>
			</page>
XML;
		return $xml;
	}

	/**
	* this function tells us if we're a member of the group or not.  it checks
	* the members array.
	*/
	public function isMember($id)
	{
		return ($this->members[$id] instanceOf $this->joinedObject);
	}

	/**
	* this function adds a member to the group
	*
	* @param $id the id of the member to add.
	*/
	public function addMember($id)
	{
		//handy dandy dupe remover
		$this->removeMember($id);

		//put it in the db.
		dbExecute("
			INSERT INTO $this->joinerTable
				(object_id, group_id, join_date)
			VALUES
				('$id', '$this->id', NOW())
		");

		//add it to our list.
		$this->members[$id] = new $this->joinedObject($id);
	}

	/**
	* removes a member from the group
	*
	* @param $id the id of the member to remove
	*/
	public function removeMember($id)
	{
		//remove from our db.
		dbExecute("
			DELETE FROM $this->joinerTable
			WHERE object_id = '$id'
				AND group_id = '$this->id'
		");

		//clear our list.
		unset($this->members[$id]);
	}

	/**
	* get the id's of the group members
	*
	* @return an array of ids
	*/
	public function getGroupMemberIds()
	{
		$ids = array();
		$rs = dbQuery("
			SELECT object_id
			FROM $this->joinerTable
			WHERE group_id = '$this->id'
		");
		while ($ar = dbFetchAssoc($rs))
			$ids[] = $ar['object_id'];

		return $ids;
	}

	/**
	* tells us if we can join the group. used in initJoinPage()
	*
	* @return boolean if we can join or not.
	*/
	public function canJoin()
	{
		global $me;
		
		return !$this->isMember($me->id);
	}

	/**
	* tells us if we can leave the group. used in initLeavePage()
	*
	* @return boolean if we can leave or not.
	*/
	public function canLeave()
	{
		global $me;

		return $this->isMember($me->id);
	}

	/**
	* initializes our join page.  checks to see if we can join, asserts login, etc.
	*/
	public function initJoinPage()
	{
		$this->assertLogin();

		$this->id = $this->params('id');
		$this->pageTitle = 'Join Group - ' . $this->getName();
		
		if (!$this->canJoin())
			throw new PageError('You cannot join this group.');
	}

	/**
	* adds us to the group and draws a status message.
	*/
	public function drawJoinPage()
	{
		global $me;
		
		$this->addMember($me->id);

		echo "<p>You are now a member of " . $this->getName(true) . "</p>";
	}
	
	/**
	* initializes our leave page.  checks if we can, asserts login, etc
	*/
	public function initLeavePage()
	{
		$this->assertLogin();

		$this->id = $this->params('id');
		$this->pageTitle = 'Leave Group - ' . $this->getName();

		if (!$this->canLeave())
			throw new PageError('You cannot leave this group.');
	}

	/**
	* removes us from the group, draws status, etc.
	*/
	public function drawLeavePage()
	{
		global $me;
		
		$this->removeMember($me->id);

		echo "<p>You are no longer a member of " . $this->getName(true) . "</p>";
	}

	/**
	* gets us a name to draw.
	*/
	public function getName($link = false)
	{
		return parent::getName($link, $this->name);
	}

	/**
	* add in our name / description fields to edit. see BaseObject
	*/
	public function editFormAddFields($form)
	{
		parent::editFormAddFields($form);

		$form->add('TextField', 'name', array(
			'required' => true,
			'width' => '95%'
		));
		$form->add('EditorField', 'description', array(
			'required' => true,
			'width' => '95%',
			'height' => '125px'
		));
		
		return $form;
	}

	/**
	* post processing, add creator as member
	*/
	protected function editPagePostSuccess($form)
	{
		global $me;
		
		$this->addMember($me->id);
		
		parent::editPagePostSuccess($form);
	}

	/**
	* hers is our table creation sql.  extended to also create joiner table.
	* you wont need to override unless you are adding tables
	*
	* @return the sql string ready to execute.
	*/
	public function getCreateTableSql()
	{
		$sql = parent::getCreateTableSql();

		return $sql;
	}
		
	/**
	* see BaseModule.  adding in our joiner code
	*/
	public function drawCreateTablePage()
	{
		parent::drawCreateTablePage();
		
		echo "<p>" . nl2br($this->getJoinerTableSql()). "</p>";
	}
	
	/**
	* see BaseModule.  adding in our joiner code
	*/
	public function runCreateTableSql()
	{
		dbExecute($this->getJoinerTableSql());
		
		parent::runCreateTableSql();
	}

	/**
	* here is our joiner table creation sql.  you probably wont need to override.
	*
	* @return sql ready to execute
	*/
	public function getJoinerTableSql()
	{
			
		$fields = implode(",\n", $this->getJoinerFieldsArray());
		$indexes = implode(",\n", $this->getJoinerIndexesArray());
		
		return "CREATE TABLE IF NOT EXISTS $this->joinerTable\n(\n$fields,\n$indexes\n);";
	}

	/**
	* adds in our name and description fields... see BaseObject
	*/
	public function getCreateFieldsArray()
	{
		$array = parent::getCreateFieldsArray();
		
		$array['name'] = "name varchar(255) default '' not null";
		$array['description'] = "description text default '' not null";

		return $array;
	}

	/**
	* get an array of fields for our joiner table. extend this to add fields to the table.
	*
	* @return the array of field name to create syntax
	*/
	public function getJoinerFieldsArray()
	{
		return array(
			'object_id' => 'object_id int(11) default 0 not null',
			'group_id' => 'group_id int(11) default 0 not null',
			'join_date' => 'join_date datetime not null',
		);
	}

	/**
	* get an array of keys for the joiner table.  extend this to add indexes to the table.
	*
	* @return an array of key name to create syntax
	*/
	public function getJoinerIndexesArray()
	{
		return array(
			'object_id' => 'KEY (object_id)',
			'group_id' => 'KEY (group_id)'
		);
	}

	/**
	* get our rss action for this group! see BaseObject
	*/
	public function getRssItem()
	{
		$item = parent::getRssItem();

		$item->title = $this->name;
		$item->description = $this->description;

		return $item;
	}

	/**
	* get our data for this group!  see BaseObject
	*/
	public function getPublicData()
	{
		$data = parent::getPublicData();

		$data['name'] = $this->name;
		$data['description'] = $this->description;

		return $data;
	}
}
?>
