<?
/**
* @ingroup BaseObjects
* @brief BaseImage is a class for dealing with images.
*
* A class that simplifies loading, editing, deleting, resizing, and saving of
* an image linked to a mysql database.
*
* @todo de-couple this so closely from albums, or 'content_id' stuff.  perhaps
* add a new class, ContentImage that extends off this one.
*/
abstract class BaseImage extends MyObject
{
	/**
	* @b protected $dataDir is the system path where to store resized images.
	*/
	protected static $dataDir = '';
	
	/**
	* @b protected $origDir is the system path where to store original images.
	*/
	protected static $origDir = '';

	/**
	* @b protected $webDir is the path via the web where the datadir is located.
	*/
	protected static $webDir = '';

	/**
	* @b protected $sizeAr is an array of ImageSize objects that it uses for various things including resizing.
	*/
	protected $sizeAr = array();
	
	/**
	* @b protected $fullPath is the system path where the image is stored... auto generated.
	*/
	protected $fullPath = '';
	
	/**
	* @b protected $origPath is the system path where the original image is stored... auto generated.
	*/
	protected $origPath = '';

	/**
	* @b protected $doResize tells us whether we want to resize or print out the commands.
	*/
	public $doResize = true;
	
	/**
	* Our constructor.
	*
	* @param $data See BaseObject
	* @param $table See BaseObject
	* @param $dataDir the directory where to store images. no trailing slash.
	* @param $webDir the webpath to the image directory. use trailing slash.
	* @param $origDir the directory where we store original images.  no trailing slash.
	*/
	function __construct($data = null, $table = 'images', $dataDir = null, $webDir = null, $origDir = null)
	{
		if ($dataDir === null)
			$dataDir = Config::get('img_data_dir');
		if ($webDir === null)
			$webDir = Config::get('img_dir');
		if ($origDir === null)
			$origDir = Config::get('img_orig_dir');
			
		$this->objectEnglish = 'image';
		
		//we need these directories.
		self::$dataDir = realpath($dataDir);
		self::$origDir = realpath($origDir);
		self::$webDir = $webDir;

		//do some automagic on defaults..
		if (!Config::get('default_image_path'))
		{
			Config::set("default_image_path", "lib/img/base-image");
			Config::set("default_image_file", "missing-image.jpg");
			Config::set("default_image_width", "228");
			Config::set("default_image_height", "338");
		}
		
		//setup our default sizes...
		$this->sizeAr['tiny'] = new ImageSize("Tiny Icon", "tiny", 16, 16, true, 'square');
		$this->sizeAr['icon'] = new ImageSize("Medium Icon", "icon", 45, 45, true, 'square');
		$this->sizeAr['lgicon'] = new ImageSize("Large Icon", "lgicon", 95, 95, true, 'square');
		$this->sizeAr['thumb'] = new ImageSize("Thumbnail", "thumb", 120, 120, true, 'square');
		$this->sizeAr['profile'] = new ImageSize("Profile", "profile", 250, 250, true);
		$this->sizeAr['web'] = new ImageSize("Web Image", "web", 540, 1200, true);
		$this->sizeAr['large'] = new ImageSize("L", "800x600", 800, 1600); 
		$this->sizeAr['hires'] = new ImageSize("XL", "1600x1200", 1600, 1200); 
		
		//call our parent constructor.
		parent::__construct($data, $table);

		$this->fullTextFields[] = 'caption';
	}

	/**
	* load data, set paths too
	*
	* @see BaseObject::load()
	*/
	public function load($data, $deep = true)
	{
		parent::load($data, $deep);

		if (!$this->id)
		{
			$this->width = Config::get('default_image_width');
			$this->height = Config::get('default_image_height');
		}
		
		//set our paths too
		$this->setPaths();
	}

	/**
	* adds pages to the module, included are bulkconfig, bulkform, bulkpost and resize.
	*
	* @return See BaseObject
	*/
	function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= <<<XML
			<page name="bulkconfig" desc="config for bulk uploads">
				<param name="content_id" required="1"/>
				<param name="auth" required="1"/>
			</page>
			<page name="bulkform" desc="form for bulk uploads">
				<param name="content_id"/>
			</page>
			<page name="bulkpost" desc="post handler for bulk uploads">
				<param name="content_id" required="1"/>
				<param name="auth" required="1"/>
			</page>
			<page name="resize" desc="resize images!">
				<param name="id" required="1"/>
			</page>
XML;
		return $xml;
	}

	/**
	* This initializes the module... and sets the paths.
	*
	* @see BaseObject::init()
	*/
	public function init($page = 'main', $vars = array())
	{
		parent::init($page, $vars);

		if ($this->id)
			$this->setPaths();
	}

	/**
	* This deletes our files and passes it off to our parent for database.
	*/
	function delete()
	{
		$this->deleteFiles();
		
		parent::delete();
	}

	/**
	* this deletes all our files and directory structures
	*/
	function deleteFiles()
	{
		//delete all our sizes
		foreach ($this->sizeAr AS $size)
		{
			if (file_exists($size->fullPath))
				unlink($size->fullPath);
		}

		//delete our originals... suppress errors
		@unlink("$this->origPath/$this->file");

		//now remove our dir... suppress errors
		@rmdir($this->origPath);
		@rmdir($this->fullPath);
	}

	/**
	* this function sets our paths in our ImageSize objects
	*/
	function setPaths()
	{
		//get our full path to the image dir...
		if ($this->id)
			$this->fullPath = self::$dataDir . "/$this->id";
		//otherwise... show a default image.
		else
			$this->file = DEFAULT_IMAGE_FILE;
		
		if ($this->id)
			$this->origPath = self::$origDir . "/$this->id";
		
		//setup our basic file paths...
		if ($this->file)
		{
			//break it up.
			$fileAr = explode(".", $this->file);

			//get our extension
			$ext = $fileAr[count($fileAr)-1];
			
			//remove it.
			unset($fileAr[count($fileAr)-1]);

			//now get the base file name...
			$base = implode(".", $fileAr);

			//make all our size file names
			foreach ($this->sizeAr AS $size)
			{
				$size->fullPath = $this->fullPath .
					'/'. $base . "-" . $size->extension . "." . $ext;
				
				if ($this->id)
					$size->webPath = self::$webDir . $this->id . "/" .  
						$base . "-" . $size->extension . "." . $ext;
				else
					$size->webPath = DEFAULT_IMAGE_PATH . "/" .  
						$base . "-" . $size->extension . "." . $ext;
			}
		}
	}

	/**
	* this will search for an ImageSize that has path info and such
	*
	* @param $search the string name of the size.
	*/
	function getSize($search)
	{
		foreach ($this->sizeAr AS $size)
		{
			if ($size->extension == $search)
				return $size;
		}
		return false;
	}

	/**
	* this adds fields to the edit form.
	*/
	function editFormAddFields($form)
	{
		//the file field shouldnt be required for images we already have.
		if ($this->id)
			$form->add("UploadField", "file", array(
				"dir" => self::$origDir
			));
		else
			$form->add("UploadField", "file", array(
				"required" => true,
				"dir" => self::$origDir
			));
	}

	/**
	* this handles the sync part... if we didnt get a file, remove the upload
	* field to prevent erasing.
	*/
	function editPagePostSaveSync($form)
	{
		//we want this script to run no matter what...
		set_time_limit(0);
		ignore_user_abort(true);
		
		//what do we do with the image on edits?
		if ($this->id)
		{
			if (!$form->getData('file'))
				$form->remove('file');
			else
				$this->deleteFiles();
		}

		parent::editPagePostSaveSync($form);
	}

	/**
	* this handles resizing the images and such.
	*
	* @param $url the url to redirect to afterwards.
	*/
	function editPagePostSuccess($form, $url)
	{
		//only do the resize if we got a file...
		if ($form->hasField('file'))
		{
			$this->prepFile();
			$this->resizeUploadedFile();
		}

		//save our album.
		$album = new Album($this->content_id);
		$album->save();

		//get a url if we dont have one...
		if ($url == null)
			$url = $this->getUrl();
		
		Util::redirect($url);
	}  

	/**
	* this prepares a file for resizing and such.  cleans the name, creates
	* directories.
	*/
	function prepFile()
	{
		//make sure our paths are set correctly
		$this->setPaths();
		
		//make the folder if it doesnt exist.
		//make our original folder.
		if (!is_dir($this->origPath))
			mkdir($this->origPath);
		
		//first change that name...
		$newFile = $this->cleanName($this->file);
		
		//move it...
		rename(self::$origDir . "/$this->file", "$this->origPath/$newFile");
		
		//and now save the change
		$this->file = $newFile;
		$this->save();
		
		//make sure our paths are set correctly
		$this->setPaths();
	}

	/**
	* this resizes the uploaded file.  to make it faster for the end user, it
	* actually spawns an async http request.  woot!
	*/
	function resizeUploadedFile()
	{
		//this can take a while... background it.
		Util::spawnHttpAsync($this->getUrl(".resize?id=$this->id"));
	}

	/**
	* get the commands to do the resizing.
	*
	* this is intended for use when you are upgrading and need to do bulk
	* resizing via shell scripts, etc.
	*
	* @return the shell commands to run
	*/
	public function getResizeCommands()
	{
		$this->doResize = false;

		$this->resize();
	}
	
	/**
	* this does the real resizing of the image to each ImageSize
	*/
	function resize()
	{
		//correct paths = good
		$this->setPaths();
		
		//make the dir
		if (!is_dir($this->fullPath))
			mkdir($this->fullPath);
		
		//first off get its info...
		$info = getimagesize("$this->origPath/$this->file");
		$this->width = $info[0];
		$this->height = $info[1];
		
		//we want to save those images..
		$this->save();

		//now find our biggest size...
		foreach ($this->sizeAr AS $size)
		{
			if ($this->width >= $size->maxWidth || $this->height >= $size->maxHeight || $size->required)
				$this->resizeToSize($size);
		}

		return true;
	}

	/**
	* this resizes an image to a particular size
	*/
	function resizeToSize($size)
	{
		if ($this->doResize)
			copy("$this->origPath/$this->file", $size->fullPath);
		else
			echo "cp " . escapeshellarg("$this->origPath/$this->file") . " " . escapeshellarg($size->fullPath) . ";\n";
		
		//center crop?
		if ($size->scaleType == 'square')
			$this->centerCrop($size);
		
		//resize
		Img::resize($size->fullPath, $size->fullPath, "$size->maxWidth:$size->maxHeight", $this->doResize);
	}

	/**
	* this crops an image to a square, centered on the image.
	*
	* @param $size an ImageSize with the appropriate sizes set.
	*/
	function centerCrop($size)
	{
		
		//big width...  that also meets the crop requirement.:
		if ($this->width > $this->height && $this->width >= $size->maxWidth)
			$crop = $this->height;
		else if($this->height > $size->maxWidth)
			$crop = $this->width;
		
		if ($crop < $size->maxWidth)
			$crop = $size->maxWidth;
	
		$cmd = "/usr/bin/convert -gravity Center -crop {$crop}x{$crop}+0+0 " .
			escapeshellcmd($size->fullPath) . " " . escapeshellcmd($size->fullPath);
		if ($this->doResize)
			Util::shellExec($cmd);
		else
			echo "$cmd;\n";
					
	}

	/**
	* this cleans a name passed into it so its safe. underscores to dashes,
	* spaces to dashes, any non alphanumeric, dash, or dot gets removed.
	*
	* @param $name the name to clean
	*
	* @return the name, cleaned
	*/
	function cleanName($name)
	{
		//make spaces into dashes.
		$name = str_replace("_", "-", $name);
		$name = str_replace(" ", "-", $name);

		//take out anything non conforming.
		$name = preg_replace('/[^a-zA-z0-9.-]/', '', $name);

		return $name;
	}

	/**
	* draws our view page!
	*/
	function drawViewPage()
	{
		//draw our web image.
		$this->drawImage("web", false);

		//do we show them the admin links
		if ($this->canEdit() || $this->canDelete())
		{
			echo "<p>";
			$this->drawViewNav();
			echo "</p>";
		}

		$this->drawSizes();
	}

	/**
	* draws navigation for our view page
	*/
	function drawViewNav()
	{
		if ($this->canEdit())
			$this->drawEditLink("Edit this $this->english");
		echo " ~ ";
		if ($this->canDelete())
			$this->drawDeleteLink("Delete this $this->english");
	}

	/**
	* draws a list of sizes bigger than 'web'
	*/
	function drawSizes()
	{
		//get our other sizes...
		$web = $this->getSize("web");
		
		$flag = 0;
		$hanes = 'L';
		foreach ($this->sizeAr AS $size)
		{
			if ($size->maxWidth > $web->maxWidth)
			{
				if ($this->width >= $size->maxWidth || $this->height >= $size->maxHeight)
				{
					if (!$flag)
					{
						echo "\n<p>Download: ";
						$flag = 1;
					}
					
					echo "<a href=\"$size->webPath\">$hanes</a> ";
				}
			
				$hanes = 'X' . $hanes;
			}
		}
		echo "</p>\n";
	}

	/**
	* gets the url to an image size.
	*
	* @param $type the name of an ImageSize you want
	*
	* @return the web url to the image.
	*/
	function url($type = "thumb")
	{
		if ($size = $this->getSize($type))
			return $size->webPath;
		else
			return false;
	}

	/**
	* gets the html markup for the image.
	*
	* @param $type the name of the ImageSize you want.
	* @param $link boolean if you want it to be linked to the view page.
	* @param $drawCaption boolean if you want the caption to be drawn.
	*
	* @return teh markup of the image.
	*/
	function getImage($type = "thumb", $link = true, $drawCaption = true)
	{
		$rval = '';
		//get our size...
		if ($size = $this->getSize($type))
		{
			//create our view link
			$viewLink = $this->getUrl(array("page" => "view", "id" => $this->id));
		
			//do we want to link it?
			if ($link)
				$rval .= "<a href=\"$viewLink\">";
		
			//get our scaling...
			$sizeAr = $size->getScaled($this->width, $this->height);

			//draw image
			$rval .= "<img class=\"img_$type\" src=\"$size->webPath\" width=\"$sizeAr[x]\" height=\"$sizeAr[y]\" border=\"0\" alt=\"$alt\" />";
		
			if ($link)
				$rval .= "</a>";
			
			//did we get a caption?
			if ($this->caption && $drawCaption)
				$rval .= "<p>$this->caption</p>";
		}
		
		return $rval;
	}
	
	/**
	* draws teh markup for the image. see getImage() for parameter descriptions.
	*/
	function drawImage($type = "thumb", $link = true, $drawCaption = true)
	{
		echo $this->getImage($type, $link, $drawCaption);
	}

	/**
	* Draws teh thumbnail of the image in a div with id 'imageCell'.
	*/
	function drawRow()
	{
		$size = $this->getSize("thumb");
		echo "<div id='imageCell' style=\"height: {$size->maxHeight}px;\">\n";
		$this->drawImage("thumb", true, false);
		echo "</div>\n";
	}

	/**
	* inializes the bulk config page...
	*/
	function initBulkConfigPage()
	{
		global $me;

		//authenticate it!
		$_COOKIE['auth'] = $this->params('auth');
		$me->authenticate();
		
		//gotta be logged in.
		$this->assertLogin();

		$this->setTemplate(new BlankTemplate());
	}

	/**
	* draws teh bulk configuration page.  see the url for options.  needs to
	* pass teh session and stuff since the java module doesnt have your cookies
	* and shit.
	*
	* @todo update to remove content_id in an extensible fashion
	*/
	function drawBulkConfigPage()
	{
		# Configuration file for Thin File Upload
		#
		# Online documentation is available at http://upload.thinfile.com/docs/
		# Lines begining with the '#' symbol denote comments. They will not
		# be processed.
		#


		#
		# The url is the upload destination. It should point to the url on your
		# server that will accept the uploaded file
		# examples:
		#     url=http://upload.thinfile.com/demo/upload.php
		#     url=http://upload.thinfile.com/cgi-bin/upload.cgi
		#

		$contentId = $this->params('content_id');
		echo "url=" . $this->getUrl(".bulkpost?content_id=$contentId&auth=" . $this->params('auth'), null, true) . "\n";

		#
		# To change the welcome message displayed when the applet starts up,
		# change the message property. It should be a valid url and should point
		# to a web page.
		#
		# examples:
		#   message=http://upload.thinfile.com/demo/init.php
		#

		echo "allow_types=jpg,gif,png\n";
		echo "filter_action=reject\n";
		echo "reject_message=Only images are allowed to be uploaded.\n";
		
		#this is if they dont accept the applet...
		#echo "denied_message=You must agree to run this as 'trusted'.  Otherwise the applet doesn't have permission to open and upload your images.\n";

		#
		# If you want to impose a limit on the total size of the file upload
		# enter a value in kilobytes for the max_upload parameter. A value
		# of 0, the default means unlimited. Please make sure that the server
		# side configuration does not impose a lower limit than what you choose
		# for max_upload
		#
		# Example:
		#	 max_upload=1024
		#	 will impose a limit of 1 Mega Byte
		#

		echo "max_upload=7168\n";

		#
		# The max_upload property checks the sum of file sizes. You can impose
		# a limit on the size of individual files using the max_file property.
		# The value is in kilobytes. 0 means no limit.
		#
		# Example:
		#	max_file=2048
		#	for a limit of 2 Mega Bytes

		echo "max_file=7168\n";

		#
		# size_exceeded message will be displayed when either the max_upload or
		# max_file setting has been exceeded. If you enter a text message it
		# will be displayed as a popup. If you enter a URL, the chosen page be
		# loaded with in the applet.
		#
		# example:
		echo "size_exceeded=Oops!  Your files were too big!  Try uploading fewer in one try.\n";
		#

		#
		# As the name suggests the full_path setting determines if absolute pathnames
		# should be sent to the server. If you switch this off, folder information will
		# be stripped from the filenames.
		#

		echo "full_path=no\n";

		#
		# When the translate_path setting is switched on, windows style pathnames will
		# be converted to unix style paths. In other words '\' becomes '/'.  This
		# setting is required for Resumable file upload.
		#

		echo "translate_path=yes\n";

		#
		# When encode_path setting is switched on, pathnames are URLEncoded. This is
		# usefull if you are dealing with filenames that contain special characters.
		# this setting is required for resumable file upload.
		#

		echo "encode_path=yes\n";

		#
		# The progress indicator can display a thumbnail of each image as it is being
		# uploaded. To enable this feature uncomment the show_thumb property below.
		# examples:
		echo "show_thumb=1\n";

		#
		# If you need to disable the multiple upload feature, and to upload files one
		# at a time, switch to bachelor mode. When bachelor property is set the applet
		# will complain if you try to upload more than one file. Use the angry_bachelor
		# property to set the error message to be displayed.
		#
		# bachelor=1;
		# angry_bachelor=http://upload.thinfile.com/demo/single.html
		#
		# Sometimes you might want to bring up a file-open dialog box. If you use the
		# browse setting the drop target listens for mouse clicks and brings up a file
		# dialog. If instead of clicking on the drop target you wish to display a browse
		# button set the browse_button property as well.
		# example:
		#    browse=1
		#    browse_button=1

		echo "browse=1\n";
		echo "browse_button=1\n";

		#
		# By default the progress indicator will be hidden (closed) when the upload
		# completes. By uncommenting the following line you can continue to keep the
		# progress bar visible even after upload has been completed. The user will
		# then have to manually close the progress bar.
		# example:
		# 	monitor_keep_visible=yes
		#

		#
		# The applet or the entire browser can be redirected to another page
		# when upload comples. Select the destination URL with the
		# external_redir parameter.
		#
		# If you do not enter a value for the external_target property, the URL
		# given external_target will be loaded with in the applet. Otherwise
		# the page will be loaded in the target frame. To redirect the entire
		# browser window use '_top' as the target.
		#
		# If you wish to delay the redirect, enter a value for the
		# redirect_delay property (in milliseconds).
		# example:
		#echo "external_redir=" . $this->getUrl('.bulksuccess', null, true) . "\n";
		#echo "external_target=_top\n";
		#echo "redirect_delay=1000\n";

		echo "scale_images=yes\n";
		echo "img_max_width=600\n";
		echo "img_max_height=800\n";
	}

	/**
	* inits the bulk form page.
	*/
	function initBulkFormPage()
	{
		$this->assertLogin();
		
		$this->pageTitle = 'Bulk Image Upload';
	}

	/**
	* draws our bulk form page.  links to download java, upload single images, and the bulk applet
	*
	* @todo remove content id in extensible way
	*/
	function drawBulkFormPage()
	{
		global $me;
		
		//which album?
		$contentId = $this->params('content_id');

		//show our form if we can.
		if ($contentId)
		{
			echo "<table><tr><td width=\"50%\">";
			
			echo "<p>Welcome to our bulk upload tool.</p>";
			
			echo "<p>Please click 'Run', 'Always', or 'Yes' when it asks you to.  If
			you dont, you'll have to " .
			$this->getLink(".edit?content_id=$contentId", "upload images one by
			one") . ".</p>\n";
			
			echo "<p>If for some reason the tool doesn't show up, or just hangs on upload, you will
			need to <a
			href=\"http://www.java.com/en/download/index.jsp\">install Java&copy;</a></p>\n";

			echo "<p><b>Make sure you have the latest version of Java&copy; installed.</b></p>\n";
			
			echo "</td><td width=\"50%\" align=\"center\">";
			$this->drawBulkApplet($contentId);
			echo "</td></tr></table>";
		}
		//do we need to show a content select form?
		else
			$this->drawBulkFormSelect();
	}

	/**
	* Draws teh select form for what album to upload to...
	*
	* @todo remove content id in an extensible fashion
	*/
	function drawBulkFormSelect()
	{
		global $me;

		//we need to do a search for albums...	
		$albums = new Albums();
		$rs = $albums->search(array('user_id' => $me->id, 'show_empty' => 1));

		if (count($rs))
		{
			$items = array();
			foreach ($rs AS $obj)
				$items[$obj->id] = $obj->getName();
				
			$form = new Form();
			$form->action = $this->getUrl('.bulkform');

			$form->add('SelectField', 'content_id', array(
				'options' => $items,
				'required' => true,
				'title' => 'Choose an album to upload to:'
			));
			$form->addSubmit('Continue');

			$form->draw();
		}

		echo "<br/>" . $this->getLink('album.edit', 'Click here') . " if you want to create a new album.";
	}

	/**
	* draws teh markup for the bulk applet.
	*
	* @todo remove contentId in an extensible fashion
	*/
	function drawBulkApplet($contentId)
	{
		//use applet for weird browsers...
		$user_agent = $_SERVER['HTTP_USER_AGENT'];
		
		//give IE some weird update stuff
		if(strstr($user_agent,"MSIE"))
		{
			echo "<object classid=\"clsid:8AD9C840-044E-11D1-B3E9-00805F499D93\"";
			echo " width=\"300\" height=\"310\"";
			echo " codebase=\"http://java.sun.com/update/1.5.0/jinstall-1_5-windows-i586.cab#version=1,4,1\">";
		}
		//and firefox wants a nice standard object.
		else
		{
			echo '<object type="application/x-java-applet;version=1.4.1" width= "300" height= "310"  id="thinup" name="ThinUpload">';
			echo '<param name="MAYSCRIPT" value="yes">';
		} 

		if (!Config::get('thin_image_path'))
			echo '<param name="archive" value="http://' . Config::get('site_hostname') . Config::get('site_webpath') . 'ThinImage.jar">';
		else
			echo '<param name="archive" value="' . Config::get('thin_image_path') . 'ThinImage.jar">';
		echo '<param name="code" value="com.thinfile.upload.ThinImageUpload">';
		echo '<param name="name" value="Thin Image Upload">';
		echo '<param name="props_file" value="' . $this->getUrl(".bulkconfig?content_id=$contentId&auth=$_COOKIE[auth]", null, true) . '">';
		echo '</object>';
	}

	/**
	* inits the bulk post pages and authenticates.
	*/
	function initBulkPostPage()
	{	
		global $me;
		
		//authenticate it!
		$_COOKIE['auth'] = $this->params('auth');
		$me->authenticate();
		
		//we want this script to run no matter what...
		set_time_limit(0);
		ignore_user_abort(true);
		
		$this->setTemplate(new BlankTemplate());
	
		//try and authenticate the user...
		if (!$me->id)
			throw new PageError('We could not authenticate you.');
	}

	/**
	* handles the bulk post... spawns resize processes, etc.
	* @todo remove contentId in an extensible fashion
	*/
	function drawBulkPostPage()
	{
		global $me;
		
		$contentId = $this->params('content_id');
		
		//get all our file info..
		$data = $_FILES['userfile'];
		$files = $data['name'];
		$tmp = $data['tmp_name'];
		$type = $data['type'];

		//go thru our files...
		if (count($files))
		{
			foreach ($files AS $key => $file)
			{
				//create our new image
				$image = new Image();
				$image->file = $file;

				if ($image->hasField('content_id'))
					$image->content_id = $contentId;
				
				$image->user_id = $me->id;
				$image->add_date = date("Y-m-d H:i:s");
				$image->edit_date = date("Y-m-d H:i:s");

				//if its legit...?
				if (self::isImage($tmp[$key]))
				{
					//a real file?
					if ($image->moveUploadedFile($tmp[$key]))
					{
							//save it and process...
							$image->save();
							$image->prepFile();
							$image->resizeUploadedFile();
					}
					else
						echo "There was an error uploading $image->file, skipped.<br>\n";
				}
				else
					echo "$image->file is not a valid image file, skipped.<br>\n";
			}
			
			$album = new Album($contentId);
			$album->save();
		
			$this->drawBulkSuccessText($contentId);
		}
		else
			echo "There were no files uploaded.";
	}
	
	/**
	* draws the bulk post success
	* @param $contentId the id of the content if you want to link to it.
	*/
	function drawBulkSuccessText($contentId)
	{
		echo "Your images have been uploaded.<br><br>You may upload more if you wish.";
		echo "<br><br>Please allow a minute or two for your images to show up.";
	}

	/**
	* determines whether something is a real image (gif, jpg, png... no bmps here bitch).
	*
	* @param $path path to the image
	*
	* @return boolean of image status
	*/
	static public function isImage($path)
	{
		list($width, $height, $type, $attr) = getimagesize($path);

		//it must be a gif, jpg, or png
		if ($type < 4 && $type)
			return true;
		else
			return false;
	}

	/**
	* moves our uploaded file to a safe location
	*
	* @param $tmp the path to the temp file.
	*/
	function moveUploadedFile($tmp)
	{
		return move_uploaded_file($tmp, self::$origDir . "/$this->file");
	}

	/**
	* initializes the resize page... intended to be called async
	*/
	function initResizePage()
	{
		//we're gonna be called async... finish regardless
		ignore_user_abort();

		//we dont need no stinkin template
		$this->setTemplate(new BlankTemplate());

		$this->id = (int)$this->params('id');
	}

	/**
	* calls teh resize functions.  remember: called async.
	*/
	function drawResizePage()
	{
		if ($this->resize())
			echo 'Success!';
		else
			echo 'Failure. :/';
	}

	/**
	* gets the rss for the image. you shouldnt have to define getRssItem() for your image... automatic image rss feeds!!
	*
	* @return see BaseObject
	*/
	function getRssItem()
	{
		$item = parent::getRssItem();

		$item->title = $this->file;
		$item->description = $this->getImage('web');

		return $item;
	}

	/**
	* gets public data for serialized formats. see BaseObject for more info.
	*/
	function getPublicData()
	{
		$data = parent::getPublicData();

		if ($this->hasField('caption'))
			$data['caption'] = $this->caption;

		//save our sizes.
		$data['sizes'] = array();
		foreach ($this->sizeAr AS $size)
		{
			$data['sizes'][$size->extension]['html'] = $this->getImage($size->extension, false, false);
			$data['sizes'][$size->extension]['url'] = $this->url($size->extension);
		}
		return $data;
	}

	/**
	* get an array of fields in the table that stores the images
	*
	* @todo remove contentId in an extensible fashion
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getCreateFieldsArray()
	{
		$fields = parent::getCreateFieldsArray();

		$fields['content_id'] = "content_id int(11) default 0 not null";
		$fields['caption'] = "caption text default '' not null";
		$fields['file'] = "file varchar(255) default '' not null";
		$fields['image_size'] = "image_size varchar(30) default '' not null";
		$fields['width'] = "width int(11) default 0 not null";
		$fields['height'] = "height int(11) default 0 not null";
	
		return $fields;
	}

	/**
	* get an array of indexes on fields in the images table
	*
	* @todo remove contentId in an extensible fashion
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getCreateIndexesArray()
	{
		$fields = parent::getCreateIndexesArray();

		$fields['content_id'] = "key(content_id)";

		return $fields;
	}
}

/**
* @ingroup BJHelpers
* @brief ImageSize is a class for dealing with various image sizes.
* 
* It supports resizing, path storing, and resizing/cropping to squares.
*/
class ImageSize
{
	/**
	* @b see constructor params.
	*/
	public $name;
	/**
	* @b see constructor params.
	*/
	public $extension;
	/**
	* @b see constructor params.
	*/
	public $maxWidth;
	/**
	* @b see constructor params.
	*/
	public $maxHeight;
	/**
	* @b see constructor params.
	*/
	public $required;
	/**
	* @b see constructor params.
	*/
	public $scaleType;
	
	/**
	* our constructor to create the object.
	*
	* @param $name string of english name of size.
	* @param $extension simple, index key type string added to filename and used for searching for types.
	* @param $maxWidth integer maximum width
	* @param $maxHeight integer maximum height
	* @param $required boolean if its required to be resized.
	* @param $scaleType what type of scaling... ratio or square.  ratio preserves aspect, square crops to a center square.
	*/
	function __construct($name, $extension, $maxWidth, $maxHeight, $required = false, $scaleType = 'ratio')
	{
		$this->name = $name;
		$this->extension = $extension;
		$this->maxWidth = $maxWidth;
		$this->maxHeight = $maxHeight;
		$this->required = $required;
		$this->scaleType = $scaleType;
	}

	/**
	* gets the scaled size to resize to.
	*
	* @param $x the width of image
	* @param $y the height of image
	*
	* @param an array of sizes to resize to... index of x and y respectively.
	*/
	function getScaled($x, $y)
	{
		$rval = array();
		
		if ($this->scaleType == 'square')
		{
			if ($x < $this->maxWidth)
				$rval['x'] = $x;
			else
				$rval['x'] = $this->maxWidth;
				
			if ($x < $this->maxHeight)
				$rval['y'] = $x;
			else
				$rval['y'] = $this->maxHeight;

			return $rval;
		}
		else if ($this->scaleType == 'ratio')
		{
			if ($x && $y)
			{
				$xRatio = $this->maxWidth / $x;
				$yRatio = $this->maxHeight / $y;

				if ($xRatio > $yRatio)
					$ratio = $yRatio;
				else
					$ratio = $xRatio;

				if ($ratio >= 1)
				{
					$rval['x'] = $x;
					$rval['y'] = $y;
				}
				else
				{
					$rval['x'] = round($ratio * $x);
					$rval['y'] = round($ratio * $y);
				}
				
				return $rval;
			}
			else
				return false;
		}
	}
}
?>
