<?
/**
* @ingroup BaseObjects
* @brief A basic invitation object to handle invitations for a variety of things.
*
* It's designed to handle invitations for events, friendship, groups, etc.  It
* has approve/deny payload hooks and pages to handle them. 
*/
class BaseInvitation extends MyObject
{
	/**
	 * our constructor!
	 */
	public function __construct($id = null, $table = 'invites')
	{
		$this->hasAuthor = false;

		parent::__construct($id, $table);
	}

	/**
	 * our pages as xml.  invitations adds invite, approve, and deny.  we also remove edit.
	 *
	 * @return xml string data.
	 */
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();
		$xml .= $this->getInvitePageXml();
		$xml .= $this->getApprovePageXml();
		$xml .= $this->getDenyPageXml();

		return $xml;
	}

	/**
	 * this function gets our xml for the invite page.
	 */
	protected function getInvitePageXml()
	{
		return <<<XML
			<page name="invite">
			<param name="id" type="int" desc="id of the user you want to invite." required="1"/>
			</page>
XML;
	}

	/**
	 * this function gets our xml for the approve page.
	 */
	protected function getApprovePageXml()
	{
		return <<<XML
			<page name="approve">
			<param name="id" type="int" desc="id of the invite you're approving" required="1"/>
			</page>
XML;
	}

	/**
	 * this function gets our xml for the deny page.
	 */
	protected function getDenyPageXml()
	{
		return <<<XML
			<page name="deny">
			<param name="id" type="int" desc="id of the invite you're denying"/>
			</page>
XML;
	}

	/**
	 * this function gets our xml for the edit page. in this case, removes it.
	 */
	protected function getEditPageXml()
	{
		return '';
	}

	/**
	 * init our main page... invites.
	 */
	public function initMainPage()
	{
		$this->assertLogin();
		$this->pageTitle = 'Invite Friends';
	}

	/**
	 * draw our main page... recommend stuff.
	 */
	public function drawMainPage()
	{
		echo "<p>You should override " . get_class($this) . "::drawMainPage() and show a form for searching / inviting new users.</p>";
	}

	/**
	 * draw our info and links to processing pages.
	 */
	public function drawRow()
	{
		echo "<div class=\"" . get_class($this) . "Row\">";
		$this->drawLine();
		echo "</div>";
	}

	/**
	 * draw our info and links to processing pages.
	 */
	public function drawLine()
	{
		$inviter = new User($this->inviter_id);
		$invitee = new User($this->invitee_id);
		
		//who invited?
		if ($inviter->isMe())
			echo 'You have ';
		else
			echo $inviter->getName(true) . " has ";
			
		echo "invited ";
		
		//who was invited?
		if ($invitee->isMe())
			echo " you to join $inviter->their " . $this->getGroup() . ". ";
		else
			echo $invitee->getName(true) . " to join your " . $this->getGroup() . ". ";

		//can they do anything?
		if ($invitee->isMe())
		{
			echo "You may " . $this->getApproveLink() . " or " . $this->getDenyLink();
			echo " $inviter->their request.";
		}
	}

	/**
	 * gets a link to the approve page.
	 *
	 * @param $text the text to link to
	 *
	 * @return html string of link.
	 */
	public function getApproveLink($text = 'approve')
	{
		return $this->getLink(".approve?id=$this->id", $text);
	}

	/**
	 * gets a link to the deny page.
	 *
	 * @param $text the text to link to
	 *
	 * @return html string of link.
	 */
	public function getDenyLink($text = 'deny')
	{
		return $this->getLink(".deny?id=$this->id", $text);
	}

	/**
	 * can we delete the invite?
	 *
	 * @return if we can delete it or not... admins and inviters only.
	 */
	public function canDelete()
	{
		global $me;

		return ($me->isAdmin() || $this->inviter_id == $me->id);
	}

	/**
	 * can we view the invite?
	 *
	 * @return if we can delete it or not... admins and inviters/invitees only.
	 */
	public function canView()
	{
		global $me;

		return ($me->isAdmin() || $this->inviter_id == $me->id || $this->invitee_id == $me->id);
	}

	/**
	* init our view page.
	*/
	public function initViewPage()
	{
		parent::initViewPage();

		$this->pageTitle = 'Respond to Invitation';

		$this->assertLogin();
	}

	/**
	* draw our view page... basically link to the responding stuff.
	*/
	public function drawViewPage()
	{
		if ($this->canRespond())
			$this->drawLine();
		else
			echo "<p>There's nothing to see/do here, sorry.</p>";
	}

	/**
	 * inits our invite page.  logged in only.
	 */
	public function initInvitePage()
	{
		$this->pageTitle = 'Invite Friend';

		$this->assertLogin();
	}

	/**
	 * draw our invite page. gives them the note form, etc.
	 */
	public function drawInvitePage()
	{
		global $me;

		$user = new User($this->params('id'));

		$form = $this->createInviteForm($user);
		if ($form->isSubmitted())
		{
			$form->validate();
			$this->validateInvite($form);

			if (!$form->hasError())
				$this->invitePost($form);
		}
		else
			$form->setData($this->params());

		//finally draw the form if needed
		if ($form->needsDrawn())
			$form->drawAll();
	}

	/**
	 * creates our invite form.
	 *
	 * @param $user user object we want to invite.
	 *
	 * @return the form object to use
	 */
	public function createInviteForm($user)
	{
		$form = new Form(new OneColumnTableLayout());
		$form->action = $this->getUrl(".invite?id=$user->id");

		$this->addInviteFormFields($form, $user);

		return $form;
	}

	/**
	 * add fields to the invitation form.
	 *
	 * @param $form the form to add fields to.
	 * @param $object the user or object we're inviting.
	 */
	protected function addInviteFormFields($form, $object)
	{
		$form->add('TextAreaField', 'note', array(
					'title' => 'Note to ' . $object->getName(),
					'width' => '95%',
					'height' => '100px'
					));
		$form->addSubmit('Invite ' . $object->getName());
	}

	/**
	 * does any invite validation.  for you to hook into. set errors on teh form itself.
	 *
	 * @param $form the form to validate with.
	 */
	protected function validateInvite($form)
	{
		return true;
	}

	/**
	* create an invitation to a user.
	*
	* @param $inviteeId the id of the user you wish to invite.
	* @param $note an optional note to pass along
	*/
	public function invite($inviteeId, $note = null)
	{
		global $me;

		//set our ids.
		$this->saveInvite($inviteeId);

		//email them
		$this->emailInvite($note);
	}
	
	/**
	* process the invitation form post.  you should override this to add
	* success text.
	*
	* @param form the invitation form object.
	*/
	protected function invitePost($form)
	{
		$this->invite($this->params('id'), $form->getData('note'));
	}

	/**
	 * save our invite after its been posted.  override to add additional data.
	 *
	 * @param $form the form to get data from.
	 */
	protected function saveInvite($inviteeId)
	{
		global $me;

		//set our ids.
		$this->inviter_id = $me->id;
		$this->invitee_id = $inviteeId;
		$this->add_date = date("Y-m-d H:i:s");

		//save it!
		$this->save();
	}

	/**
	 * email the user that they have received an invite.
	 *
	 * @param $note the note passed in from a form or whatever
	 */
	protected function emailInvite($note = null)
	{
		global $me;

		//get our data.
		$user = new User($this->invitee_id);

		//create our email.
		if ($user->email_invites)
		{
			$subject = "New Friend Invitation";
			$body = '';
			$html = '';

			//do our text email!
			if ($note)
				$body .= $note . "\n\n" . "************************************************\n\n";
			$body .= $me->getName() . " ($me->email) has invited you to join $me->their " . $this->getGroup() . " on " . Config::get('site_name') . ".\n";
			$body .= "For your safety and privacy, only approve invitations from people you know.\n\n";
			$body .= "Visit the link below to process $me->their request.\n\n";
			$body .= $user->getTicket(strtolower(get_class($this)) . ".view?id=$this->id") . "\n\n";
			$body .= "Please note, you will not be connected until you approve the request.\n\n";
			$body .= "Thank you,\nThe Happy " . Config::get('site_name') . " Invitation Robot\n\n";
			$body .= "PS. You can turn these emails off at: " . $user->getUrl(".emailprefs", null, true);

			//do our html email!
			if ($note)
				$html .= $note . "<br/><hr/><br/>";
			$html .= $me->getName(true) . " ($me->email) has invited you to join $me->their " . $this->getGroup() . " on " . Config::get('site_name') . ".<br/>";
			$html .= "For your safety and privacy, only approve invitations from people you know.<br/><br/>";
			$html .= $user->getTicket(strtolower(get_class($this)) . ".view?id=$this->id", "Click here") . " to process $me->their request.<br/><br/>";
			$html .= "Please note, you will <b>not</b> be connected until you approve the request.<br/><br/>";

			$html .= "Thank you,<br/>The Happy " . Config::get('site_name') . " Invitation Robot<br/><br/>";
			$html .= "PS. You can turn these emails off " . $user->getLink(".emailprefs", 'here.', null, true);

			//send it!
			$user->mail($subject, $body, $html);
		}
	}

	/**
	* this function gets the english name of our group.
	*
	* @return the name as a string
	*/
	protected function getGroup()
	{
		return 'friends';
	}

	/**
	* tells us if we are allowed to respond to this invitation.
	*
	* @return boolean if we can respond or not.
	*/
	public function canRespond()
	{
		global $me;

		return ($me->id == $this->invitee_id);
	}

	/**
	 * do our init for approve.  they have to be the invitee to view it.
	 */
	public function initApprovePage()
	{
		global $me;

		//basic init stuff.
		$this->assertLogin();
		$this->id = $this->params('id');

		$this->pageTitle = 'Approve Invitation';

		//can they respond?
		if (!$this->canRespond())
			throw new PageError('You can only respond to your own invitations.');
	}

	/**
	 * draw the approve page.
	 */
	public function drawApprovePage()
	{
		$this->doApprovePayload();

		$inviter = new User($this->inviter_id);
		echo "<p>You have approved " . $inviter->getName(true) . "'s request.</p>";
	}

	/**
	 * do our approval payload.  deletes invite and recreates our session w/
	 * data.  you'll want to call addBuddy() here or whatnot.
	 */
	protected function doApprovePayload()
	{
		global $me;

		//delete our invite... we're done with it.
		$this->delete();

		//regenerate my tree...
		$me->createSession();
	}

	/**
	 * do our init for delete.  they have to be the invitee to do it.
	 */
	public function initDenyPage()
	{
		global $me;

		//basic init stuff.
		$this->assertLogin();
		$this->id = $this->params('id');

		$this->pageTitle = 'Deny Invitation';

		//can they respond?
		if (!$this->canRespond())
			throw new PageError('You can only respond to your own invitations.');
	}

	/**
	 * draw the deny page.
	 */
	public function drawDenyPage()
	{
		$this->doDenyPayload();

		$inviter = new User($this->inviter_id);
		echo "<p>You have denied " . $inviter->getName(true) . "'s request.</p>";
	}

	/**
	 * do our deny payload.  deletes invite and recreates our session w/
	 * data.  you probably wont need to add anything, but here it is anyway.
	 */
	protected function doDenyPayload()
	{
		global $me;

		//delete our invite... we're done with it.
		$this->delete();

		//regenerate my tree...
		$me->createSession();
	}

	/**
	 * get our default fields for the invitation
	 *
	 * @return a keyed array of field names to create statements for the field.
	 */
	public function getCreateFieldsArray()
	{
		//get daddy's fields.
		$fields = parent::getCreateFieldsArray();

		//no edit date
		unset($fields['edit_date']);

		//and user fields
		$fields["inviter_id"] = "inviter_id int(11) default 0 not null";
		$fields["invitee_id"] = "invitee_id int(11) default 0 not null";

		//send it back.
		return $fields;
	}

	/**
	 * get our default keys for the invitation
	 *
	 * @return a keyed array of field names to create index statements for the field.
	 */
	public function getCreateIndexesArray()
	{
		//get daddy's keys.
		$keys = parent::getCreateIndexesArray();

		//no edit date
		unset($keys['edit_date']);

		//and user keys
		$keys["inviter_id"] = "KEY (inviter_id)";
		$keys["invitee_id"] = "KEY (invitee_id)";

		//send it back.
		return $keys;
	}
}
?>
