<?
/**
* @ingroup BaseObjects
* @brief Basic, but full featured Message class.
*
* This is used in tandem with the BaseInbox and BaseOutbox modules to provide a fully functional private message system.  Its pretty much plug and play, while allowing you to customize.
*
* @todo Extend this from BaseComment
* @todo add create table stuff for in/outboxes
*/
abstract class BaseMessage extends MyObject
{
	/**
	* boolean if its read or not
	*/
	public $read = false;
	
	/**
	* table to store inbox relationships
	*/
	public $inboxTable;

	/**
	* table to store outbox relationships
	*/
	public $outboxTable;

	/**
	* @see BaseObject::__construct
	*
	* @param $inboxTable name of the inbox table
	* @param $outboxTable name of the outbox table
	*/
	public function __construct($data = null, $table = 'messages', $inboxTable = 'inbox', $outboxTable = 'outbox')
	{
		parent::__construct($data, $table);

		//save our tables.
		$this->inboxTable = $inboxTable;
		$this->outboxTable = $outboxTable;

		$this->recipients = $this->getRecipients();

		$this->needsJs('lib/js/message.js');
		
		if (is_array($data))
			$this->read = $data['read'];

		$this->fullTextFields[] = 'subject';
		$this->fullTextFields[] = 'body';
	}

	/**
	* delete our message, and in/outboxes, and kids too
	*/
	public function delete()
	{
		dbExecute("
			DELETE FROM $this->inboxTable
			WHERE message_id = '$this->id'
		");
		
		dbExecute("
			DELETE FROM $this->outboxTable
			WHERE message_id = '$this->id'
		");

		$kids = $this->getChildren();
		if (count($kids))
		{
			foreach ($kids AS $kid)
				$kid->delete();
		}

		parent::delete();
	}

	/**
	* get recipient ids of the message
	*
	* @return an array of recipients ids, or the id of lone one
	*/
	protected function getRecipients()
	{
		if (!is_array($this->recipients))
			$ret = explode(",", $this->recipients);
		else
			$ret = $this->recipients;
		
		return $ret;
	}

	/**
	* get edit page params to allow us to pass in user id, parent_id, etc
	*/
	protected function getEditPageParamXml()
	{
		return <<<XML
			<param name="subject"/>
			<param name="body"/>
			<param name="user_id"/>
			<param name="parent_id"/>
XML;
	}

	/**
	* prep our edit page, can we reply, etc.
	*/
	public function initEditPage()
	{
		global $me;
		
		parent::initEditPage();

		$this->pageTitle = 'Send Private Message';
		
		$parentId = $this->params('parent_id');
		if ($parentId)
	 	{
			$class = get_class($this);
			$parent = new $class($parentId);
			if (!$parent->canView())
				throw new PageError('You cannot reply to this message.');
		}
	}

	/**
	* can we edit our message? default: no
	*/
	public function canEdit()
	{
		global $me;
		
		if ($this->id && $me->isAdmin())
			return true;
		if (!$this->id)
			return true;
		
		return false;
	}

	/**
	* add our fields to our edit form: recipients, subject, message, etc.
	*/
	public function editFormAddFields($form)
	{
		global $me;

		//set our layout
		$form->layout = new OneColumnTableLayout(); 
		
		$recipients = array();
		
		//create our recipient list
		//did we get a user id? (sending it directly to someone)
		$userId = $this->params('user_id');
		$user = new User($userId);
		if ($userId)
			$recipients[$user->id] = $user->getName();

		//otherwise call our hook function
		$recipients = $this->createRecipientsList($recipients);
		
		//add our recipients field!
		$form->add('MultiSelectField', 'recipients', array(
			'multiple' => true,
			'required' => true,
			'size' => 10,
			'noneTitle' => false,
			'options' => $recipients,
			'explanation' => '
				<p>Tip: By holding the \'Ctrl\' key and clicking on names, you
				can select multiple people.</p>

				<p>Additonally, you can select large chunks of the list by
				holding \'Shift\' and clicking names</p>
			'
		));
		
		$form->add('TextField', 'subject', array(
			"required" => true,
			"width" => '95%',
		));
		
		$form->add('TextAreaField', 'body', array(
			"required" => true,
			"title" => 'Message',
			"width" => '95%',
			"height" => '150px'
		));

		$form->add('HiddenField', 'parent_id', array());

		$form->addSubmit('Send Message');
	}
	
	/**
	* stub function for you to add custom recipients in.
	*
	* @param $recipients an array of recipients as user_id => name
	* @return an array of recipients as user_id => name
	*/
	protected function createRecipientsList($recipients)
	{
		return $recipients;
	}

	/**
	* create a form for replying
	*
	* @param $recipients an array of recipients ids for message
	* @return a Form object
	*/
	public function createReplyForm($recipients)
	{
		$form = new Form();
		$form->action = $this->getUrl(".edit");
		
		//if we have recipients... its a reply all
		if (count($recipients) > 1)
			$form->name = 'replyAllForm_' . $this->id;
		else
			$form->name = 'replyForm_' . $this->id;

		//we're the parent..
		$form->add('HiddenField', 'parent_id', array(
			'value' => $this->id
		));
		//here are our recipients..
		$form->add('HiddenField', 'recipients', array(
			'value' => array_unique($recipients)
		));
		//set our subject too.
		$form->add('HiddenField', 'subject', array(
			'value' => $this->subject
		));
		//let them enter text for the body.
		$form->add('TextAreaField', 'body', array(
			'height' => '75px',
			'width' => '95%',
			'title' => ''
		));

		if (count($recipients) == 1)
		{
			$user = new User($recipients[0]);
			$form->addSubmit('Send to ' . $user->getName());
		}
		else
			$form->addSubmit('Send to All');

		return $form;
	}

	/**
	* load in our recipients and stuff
	*
	* @param $form form to load data into
	*/
	function editFormLoad($form)
	{
		$userId = $this->params('user_id');
		$parentId = $this->params('parent_id');
		
		if (!$this->id)
		{
			$data = array();
			
			//are we replying to a parent?
			if ($parentId)
			{
				$class = get_class($this);
				$parent = new $class($parentId);
				
				//our subject
				if (preg_match('/^Re:/', $parent->subject))
					$subject = $parent->subject;
				else
					$subject = 'Re: ' . $parent->subject;
				
				//create our data...
				$data['parent_id'] = $parentId;
				$data['recipients'] = implode(',', $parent->recipients);
				$data['subject'] = $subject;
			}
			//are we sending to a specific user?
			else if ($userId)
				$data['recipients'] = array($userId);

			//finally set our data.
			$form->setData($data);
		}
		else
			parent::editFormLoad($form);
	}

	/**
	* post processing, add recipients, etc.
	*
	* @see editPagePostSaveSync()
	*/
	function editPagePostSaveSync($form)
	{
		global $me;
		
		parent::editPagePostSaveSync($form);

		//add our recipients
		$recipients = explode(',', $form->getData('recipients'));
		foreach ($recipients AS $id)
			$this->addRecipient($id);

		//set it as ours
		$this->addSender($me->id);
	}

	/**
	* add outbox record for sender
	*
	* @param $id id of sender
	*/
	protected function addSender($id)
	{
		dbExecute("
			INSERT INTO $this->outboxTable
				(user_id, message_id)
			VALUES
				('$id', '$this->id')
		");
	} 

	/**
	* add recipient for message to inbox
	*
	* @param $id the id of the user
	*/
	protected function addRecipient($id)
	 {
		//put it in our table.
		dbExecute("
			INSERT INTO $this->inboxTable
				(user_id, message_id)
			VALUES
				('$id', '$this->id')
		");

		//email our person.
		$this->emailRecipient($id);
	}

	/**
	* email new message notification.
	*
	* $id id of the user to send to
	*/
	protected function emailRecipient($id)
	{
		global $me;
		
		$user = new User($id);
			
		$subject = $me->getName() . ' sent you a message on ' . Config::get('site_name');

		//plain text
		$body = "You can either log in and view your inbox, or go straight to the message with the link below.\n\n";
		$body .= $user->getTicket(strtolower(get_class($this)) . ".view?id=$this->id") . "\n\n";
		$body .= "Thank you,\nThe Happy " . Config::get('site_name') . " Message Notification Robot";
		$body .= "\n\nPS. You can turn these emails off at:\n" . $user->getUrl(".emailprefs", null, true);				
		
		//html
		$html = $user->getTicket(strtolower(get_class($this)) . ".view?id=$this->id", 'Click here to read the message.') . "<br/><br/>";
		$html .= "Thank you,<br/>The Happy " . Config::get('site_name') . " Message Notification Robot";
		$html .= "<br/><br/>PS. You can turn these emails off " . $user->getLink('.emailprefs', 'here.', null, true);				

		$user->mail($subject, $body, $html);
	}

	/**
	* prep view page: assert login, etc.
	*/
	public function initViewPage()
	{
		$this->assertLogin();

		//get our id..
		$this->id = $this->params('id');
		$this->recipients = $this->getRecipients();

		//init our page... with subject as the title
		parent::initViewPage();

		$this->pageTitle = "View Message - $this->subject";
	}
	
	/**
	* can we view it?  must be recipient or sender
	*/
	public function canView()
	{
		global $me;
		
		if ($this->user_id == $me->id)
			return true;
		if (in_array($me->id, $this->recipients))
			return true;
		return false;
	}

	/**
	* show our threaded message.
	*/
	public function drawViewPage()
	{
		global $me;
	
		//if we have a parent... draw that one first to give context.
		if ($this->parent_id)
		{
			$class = get_class($this);
			$parent = new $class($this->parent_id);

			if ($parent->parent_id)
			{
				echo "<div class=\"titleBar\">\n";
				echo "<div class=\"titleText\">\n";
				$this->drawUrl(".view?id=$parent->parent_id", 'Previous Message');
				echo "</div></div>";
			}

			echo "<ol class=\"threaded\">\n";
			$parent->drawMessage(true);
			echo "</ol>\n";
		}
		//otherwise just draw our message.
		else
		{
			echo "<ol class=\"threaded\">\n";
			$this->drawMessage(true);
			echo "</ol>\n";
		}
	}

	/**
	* draw our recipients
	*/
	public function drawRecipients()
	{
		$recips = array();
		foreach ($this->recipients AS $key)
		{
			$recipient = new User($key);
			$recips[] = $recipient->getName(true, 'tiny');
		}
		echo implode(", ", $recips);
	}
	
	/**
	* get ids of our recipients
	*
	* @param $ignoreMe do we ignore the current user or not?
	*
	* @return an array of recipient ids
	*/
	protected function getRecipientIds($ignoreMe = false)
	{
		global $me;
		
		$recips = array();
		foreach ($this->recipients AS $key)
		{
			if (!$ignoreMe || ($ignoreMe && $me->id != $key))
				$recips[] = $key;
		}
		
		return $recips;
	}

	/**
	* draw our message
	*
	* @param $kids the array of messages to use
	*/
	public function drawMessage($kids = false)
	{
		global $me;

		if ($this->canView())
		{
			//it's been read.
			$this->markRead();
			
			echo "<li class=\"threaded\">\n";
		
			//our title bar stuff.
			$this->drawTitleBar();
			$this->drawMessageDetails();
		
			//draw our container
			echo "<div class=\"childContainer\">\n";
		
			//our actual content
			$this->drawMessageBody();
		
			//only reply if we sent it.
			if (!$this->creator->isMe())
				$this->drawMessageReplyForm();
		
			//draw our children
			$this->drawChildMessages($kids, $this->id);
			
			echo "</div>";
			
			//end comment
			echo "</li>";
		}
	}

	/**
	* draw our title bar.
	*/
	protected function drawTitleBar()
	{
		//heres our title bar
		echo "<div class=\"titlebar\">";
		
		//our title icon
		echo '<table class="struct"><tr><td class="titleLeft">';
		
		echo $this->creator->getIcon();

		//our title div
		if ($this->creator->isMe())
			echo "You";
		else
			echo $this->creator->getName(true);
		echo " said to ";

		//who did it get said to?
		if (count($this->recipients) > 1)
			echo "everyone";
		else if ($this->recipients[0] == $me->id)
			echo "me";
		else
		{
			$recip = new User($this->recipients[0]);
			echo $recip->getName(true);
		}
			
		echo ": " . Time::formatRelative($this->add_date);

		//mid-structure
		echo '</td><td class="titleRight">';
	
		//our reply links
		if (!$this->creator->isMe())
		{
			echo "\t<span class=\"titleLink\" id=\"messageReplyLink_$this->id\">";
			echo "<a href=\"" . $this->getUrl(array('page' => 'edit', 'parent_id' => $this->id));
			echo "\" onclick=\"return showReplyForm('$this->id');\">reply</a></span>\n";
			if (count($this->recipients) > 1)
			{
				echo "\t<span class=\"titleLink\" id=\"messageReplyAllLink_$this->id\">";
				echo "<a href=\"" . $this->getUrl(array('page' => 'edit', 'parent_id' => $this->id));
				echo "\" onclick=\"return showReplyAllForm('$this->id');\">reply all</a></span>\n";
			} 
		}
		
		echo "\t<span class=\"titleLink\" id=\"messageOptions_$this->id\">";
		echo "<a href=\"#\" onclick=\"return showMessageOptions('$this->id');\">options</a>";
		echo "</span>\n";

		//end our stupid structure
		echo "</td></tr></table>";

		//end title bar
		echo "</div>\n";
	}

	/**
	* draw the details div of our message
	*/
	protected function drawMessageDetails()
	{
		//our hidden detail div.
		echo "<div class=\"messageDetail\" id=\"message_$this->id\" style=\"display: none;\">";
		echo "<table>";
		echo "<tr><td>Subject:</td><td> $this->subject</td></tr>";
		echo "<tr><td>Sent by:</td><td> " . $this->creator->getName(true) . "</td></tr>\n";
		echo "<tr><td>Sent: </td><td>" . Time::formatRelative($this->add_date, false) . "</td></tr>\n";
		echo "<tr><td>Recipients: </td><td>";
		$this->drawRecipients();
		echo "</td></tr></table>";
		echo "</div>\n";
	}
	
	/**
	* draw the body of the message w/ optional signature
	*/
	protected function drawMessageBody()
	{
		echo "<div class=\"messageBody\">";
		echo Linkify::bbcode($this->body);
		$this->drawMessageSignature();
		echo "</div>";
	}

	/**
	* optional message signature
	*
	* extend to implement
	*/
	protected function drawMessageSignature()
	{
	}

	/**
	* draw message reply forms for both single and multiple (if possible)
	*/
	protected function drawMessageReplyForm()
	{
		//and our actual reply form
		echo "<div class=\"messageReply\" id=\"messageReply_$this->id\" style=\"display: none;\">\n";
		$form = $this->createReplyForm(array($this->creator->id));
		$form->drawAll();
		echo "</div>\n";
		
		//and our actual reply to all form
		if (count($this->recipients) > 1)
		{
			echo "<div class=\"messageReply\" id=\"messageReplyAll_$this->id\" style=\"display: none;\">\n";
			
			$recips = $this->getRecipientIds(true);
			$recips[] = $this->user_id;

			$form = $this->createReplyForm($recips);
			$form->drawAll();
			echo "</div>\n";
		}
	}

	/**
	* draw the child messages of a message
	*
	* @todo make this more like BaseComment, or extend from it
	*/
	function drawChildMessages()
	{ 
		//draw our kids if we can.
		$kidRs = dbQuery("
			SELECT id
			FROM $this->tableName
			WHERE parent_id = '$this->id'
			ORDER BY add_date DESC
		");
	
		if (dbGetNumRows($kidRs))
		{
			echo "<div class=\"threaded\">";
			echo "<ol class=\"threaded\">\n";
			while ($kidAr = dbFetchAssoc($kidRs))
			{
				$class = get_class($this);
				$kid = new $class($kidAr['id']);
				$kid->drawMessage(true);
			}
			echo "</ol>\n";
			echo "</div>";
		}
	}

	/**
	* draw the header row for in/outbox display
	*
	* @param $type either inbox or outbox
	*/
	function drawHeaderRow($type)
	{
		echo "<tr class=\"headerRow\">\n";
		echo "\t<th id=\"messageCheck\">\n";
		echo "\t\t<input type=\"checkbox\" name=\"checkAll\" onclick=\"setAllMessages(this.checked);\"/>\n";
		echo "\t</th>\n";
	
		if ($type == 'inbox')
			echo "\t<th id=\"messagePeople\">From</th>\n";
		else
			echo "\t<th id=\"messagePeople\">Recipients</th>\n";
		
		echo "\t<th id=\"messageSubject\">Subject</th>\n";
	
		echo "\t<th id=\"messageDate\">Sent</th>\n";
	
		echo "</tr>\n";
	}

	/**
	* draw our row
	*
	* @param $type inbox or outbox
	* @param $on on/off row alternation.  for css classes.
	*/
	function drawRow($type, $on)
	{
		global $me;
		
		$class = "messageRow";
		if (!$this->read && $me->id != $this->user_id)
			$class .= " unread";
		if ($on)
			$class .= " on";
		
		echo "<tr class=\"$class\">\n";

		echo "\t<td class=\"messageCheck\">\n";
		echo "\t\t<input type=\"checkbox\" name=\"message[]\" value=\"$this->id\"/>\n";
		echo "\t</td>\n";
	
		if ($type == 'inbox')
		{
			echo "\t<td class=\"messagePeople\">\n";
			echo "\t\t" . $this->creator->getName(true, 'tiny') . "\n";
			echo "\t</td>\n";
		}
		else
		{
			echo "\t<td class=\"messagePeople\">\n";
			echo "\t\t";
			echo $this->drawRecipients();
			echo "\n\t</td>\n";
		}
		
		echo "\t<td class=\"messageSubject\">\n\t\t";
		$this->drawUrl(array('page' => 'view', 'id' => $this->id), $this->subject);
		echo "\n\t</td>\n";
	
		
		echo "\t<td class=\"messageDate\">\n";
		echo "\t\t" . Time::formatRelative($this->add_date, false) . "\n";
		echo "\t</td>\n";
		echo "</tr>\n";
	}

	/**
	* mark this message as read.
	*
	* @param $force do we force it?  otherwise will check if i'm a recipient
	*/
	function markRead($force = false)
	{
		global $me;
		
		if (in_array($me->id, $this->recipients) || $force)
		{
			dbExecute("
				UPDATE $this->inboxTable
				SET `read` = '1'
				WHERE user_id = '$me->id'
					AND message_id = '$this->id'
			");
		}
	}
	
	/**
	* get children of the message.
	*
	* @todo double check if this works or is used
	*
	* @return array of child objects
	*/
	function getChildren()
	{
		$kids = array();
		
		if ($this->id)
		{
			$rs = dbQuery("
				SELECT *
				FROM $this->tableName
				WHERE parent_id = '$this->id'
			");
			while ($ar = dbFetchAssoc($rs))
				$kids[] = new Message($ar);
		}

		return $kids;
	}

	/**
	* get message data for syndication
	*/
	function getRssItem()
	{
		$rss = parent::getRssItem();

		$rss->title = $this->subject;
		$rss->description = $this->body;

		return $rss;
	}
	
	/**
	* get message data for API's (will be protected though)
	*/
	function getPublicData()
	{
		$data = parent::getPublicData();

		$data['name'] = $this->subject;
		$data['message'] = $this->body;
		$data['read'] = $this->read;

		return $data;
	}

	/**
	* get name of message
	*/
	public function getName($link = false)
	{
		return parent::getName($link, $this->subject);
	}

	/**
	* get an array of fields in the table that stores the messages
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getCreateFieldsArray()
	{
		$fields = parent::getCreateFieldsArray();

		$fields['parent_id'] = "parent_id int(11) default 0 not null";
		$fields['subject'] = "subject varchar(255) default '' not null";
		$fields['body'] = "body text default '' not null";
		$fields['recipients'] = "recipients text default '' not null";
	
		return $fields;
	}

	/**
	* get an array of indexes on fields in the messages table
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getCreateIndexesArray()
	{
		$fields = parent::getCreateIndexesArray();

		$fields['parent_id'] = "key(parent_id)";

		return $fields;
	}
}
?>
