<?
/**
* @ingroup BaseModules
* @brief A modules that handles the very basic, core tasks of the module system.
*
* It handles pages, page validation, parameter validation, creating
* links, error handling, status messages, css, javascript, rss, and and
* provides many useful utility functions to make programming for the web as
* ridiculously simple as possible. 
*/
class BaseModule
{
	/**
	* @b public String base url to use for creating links
	*/
	public $baseUrl;

	/**
	* @b public String what page we are on.
	*/
	public $page;

	/**
	* @b public String title of the page we are on.
	*/
	public $pageTitle;

	/**
	* @b BaseTemplate our template to use to draw our page.
	*/
	public $template = null; 

	/**
	* @b public Boolean an english word description of what the module is.
	*/
	public $english;

	/**
	* @b public String xml description of object.  derived from getPagesXml.  this should probably be private.
	*/
	public $xml;

	/**
	* @b public are we caching the page?
	*/
	public $doCachePage = false;
	
	/**
	* @b public the lifetime of the cached page.  defaults to 1 hour.
	*/
	public static $pageCacheLife = 900;
	
	/**
	* @b private Keyed Array of parameters for the page.
	*/
	private $params = array();

	/**
	* @b private Array of string errors for the page.
	*/
	private $error = null;

	/**
	* @b private Array of string status messages for the page
	*/
	private $status = array();

	/**
	* @b private Array of BaseBox objects for the page.
	*/
	private $sideboxes = array();

	/**
	* @b private Array of text/links of navigation for the page.  May or may not be in future BaseModules
	*/
	private $navigation = array();

	/**
	* Array of RSS feeds for drawing in <head> of page
	*/
	private static $Feeds = array();

	/**
	* corresponding array of titles for RSS feeds to be drawn
	*/
	private static $FeedTitles = array();

	/**
	* store the name of our module for our XML
	*/
	public $_Name = null;
	
	/**
	* store the description of our module for our XML
	*/
	public $_Description = null;
	
	/**
	* store the version of our module for our XML
	*/
	public $_Version = null;
	
	/**
	* store the author of our module for our XML
	*/
	public $_Author = null;
	
	/**
	* store the website of our module for our XML
	*/
	public $_Website = null;

	/**
	* Creates a new BaseModule
	*/
	public function __construct()
	{
		$this->_Name = preg_replace("/([A-Z])/", " $1", get_class($this));

		$this->objectType = get_class($this);
		$this->english = strtolower(get_class($this));
		$this->baseUrl = Config::get('site_baseurl');
	}
	
	/**
	* This function initializes a page in a module.
	*
	* @param $page the page to initialize.  It must be defined in the xml. Optional
	* @param $vars a keyed array of parameters for the page.
	*/
	public function init($page = 'main', $vars = array())
	{
		//not installed?  give a status message
		if (!Config::get('basejumper_installed'))
		{
			$this->addStatus('BaseJumper is in installation mode. Visit the ' .
			$this->getLink("main.sitesql", "site SQL") . ' page to create your
			site sql.<br/><br/>After that, do a
			<b>Config::set("basejumper_installed", true);</b> in your
			config.inc.php when you have finished installing your
			site.<br/><br/>You may also want to visit the <a
			href="http://www.osotite.com/documentation/installation">BaseJumper
			installation</a> page.');
		}
			
		//trim the bush
		$page = trim($page);
	
		//make sure we got a value
		if (!$page)
			$page = 'main';

		//now save it.
		$this->page = $page;
		
		//get our module xml..
		$xml = $this->parseModuleXml();

		//does our page exist?
		$rs = $xml->xpath("//page[@name='$this->page']");
			
		//get our specific page
		$pageXml = $rs[0];
			
		//what about custom template?  we may need to set this before we do any error checking.
		if ($pageXml['template'])
		{
			$templateClass = "$pageXml[template]Template";
			if (class_exists($templateClass))
				$this->setTemplate(new $templateClass());
		}
		
		//do a bit of error checking.
		if (!count($rs))
			throw new PageError("The page '$this->page' does not exist in the '" . get_class($this) . "' module.\n");
		else if (count($rs) > 1)
			throw new PageError("There is more than one '$this->page' in the '" . get_class($this) . "' module.");
		else
		{
			//get our title
			if ($pageXml['title'])
				$this->pageTitle = $pageXml['title'];

			//are we caching?
			if ($pageXml['cache'] == 'true' || $pageXml['cache'] == '1')
				$this->doCachePage = true;
		
			//initialize our parameters
			$this->initParams($pageXml, $vars);
			
			//our page too
			$this->initPage($this->page);
		}
	}

	/**
	* This function handles drawing the content area for the module page.
	* Additionally, if any extra markup is needed, it draws that too.
	*/
	public function draw()
	{
		//draw our stuff.
		$this->template->draw();
	}

	/**
	* sets our template object.
	*
	* @param the template object to set.
	*/
	public function setTemplate(BaseTemplate $template)
	{
		$this->template = $template;
		$this->template->object = $this;
	} 

	/**
	* This function initializes our page parameters and does quite a bit of error checking.
	*
	* @param $page a simplexml object that is just the page xml (options and such)
	* @param $vars a keyed array of variables to parse
	*/
	public function initParams($page, $vars = array())
	{
		//get our params..
		$this->params = array();
		if (is_object($page->param[0]))
		{
			foreach ($page->param AS $param)
			{
				$name = (string)$param['name'];
				$required = (int)$param['required'];
				$type = (string)$param['type'];
				$regex = (string)$param['regex'];
				
				//if its required... make sure its there
				if ($required)
				{
					if (isset($vars[$name]) && trim($vars[$name] != ''))
						$this->params[$name] = trim($vars[$name]);
					else
						throw new PageError("Expected parameter '$name' for page '$page[name]'");
				}
				//otherwise just get it and put it in our params array
				else if (isset($vars[$name]))
						$this->params[$name] = $vars[$name];

				//start off assuming its valid.
				$valid = true;

				//if we actually got something... verify it.
				if (isset($this->params[$name]))
				{
/*
					//does it need to be an array?
					if ($type == 'array')
					{
						if (!is_array($vars[$name]))
							throw new PageError("Expected parameter '$name' to be an array.");
					}
*/
					//or an integer?
					if ($type == 'int')
					{
						if ($this->params['name'] != (int)$this->params['name'])
							throw new PageError("Parameter '$name' must be an integer.");
					}
					//or a float?
					else if ($type == 'float')
					{
						if ($this->params['name'] != (float)$this->params['name'])
							throw new PageError("Parameter '$name' must be an float.");
					}
					//okay, if we have options... make sure its one of those.
					else if (is_object($param->option[0]))
					{
						$valid = false;
						foreach ($param->option AS $option)
						{
							if ($option['value'] == $this->params[$name])
							{
								$valid = true;
								break;
							}
						}
					
						//if its invalid... create our error.
						if (!$valid)
						{
							$error = "The parameter '$name' can only be one of these values: ";
							foreach ($param->option AS $option)
								$values[] = "'$option[value]'";
							
							$error .= implode(", ", $values);

							throw new PageError($error);
						}
					}
					//otherwise, is it a regex...?
					else if ($regex)
					{
						if (!preg_match($regex, $this->params[$name]))
							throw new PageError("The parameter '$name' must
							must match the <a
							href=\"http://www.php.net/pcre\">regular
							expression</a>:</b><br/>\n
							<pre>$regex</pre>\n");
					}
				}
			}
		}
	}
		
	/**
	* This function initializes the page we eventually want to draw.  It passes off the page initialization to init{$page}Page()
	* 
	* @param $page the page we want to initialize
	*/
	function initPage($page)
	{
		//some pages may have custom init functions (we want this called first
		//as they may want to redirect after they are done and not have to call
		//a draw function.
		$func = "init" . ucfirst($page) . "Page";
		if (method_exists($this, $func))
			$this->$func();
		
		//make sure our draw function exists!
		$func = "draw" . ucfirst($page) . "Page";
		if (!method_exists($this, $func))
			throw new PageError(get_class($this) . "::$func() does not exist.  You must define it.\n");
	}

	/**
	* This function gets the title of the page
	*
	* @return The title of the page.
	*/
	public function getTitle()
	{
		if ($this->hasError())
			$this->pageTitle = 'Error';

		return $this->pageTitle;
	}

	/**
	* This function gets our page as a string
	*
	* @return a string consisting of the contents of the page
	*/
	public function getContent()
	{
		//start our buffering...
		ob_start();
		
		//call our drawing function
		$func = "draw" . ucfirst($this->page) . "Page";
		$this->$func();

		//get our content...
		$content = trim(ob_get_contents());

		//end output buffering...
		ob_end_clean();
	
		return $content;
	}

	/**
	* This function gets the errors for the module
	*
	* @return the errors generated during page initialization
	*/
	public function getError()
	{
		return $this->error;
	}
	
	/**
	* This function gets the status messages for the module
	*
	* @return the status messages generated during the running of the page
	*/
	public function getStatus()
	{
		return $this->status;
	}
	
	/**
	 * Gets a string url from an array of parameters
	 * @param $params the parameters for the link.
	 * @param $base the base url to use.  default: $this->baseUrl
	 *
	 * @return the string url you want!
	 */
	public function getUrl($params = array(), $base = null, $full = false)
	{
		if ($base === null)
			$url = $this->baseUrl;
		else
			$url = $base;

		if ($full)
			$url = "http://" . Config::get('site_hostname') . $url; 

		//do some regexp foo?
		if (is_string($params))
		{
			//defaults
			$module = strtolower(get_class($this));
			$page = 'main';
			$vars = array();
			
			//if (preg_match("/\%((\s*)\.?) (\s*)(\?)(.*))/", $params, $match))
			if (preg_match("/^\w*$/", $params, $match))
				$module = $match[0];
			else if (preg_match("/(\w*)\.(\w*)\??([^#]*)(#\w*)?/", $params, $match))
			{
				//get our matches.
				if ($match[1])
					$module = $match[1];
				//and our page
				if($match[2])
					$page = $match[2];
				//plus parameters
				if ($match[3])
				{
					$pairs = explode("&", $match[3]);
					if (count($pairs))
					{
						foreach ($pairs AS $pair)
						{
							$var = explode("=", $pair);
							$vars[$var[0]] = $var[1];
						}
					}
				}
				if ($match[4])
					$anchor = $match[4];
			}

			//setup our params array.
			$params = array(
				'module' => $module,
				'page' => $page,
			);
			$params += $vars;
		}

		//add the class name in...
		if (!isset($params['module']))
			$params['module'] = strtolower(get_class($this));
		if (!isset($params['page']))
			$params['page'] = 'main';

		//are we using mod_rewrite?
		if (Config::get("use_mod_rewrite"))
		{
			//remove php file
			$url = str_replace("index.php", "", $url);
			
			//set page and module.
			$url .= $params['module'] . "/" . $params['page'];
			unset($params['module']);
			unset($params['page']);

			//add in our params
			if (count($params))
				foreach ($params AS $key => $val)
					$url .= "/$key/" . urlencode($val);
		}
		else
		{
			//snag the params.
			if (is_array($params))
				foreach ($params AS $key => $val)
					$url = Util::setUrlParam($url, $key, $val);
		}

		//add in our #foo.
		if ($anchor)
			$url .= $anchor;

		//return it.
		return $url;
	}

	/**
	* This function gets the parameters from an array in url form
	*
	* @param $array a keyed array of parameters
	* @return a string of the parameters in url form.
	*/
	function getParams($array)
	{
		$url = $this->getUrl($array);
		$link = explode("?", $url);
		return $link[1];
	}

	/**
	* This function draws a link.
	*
	* @param $params a keyed array of parameters
	* @param $text the text to link
	*/
	public function drawUrl($params, $text)
	{
		echo $this->getLink($params, $text);
	}

	/**
	* This function gets a text link
	*
	* @param $params a keyed array of parameters
	* @param $text the text to link
	* @param $base the base url to use. Optional
	* @param $full use a full link with http:// and hostname. Optional
	*
	* @return a text/html link ready to be echo'd
	*/
	public function getLink($params, $text, $base = null, $full = false)
	{
		$url = $this->getUrl($params, $base, $full);
		return "<a href=\"$url\" class=\"BaseLink\">$text</a>";
	}

	/**
	* Do we have errors or that?
	*
	* @return Boolean true or false depending on whether errors were generated during initialization or not
	*/
	public function hasError()
	{
		return ($this->error !== null);
	}

	/**
	* Sets the error message for the page.
	*
	* @param $error an error string.
	*/
	public function setError($error)
	{
		$this->error = $error;
		$this->pageTitle = 'Error';
	}

	/**
	* Do we have status messages or not?
	*
	* @return Boolean true or false depending on whether status messages were generated during initialization or not
	*/
	public function hasStatus()
	{
		return (bool)count($this->status);
	}

	/**
	* Add a status message to the module
	*
	* @param $str string status message.
	*/
	public function addStatus($str)
	{
		$this->status[] = $str;
	}

	/**
	* This function initializes the parameters page.  You probably wont ever use it.
	*/
	public function initParametersPage()
	{
		$this->pageTitle = get_class($this) . " Module - Pages and Parameters";
	}

	/**
	* This function draws the module parameters page.. which is a nice representation of the pages contained in the module.
	*/
	public function drawParametersPage()
	{
		global $me;
		
		$xml = $this->parseModuleXml();

		echo "\n<h2>Pages</h2>\n";

		echo "<ul>\n";	
		foreach ($xml->page AS $page)
		{
			echo "\t<li><b>$page[name]</b>";

			if ($page['desc'])
				echo " - $page[desc]";
			
			echo "\n\t\t<ul>\n";
			if (is_object($page->param[0]))
			{
				echo "\t\t\t<li><b>parameters:</b> (pass these in as either get or post in the url)</li>\n";
				echo "\t\t\t\t<ul>\n";
				foreach ($page->param AS $param)
				{
					echo "\t\t\t\t\t<li>";
					
					echo "<i>$param[name]</i> - ";

					if ($param['required'] == '1' || $param['requied'] == 'true')
						echo "(<b>required</b>) ";
					else
						echo "(optional) ";
					
					if ($param['desc'])
						echo "$param[desc]";

					if (is_object($param->option[0]))
					{
						echo "\n\t\t\t\t\t\t<ul>\n";
						echo "\t\t\t\t\t\t\t<li><b>Accepted values:</b></li>\n";
						foreach ($param->option AS $option)
						{
							echo "\t\t\t\t\t\t\t<li><i>$option[value]</i>";
							
							if ($option['desc'])
								echo " - $option[desc]";
							else
								echo " - sort by the $option[value] field";
								
							echo "</li>\n";

						}
						echo "\t\t\t\t\t\t</ul>\n";
					}

					//draw our regex stuff
					$regex = (string)$param['regex'];
					if ($regex)
					{
						echo "\n\t\t\t\t\t\t<ul>\n";
						echo "\t\t\t\t\t\t\t<li>\n";
						echo "<b>Must match the <a href=\"http://www.php.net/pcre\">regular expression</a>:</b><br/>\n";
						echo "<pre>$regex</pre>\n";
						echo "\t\t\t\t\t\t\t</li>\n";
						echo "\t\t\t\t\t\t</ul>\n";
					}
						
					echo "\t\t\t\t\t</li>\n";
				}
				echo "\t\t\t\t</ul>\n";
			}
			
			echo "\t\t\t<li><b>output:</b> ";

			if ($page['type'] == 'xml')
				echo "XML data suitable for parsing.";
			else
				echo "HTML data suitable for display.";
			echo "</li>\n";

			echo "\t\t</ul>\n";
			
			echo "\t</li>\n";
		}
		echo "</ul>";
	}

	/**
	* This function initializes teh main page of the module.  You will probably override it at some point in time.
	*/
	public function initMainPage()
	{
		$this->pageTitle = get_class($this) . " Main";
	}
	
	/**
	* This function draws teh main page.  You should override this for your module if you want a main page.
	*/
	public function drawMainPage()
	{
	}

	/**
	* This function initialializes the module page.
	*/
	public function initModuleXmlPage()
	{
		$this->pageTitle = get_class($this) . " Module - XML Structure";
	
		$this->setTemplate(new XMLTemplate());
	}

	/**
	* This function draws teh module page... which is the xml that describes teh module.
	*/
	public function drawModuleXmlPage()
	{
		echo $this->xml;
	}
	
	/**
	* This function generates the xml for the module.
	* 
	* @return a simplexml object containing the xml of the module
	*/
	public function parseModuleXml()
	{
		if (!$this->xml)
			$this->xml = $this->getModuleXml();

		return simplexml_load_string($this->xml);
	}

	/**
	* This function is used to access the parameters of the object.
	*
	* @param $key the key of the parameter you want to access.
	*
	* @return the value of the parameter... if it exists.  false if it doesnt.
	*/
	function params($key = null)
	{
		if ($key === null)
			return $this->params;
		else if (array_key_exists($key, $this->params))
			return $this->params[$key];
		else
			return false;
	}

	/**
	* Update the value of a parameter for a page.
	*
	* @param $key the key of the paramater you with to update.
	* @param $val the new value of the parameter
	*/
	function setParam($key, $val)
	{
		$this->params[$key] = $val;
	}
	
	/**
	* This function gets the xml of the module in string format.
	*
	* @param an xml string containing the entire module xml.
	*/
	function getModuleXml()
	{
		$xml = "<module name=\"" . get_class($this) . "\" english=\"$this->english\">\n";
		$xml .= $this->getPagesXml();
		$xml .= "\n\t</module>";

		return $xml;
	}

	/**
	* This function gets the string representation of the page xml.  If you are
	* doing any sort of custom pages you will definitely be overriding this
	* function. 
	*
	* @return a string representation of the module's page xml.
	*/
	function getPagesXml()
	{
		global $me;
		
		$xml = $this->getMainPageXml();
		$xml .= $this->getModulePageXml();
		$xml .= $this->getParametersPageXml();
		$xml .= $this->getCreateTablePageXml();

		return $xml;
	}

	/**
	* This function gets the xml for the main page configuration.
	*
	* @return an xml string for the main page.
	*/
	function getMainPageXml()
	{
		return "\t<page name=\"main\" type=\"content\" desc=\"the main page for the module\"/>\n";
	}

	/**
	* This function gets the xml for the module page configuration.
	*
	* @return an xml string for the module page.
	*/
	function getModulePageXml()
	{
		return "\t<page name=\"modulexml\" type=\"xml\" desc=\"get the modules structural xml description\"/>\n";
	}
		
	/**
	* This function gets the xml for the parameters page configuration.
	*
	* @return an xml string for the parameters page.
	*/
	function getParametersPageXml()
	{
		return "\t<page name=\"parameters\" type=\"content\" desc=\"get the pages and parameters for an object\"/>\n";
	}
	
	/**
	* This function gets the xml for the parameters page configuration.
	*
	* @return an xml string for the parameters page.
	*/
	function getCreateTablePageXml()
	{
		return "\t<page name=\"createtable\" type=\"content\" desc=\"get the MySQL syntax to create the needed tables.\"><param name=\"run\"/></page>\n";
	}

	/**
	* This function adds a box to the module.  This is used to do custom
	* sidebars.
	*
	* @param $box an object that is extended from, or is, a BaseBox
	*/
	public function addBox($box)
	{
		$this->sideboxes[] = $box;
	}

	/**
	* This function tells us whether we need to draw a sidebox or not.
	*
	* @return Boolean if we need a sidebar or not. This is based on if we have any boxes to display or not.
	*/
	public function needsSideBar()
	{
		return (bool)count($this->sideboxes);
	}

	/**
	* This function will draw the sidebar, if needed.  It basically loops thru
	* the boxxes that have been added.
	*/
	public function drawSideBar()
	{
		if ($this->needsSideBar())
		{
			echo "\n<div id=\"sidebar\">\n";
			
			foreach ($this->sideboxes AS $box)
				$box->draw();
		
			Util::ieDrawClear();
			echo "\n</div>\n";
			echo "<!-- end sidebar -->\n";
		}
	}

	/**
	* This function adds text or link to the navigation.
	*
	* @param $text text or link to add to the navigation
	*/
	public function addNav($text)
	{
		$this->navigation[] = $text;
	}

	/**
	* Get the navigation
	*
	* @return an array of navigation text.  do what you like with it.
	*/
	public function getNav()
	{
		return $this->navigation;
	}

	/**
	* This draws the navigation as an unordered list wrapped in a div with class 'pageNav'
	*/
	public function drawNavigation()
	{	
		if (count($this->navigation))
		{
			echo "\n<div class=\"pageNav\">\n";
			echo "\t<ul>\n";
			foreach ($this->navigation AS $nav)
				echo "\t\t<li>$nav</li>\n";
			echo "\t</ul>\n";
			Util::ieDrawClear();
			echo "</div>\n";
		}
	}

	/**
	* This function is used to ensure the user is logged in.  If called and a
	* user is logged in, nothing happens.  However, if a user is not logged in,
	* it redirects them to the login page with a variable to return them to the
	* original page once they are logged in.  Very handy function!
	*/
	public function assertLogin()
	{
		global $me;

		if (!$me->id)
		{
			Util::redirect($me->getUrl(array(
				'page' => 'login',
				'redirect' => base64_encode($_SERVER['REQUEST_URI']),
				'status' => 'You must be logged in to access this part of the site.'
			)));
		}

		return $me->id;
	}

	/**
	* Add a css file to the list of needed css files.  Use this if you need some sort of extra css for your page.
	*
	* @param $file the path to the css file that is needed.
	*/
	public function needsCss($file)
	{
		global $_BJ_CSS;
		$_BJ_CSS[] = $file;
		$_BJ_CSS = array_unique($_BJ_CSS);
	}

	/**
	* Draw the markup to include the css files.  You probably wont call this directly... use drawHead() instead.
	*/
	public function drawCss()
	{
		global $_BJ_CSS;
		
		if (count($_BJ_CSS))
			foreach ($_BJ_CSS AS $file)
				echo "\t<link rel=\"stylesheet\" href=\"$file\" type=\"text/css\" />\n";
	}

	/**
	* Add a javascript file to the page.  If you need any special js... use this!
	*
	* @param $file the path to the file to be included.
	*/
	public function needsJs($file)
	{
		global $_BJ_JAVASCRIPT;

		//remove any previous ones..
		if (count($_BJ_JAVASCRIPT))
			foreach ($_BJ_JAVASCRIPT AS $key => $val)
				if ($val == $file)
					unset($_BJ_JAVASCRIPT[$key]);

		//add it in.
		$_BJ_JAVASCRIPT[] = $file;
	}

	/**
	* Draw the markup to include a javascript file. You probably wont use this,
	* instead add drawHead() to your header.
	*/
	public function drawJs()
	{
		global $_BJ_JAVASCRIPT;
		
		if (count($_BJ_JAVASCRIPT))
			foreach ($_BJ_JAVASCRIPT AS $file)
				echo "\t<script type=\"text/javascript\" src=\"$file\"></script>\n";
	}

	/**
	* This adds feed to global feed array.  that makes it so we can call
	* addFeed from modules other than the main module.
	*
	* @param $url the url to link to
	* @param $title the title of the feed
	*/
	public function addFeed($url, $title)
	{
		self::$Feeds[] = $url;
		self::$FeedTitles[] = $title;
	}

	/**
	* This draws teh markup for rss feeds in the header.  You should probably use drawHead instead.
	*/
	public function drawFeeds()
	{
		self::$Feeds = array_unique(self::$Feeds);
		self::$FeedTitles = array_unique(self::$FeedTitles);

		if (count(self::$Feeds))
			foreach (self::$Feeds AS $key => $feed)
				echo "\t<link rel=\"alternate\" type=\"application/rss+xml\" title=\"" . self::$FeedTitles[$key] . "\" href=\"$feed\">\n";
	}

	/**
	* This draws all the markup for needed css, javascript, and any rss feeds.
	* You will most definitely need to add this to your drawHeader() function
	* in your template.
	*/
	public function drawHead()
	{
		$this->drawCss();
		$this->drawJs();
		$this->drawFeeds();
	}

	/**
	* initialize our create table page.  anyone can view... easier for them to
	* setup a table to work with our data.  only admins can run the code,
	* however.
	*/
	public function initCreateTablePage()
	{
		global $me;
		
		$this->pageTitle = 'Create Table Syntax For ' . get_class($this);

		if ($this->params('run'))
		{
			if (!$me->isAdmin())
				throw new PageError('You must be an admin to run the create table query.');
			else
			{
				$this->runCreateTableSql();
				$this->addStatus('Your tables have been created.  Query(s) shown below.');
			}
		}
	}
		
	/**
	* draw our create table page.  show our queries, etc.
	*/
	public function drawCreateTablePage()
	{
		global $me;
		
		if (!$this->params('run') && $me->isAdmin())
			echo "<p>" . $this->getLink(".createtable?run=1", "Run the SQL below.") . "</p>";
		
		echo "<p>" . nl2br($this->getCreateTableSql()) . "</p>";
	}
	
	/**
	* this function actually runs our create table code.  extend this if you
	* need to do any extra queries.	
	*/
	public function runCreateTableSql()
	{
		dbExecute($this->getCreateTableSql());

		//clear our cache!
		CacheBot::delete('BaseJumper:Tables');
	}

	/**
	* this gets the sql statement(s) that create the tables needed for this
	* module.  these should be the tables the module directly accesses.
	*
	* @return teh sql string
	*/
	public function getCreateTableSql()
	{
		return '';
	}


	/**
	* get the data for our cached page!
	*
	* @return the cached data for our page... whatever that may be.
	*/
	public static function getCachedPage($key = null)
	{
		//no key?  lets try and 
		if ($key === null)
			$key = self::getPageCacheKey();
			
		return CacheBot::get($key, self::$pageCacheLife);
	}

	/**
	* save the page as cached.
	*
	* @param $data the page data
	*/
	public static function setCachedPage($data, $key = null)
	{
		//no key?  lets try and 
		if ($key === null)
			$key = self::getPageCacheKey();
		
		return CacheBot::set($key, $data, self::$pageCacheLife);
	}

	/**
	* get us a key that will be unique for the page we're on...
	*
	* @return a key that identifies the page we're on.
	*/
	public static function getPageCacheKey($page = null)
	{
		if ($page === null)
			$page = $_SERVER["REQUEST_URI"];

		return "BaseJumper:autocache:$page";
	}

	/**
	* deletes a page from the cache
	*
	* @param $page a page to delete eg. /module/page
	*/
	public static function deleteCachedPage($page)
	{
		$key = self::getPageCacheKey($page);

		CacheBot::delete($key);
	}

	/**
	* this function does two things:  it generates a key and then uses that key
	* to try the cache.  if it finds it, it shows the cached page and returns
	* false.  otherwise it returns the key for use in caching. it makes caching
	* pages easy.  have one call and an if statement that contains the setCache
	* call.
	*
	* @param $page the page / key you want to cache.
	*
	* @return the key to use in setPageCache()
	*/
	public static function tryPageCache($page = null)
	{
		//get our 
		$key = self::getPageCacheKey($page);
		
		//try to get our cached page.
		$page = self::getCachedPage($key);
		if ($page)
		{
			//send our page and return false
			echo $page;
			return false;
		}

		//nope, give them the key to cache with later.
		return $key;
	}
}
?>
