<?
//get our required stuff
require_once("lib/db.inc.php");

/**
* @ingroup BaseObjects
* @brief BaseObject is a class for dealing with simple objects.
* 
* A class that simplifies loading, editing, deleting, and saving of an object
* linked to a mysql database. Also provides hooks for API data, and easy rss
* creation. Plus, easy integration with comments and tags.
*/
abstract class BaseObject extends MyModule
{
	/**
	 * @b Private.  The name of the database table its linked to.
	 * @private
	 */
	public $tableName;

	/**
	 * @b Private. Array of fields that are dirty.
	 */
	private $dirtyFields = array();

	/**
	 * @b Protected. Internal data array
	 */
	protected $data = array();

	/**
	* this is our creator object... the user that owns us.
	*/
	public $creator;

	/**
	 * @b public. BaseThread extended object that is used for comments.
	 */
	public $comments;

	/**
	 * @b public. Boolean that tells us if we are using comments or not.
	 */
	public $commentsEnabled = false;
	
	/**
	 * @b public. Boolean Do we use and also allow searching by lat/lon (must have 2 fields in DB called lat and lon that are of type double)
	 */
	public $useLatLng = false;

	/**
	 * @b public.  Boolean if we use object caching or not. you should understand the object caching system before you use this.
	 */
	public $useObjectCaching = false;

	/**
	* @b public. Boolean Do we use tags or not?
	*/
	public $useTags = false;

	/**
	* @b public our tags object
	*/
	public $tags;
	
	/**
	* @b public. Boolean of whether the classes has a user_id field for the author.
	*/
	public $hasAuthor = true;

	/**
	 * The id that references the table.
	 */
	private $internalId;

	/**
	* how long we cache the object data in seconds. defaults to a bit over a month (3 million seconds)
	*/
	public static $objectCacheLife = 3000000;
	
	/**
	* an array of fields to include in the fulltext index.
	*/
	public $fullTextFields = array();
	
	/**
	 * Creates a new BaseObject.
	 *
	 * @param $data if $data is an integer id, in which case it will load the data from the database.
	 * If $data is an array, it will load the data from the array to the equivalent properties of the object.
	 *
	 * @param $tableName the name of the table to reference.
	 */
	public function __construct($data, $tableName)
	{
		//do our parent...
		parent::__construct();
		
		//set our table name...
		$this->tableName = $tableName;

		//check our fields.
		$this->checkTable();

		//omg noob... dont forget to load!
		$this->load($data);
			
		//do we want lat/lon?
		if ($this->useLatLng)
		{
			$this->fullTextFields[] = 'city';
			$this->fullTextFields[] = 'state';
		}
	}

	/**
	 * handle all our stuff after we clean up our object
	 */
	public function __destruct()
	{
	}

	/**
	 * This function translates the object to a string.
	 * @return the string value for the funtction
	**/
	public function __toString()
	{
		return $this->getName();
	}

	/**
	 * This function is for getting at the various data internal to the object
	 * @param $name is the name of the field
	 * @return the value from the data array or the id
	 */
	public function __get($name)
	{
		if ($name == 'id')
			return $this->internalId;
		else if ($this->hasField($name))
			return $this->data[$name];
		else
			return $this->$name;
	}

	/**
	 * Function to set data values for the object
	 * @param $name is the name of the field
	 * @param $value is the stored value
	 */
	public function __set($name, $value)
	{
		//if we got an id... we want to set the internal id... if its different
		if ($name == 'id')
		{
			if ($value != $this->internalId)
			{
				//save it and load.
				$this->internalId = $value;
				$this->__construct($value);
			}
		}
		//or do we have the field?
		else if ($this->hasField($name))
		{
			//only update it if they are different
			if ($this->data[$name] != $value || !isset($this->data['name']))
			{
				//mark it as dirty and save it.
				//WTF????  this is such a weird bug.  i dunno
				//why the fuck i have to do it, but without the
				//(string) cast it will cause the value to be
				//&UNKNOWN:0 via vardump, print_r segfaults,
				//and it totally fucks up everything. 
				$this->dirtyFields[$name] = (string)$name;
				$this->data[$name] = $value;
			}
		}
		//nope try and set it directly
		else
			$this->$name = $value;
	}

	/**
	 * This function initializes the edit page.
	 * @param $title the title of the page.
	 * @param $data some data to load the object with.
	 */
	protected function initEditPage($title = null, $data = null)
	{
		//create our object
		if ($data === null)
			$this->id = $this->params('id');
		else
			$this->data = $data;
		
		//set a pretty good default title.
		if ($title === null && $this->id)
			$this->pageTitle = 'Edit ' . ucfirst(strtolower($this->english)) . ' - ' . $this->__toString();
		else if ($this->pagetitle === null && !$this->id)
			$this->pageTitle = 'Add New ' . ucfirst(strtolower($this->english));
		else
			$this->pageTitle = $title;

		if (!$this->canEdit())
			throw new PageError("You do not have access to edit this $this->english.");
	}
		
	/**
	 * This function initializes the delete page.
	 * @param $title the title of the page.
	 */
	protected function initDeletePage($title = null)
	{
		//create our object
		$this->id = $this->params('id');
		
		//set a pretty good default title.
		if ($title === null)
			$title = 'Delete ' . ucfirst(strtolower($this->english));
		$this->pageTitle = $title;
			
		if (!$this->canDelete())
			throw new PageError("You are not authorized to delete this $this->english.");
	}

	/**
	 * This function initializes the view page.
	 * @param $title the title of the page.
	 */
	protected function initViewPage($title = null)
	{
		//create our object
		$this->id = $this->params('id');

		//set a pretty good default title.
		if ($title === null)
			$this->pageTitle = $this->getName();
		else
			$this->pageTitle = $title;
		
		if (!$this->canView())
			throw new PageError("You do not have permission to view this $this->english.");
		
		if ($this->commentsEnabled)
		{
			$this->needsJs('lib/js/comment.js');
			$this->needsJs('lib/js/prototype-1.5.0.js');
		}
	}

	/**
	* This function initializes the json page.
	*/
	public function initJSONPage()
	{
		require_once("lib/json.inc.php");
		
		$this->initViewPage();

		$this->setTemplate(new BlankTemplate());
	}

	/**
	* this function draws the json data.
	*/
	public function drawJSONPage()
	{
		echo JSON::encode($this->getPublicData());
	}
	
	/**
	* this function initializes the php serialized() data page.
	*/
	public function initPHPPage()
	{
		$this->initViewPage();

		$this->setTemplate(new BlankTemplate());
	}

	/**
	* this function draws teh object data as serialized php
	*/
	public function drawPHPPage()
	{
		echo serialize($this->getPublicData());
	}
	
	/**
	* this function initializes the xml serialized() data page.
	*/
	public function initXMLPage()
	{
		$this->initViewPage();

		$this->setTemplate(new XMLTemplate());
	}

	/**
	* this function draws teh object data as serialized xml
	*/
	public function drawXMLPage()
	{
		$options = array(
			"indent"         => "\t",
			"linebreak"      => "\n",
			"typeHints"      => false,
			"addDecl"        => false,
			"rootName" => get_class($this)
		);
		//setup our serializer
		$serial = new XML_Serializer($options);
		
		//serialize our data.
		$data = $this->getPublicData($this->params());
		if ($serial->serialize($data))
			echo $serial->getSerializedData();
	}

	/**
	* This function gets an array of publicly available data for the object.
	* it is used by json, php, and other pages that export data.
	*
	* @return an array of public data.
	*/
	public function getPublicData()
	{
		global $me;
		
		$data = array();

		if ($this->id)
		{
			$data['id'] = $this->id;

			if ($this->hasAuthor && $this->creator instanceOf BaseUser)
			{
				$data['user_id'] = $this->user_id;
				$data['username'] = $this->creator->getName();
			}
			if ($this->hasField('content_id'))
				$data['content_id'] = $this->content_id;
			if ($this->hasField('parent_id'))
				$data['parent_id'] = $this->parent_id;
			if ($this->hasField('add_date'))
				$data['add_date'] = $me->formatDateTime($this->add_date);
			if ($this->hasField('edit_date'))
				$data['edit_date'] = $me->formatDateTime($this->edit_date);
			if ($this->useLatLng)
			{
				$data['lat'] = $this->lat;
				$data['lon'] = $this->lon;
			}

			if ($this->hasField('comment_count'))
				$data['comment_count'] = $this->comment_count;

			if ($this->canView())
				$data['viewLink'] = $this->getUrl(".view?id=$this->id", null, true);
			
			if ($this->canEdit())
				$data['editLink'] = $this->getUrl(".edit?id=$this->id", null, true);
			
			if ($this->canDelete())
				$data['deleteLink'] = $this->getUrl(".delete?id=$this->id", null, true);
		}
		else
			$data['error'] = "That $this->english doesn't exist.";

		return $data;
	}

	/**
	* this function generates the rss item for each base object.  you'll need to override this to implement rss feeds for your objects.
	*
	* @return a FeedItem from the feedcreateor library.
	*/
	public function getRssItem()
	{
		$item = new FeedItem();
		$item->title = 'Error';
		$item->description = 'You must define ' . get_class($this) . "::getRssItem() to enable rss support. Generally all you have to set are title and description.";
		$item->link = $this->getUrl(".view?id=$this->id", null, true);
		$item->date = strtotime($this->edit_date);
		$item->source = "http://" . Config::get('site_hostname');
		$item->descriptionHtmlSyndicated = true;

		//author?
		if ($this->hasAuthor)
		{
			$item->author = $this->creator->getName();
		}

		return $item;
	}

	/**
	 * This function handles loading the data into the object.
	 *
	 * @todo this will change in v2.2 to load the data from cache or look it up from the db.
	 
	 * @param $data either an id of an object or an array of data that
	 * represents that object.
	 */
	public function load($data, $deep = true)
	{
		//did we get an array of data to set?
		if (is_array($data))
			$this->setDataFromCache($data, $deep);
		//nope, maybe its an id for the database...
		else if ($data)
			$this->loadData($data, $deep);
	}

	/**
	* load our objects data.
	*
	* @depreciated this will be changed around in v2.2
	* @todo remove and put comments / tags into load.  also make a loadfromdb function
	*/
	protected function loadData($id, $deep = true)
	{
		//do we have an id?
		if ($id)
		{
			//set our id first.
			$this->internalId = $id;
			
			//try our cache.. if it works, then we're good.
			if (!$this->loadCacheData($deep))
			{
				$this->lookupData($deep);

				if ($deep && $this->useObjectCaching)
					$this->setCache();
			}
		}
		
		//create our thread.
		if ($this->commentsEnabled)
			$this->threadFactory();

		//create our tags.
		if ($this->useTags)
			$this->tagFactory();
	}

	/**
	* get our data from the db.
	*
	* @todo update this whole data loading process to be much smoother
	*/
	protected function lookupData($deep = true)
	{
		//nope, load it normally.
		$data = $this->getData(true);
		if (count($data) && is_array($data))
			foreach ($data AS $key => $val)
				if ($key != 'id')
					$this->data[$key] = $val;
		
		//what about our creator?
		if ($deep)
		{
			if ($this->hasAuthor)
			{
				$this->creator = new User();
				$this->creator->load($this->user_id, false);
			}
		}
	}

	/**
	* load data from cache
	*
	* @depreciated will be removed in v2.2.
	*/
	protected function loadCacheData($deep = true)
	{
		//is it enabled?
		if ($this->useObjectCaching)
		{
			//get our cache data... is it there?
			$data = $this->getCache();
			if ($data)
			{
				//load it, and we're good.
				$this->load($data, $deep);
				return true;
			}
		}

		return false;
	}

	/**
	* This function creates our thread.  You may want to manually create your thread.
	*
	* @return the thread class.
	*/
	public function threadFactory()
	{
		$threadClass = get_class($this) . "Thread"; 
		
		if (class_exists($threadClass))
			$this->comments = new $threadClass($this->id);
	}
	
	/**
	* This function creates our tags class.  You may want to manually create your own.
	*/
	public function tagFactory()
	{
		$tagClass = get_class($this) . "Tags"; 
		
		if (class_exists($tagClass))
			$this->tags = new $tagClass($this->id);
	}
		
	/**
	* This function adds a privacy field to your edit form.  Does nothing by default.  Override if necessary.
	*/
	public function addPrivacyField()
	{
	}
	
	/**
	 * This function handles saving the object.
	 *
	 * @return true on success, false on failure.
	 */
	public function save()
	{
		//we should do any cleanup if possible
		if ($this->isDirty())
			$this->clean();

		//get our comment count
		$this->lookupCommentCount();
	
		//save it to wherever
		$data = $this->saveData();

		//save our cache.
		if ($this->useObjectCaching)
			$this->deleteCache();

		//delete our cached view page.
		self::deleteCachedPage('/' . strtolower(get_class($this)) . "/view/id/$this->id");
		
		return $data;
	}

	/**
	* lookup our comment count information
	*
	* @return the number of comments on it
	*/
	public function lookupCommentCount()
	{
		//update our comment count.
		if ($this->commentsEnabled && $this->hasField('comment_count') && $this->comments instanceOf BaseThread)
			$this->comment_count = $this->comments->searchCount(array(
				'content_id' => $this->id
			), false);
		
		return $this->comment_count;
	}

	/**
	 * This function handles any validation/cleaning of our data.
	 */
	public function clean()
	{
		foreach ($this->dirtyFields AS $field)
			$this->cleanField($field);
	}

	/**
	* Clean a field's data.  called from clean()
	*
	* @param $field the name of the field to clean.
	*/
	public function cleanField($field)
	{
		//by default... we dont clean anything
	}

	/**
	* Tells us if the object is dirty or not.  Used to determine if we clean and/or if we need to save.
	*/
	public function isDirty()
	{
		return (bool)count($this->dirtyFields);
	}

	/**
	 * This function handles deleting our object.
	 * 
	 * @return ture on success, false on failure.
	 */
	public function delete()
	{
		//if we're using comments... delete them!
		if ($this->commentsEnabled && $this->comments instanceOf BaseThread)
			$this->comments->delete();
		
		//if we're using comments... delete them!
		if ($this->useTags && $this->tags instanceOf BaseTags)
			$this->tags->delete();
		
		//delete our cache.
		if ($this->useObjectCaching)
			$this->deleteCache();
		
		//delete our cached view page.
		self::deleteCachedPage('/' + strtolower(get_class($this)) . "/view/id/$this->id");

		return $this->deleteDb();
	}

	/**
	 * This function does all the stuff we need to be done on the logic page.
	 */
	public function drawEditPage()
	{
		$this->editPageLogic();
	}

	/**
	* If your object has city, state, zip, and street fields, and also lat and
	* lon fields, this function will automatically look these up using Yahoo's
	* geocoding API.
	*/
	function lookupLatLon()
	{
		$geo = new Geocode();
		
		//get our data
		$csz['city'] = $this->city;
		$csz['state'] = $this->state;
		$csz['zip'] = $this->zip;
		$csz['street'] = $this->street;
		
		//do a search
		$result = $geo->search($csz);
		if ($result)
		{
			$this->lat = $result['lat'];
			$this->lon = $result['lon'];

			if (!$this->city)
				$this->city = $result['city'];
			if (!$this->state)
				$this->state = $result['state'];
			if (!$this->zip)
				$this->zip = $result['zip'];
		}
	}
	
	/**
	 * This function handles the view/detail page.  It prints out each field
	 * line by line.  You'll prolly want to override this guy =)
	 *
	 * @param $ignore an array of fields to ignore.
	*/
	public function drawViewPage($ignore = array())
	{
		//loop thru and print the non ignored ones
		echo "<table width=\"100%\">\n";
		if (count($this->data))
		{
			foreach ($this->data AS $key => $val)
			{
				if (!in_array($key, $ignore))
					echo "\t<tr>\n\t\t<td>$key</td>\n\t\t<td>$val</td>\n\t</tr>\n";
			}
		}
		echo "</table>\n";
	}

	/**
	 * This function handles all the delete page stuff.  Shows the page and handles the post.
	 *
	 * @param $preText this is the text to be displayed on the delete page.
	 * @param $button the text for the submit button on the delete page.
	 * @param $postText either the text to be displayed on the result page, or may be a url. Default: $this->baseUrl
	 * @param $isUrl if true, then it will redirect to the url in $postText.  Default: true
	 */
	public function drawDeletePage($preText = null, $button = null, $postText = null, $isUrl = true)
	{
		if (!isset($_POST['submit']))
			$this->deleteForm($preText, $button);
		else
			$this->deletePost($postText, $isUrl);
	}

	/**
	 * This function determines if we can edit the object.
	 * @return true if they can edit, false if they cant.
	 */
	function canEdit()
	{
		global $me;
		
		//if its a real item, they must be the owner or admin.
		if (($this->user_id == $me->id && $me->id) || $me->isAdmin())
			return true;
		//otherwise, its a new one and they just have to be logged in.
		else if (!$this->id && $me->id)
			return true;
		else
			return false;
	}

	/**
	 * This function determines if we can delete the object
	 * @return true if they can delete, false if they cant
	 */
	function canDelete()
	{
		global $me;
		
		if (($this->user_id == $me->id && $me->id) || $me->isAdmin() && $this->id)
			return true;
		else
			return false;
	}
	
	/**
	* this function draws a header row for searches.  by default doesnt do anything.
	*/
	function drawHeaderRow()
	{ 

	}
	
	/**
	* Get the name of an object... used by base-user, and many other things as
	* a shortcut for drawing the view link or just the name
	*
	* @param $link Boolean of whether or not we want to draw the link
	*
	* @return the name of the object
	*/
	function getName($link = false, $text = '')
	{
		$text = trim($text);
		if (!$text)
			$text = '???';

		if ($link)
			return $this->getLink(".view?id=$this->id", $text);
		return $text;
	}

	/**
	* Called by an objectmanager to draw the row for the object.  Does some
	* default stuff... but you'll most likely want to override it.
	*/
	function drawRow()
	{
		echo "<h3>" . $this->getName(true) . "</h3>";
		echo "<p>By: ";
		echo $this->creator->getName(true) . "<br/>"; 
		echo Time::formatRelative($this->add_date) . "<br/>";
		
		if ($this->commentsEnabled)
			echo $this->getCommentsLink($this->id);
		"</p>\n";
	}

	/**
	* get a link to the object's comment thread
	*
	* usually threads are on the view page, and they have an anchor of
	* "comments" from BaseThread.  It will also pluralize the comments word
	*
	* @return a href tag with the link
	*/
	public function getCommentsLink()
	{
		return $this->getLink(".view?id=$this->id#comments", Util::pluralize('comment', $this->comment_count, true));
	}

	/**
	* called by object manager to draw the header line for a bunch of objects.
	*/
	function drawHeaderLine()
	{
	}

	/**
	* called by object manager to draw one line of info about the object.
	* you'll need to override.
	*/
	function drawLine()
	{
	}

	/**
	* A shortcut function that draws basic info about the owner and when it was posted/updated.
	*/
	function drawPostedOwner()
	{
		global $me;
	
		echo "<h3>Created: " . Util::formatTimeRelative($this->add_date) . " by " . $this->creator->getName(true) . "</h3>\n";
		if ($this->add_date != $this->edit_date)
			echo "<h4>Last updated: " . Util::formatTimeRelative($this->edit_date) . "</h4>\n";
	}
	
	/**
	* This function gets the page xml for the object.  Adds edit, delete, view, json, and php. Override to customize.
	*
	* @return the object page xml string.
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= $this->getEditPageXml();
		$xml .= $this->getViewPageXml();
		$xml .= $this->getDeletePageXml();
		$xml .= $this->getJSONPageXml();
		$xml .= $this->getPHPPageXml();
		$xml .= $this->getXMLPageXml();

		return $xml;
	}

	/**
	* this function gets the view page xml.
	*
	* @return the view page xml string.
	*/
	protected function getViewPageXml()
	{
		return <<<XML
			<page name="view" type="content" desc="view a content">
				<param name="id" desc="id of the content" required="1" type="int"/>
			</page>
XML;
	}
	
	/**
	* this function gets the delete page xml.
	*
	* @return the delete page xml string.
	*/
	protected function getDeletePageXml()
	{
		return <<<XML
			<page name="delete" type="content" desc="delete an object">
				<param name="id" required="1" desc="the id of the module" type="int"/>
			</page>
XML;
	}

	/**
	* this function gets the json page xml.
	*
	* @return the json page xml string.
	*/
	protected function getJSONPageXml()
	{
		return <<<XML
			<page name="json" desc="fetch the json representation of this object">
				<param name="id" desc="id of the content" required="1" type="int"/>
			</page>
XML;
	}

	/**
	* this function gets the php page xml.
	*
	* @return the php page xml string.
	*/
	protected function getPHPPageXml()
	{
		return <<<XML
			<page name="php" desc="fetch the php serialize() representation of this object">
				<param name="id" desc="id of the content" required="1" type="int"/>
			</page>
XML;
	}
	
	/**
	* this function gets the xml page xml.
	*
	* @return the xml page xml string.
	*/
	protected function getXMLPageXml()
	{
		return <<<XML
			<page name="xml" desc="fetch the XML representation of this object">
				<param name="id" desc="id of the content" required="1" type="int"/>
			</page>
XML;
	}
	
	/**
	* this function gets the edit page xml.
	*
	* @return the edit page xml string.
	*/
	protected function getEditPageXml()
	{
		$var =  "\n\t<page name=\"edit\" desc=\"create or edit a(n) $this->english\">\n";
		$var .= "\t\t<param name=\"id\" desc=\"the id of the $this->english.  if not supplied, a new record will be created.\" type=\"int\"/>\n";
		$var .= $this->getEditPageParamXml();
		$var .= "\n\t</page>\n";
		
		return $var;
	}

	/**
	* this gets all the parameters for the edit page.
	*
	* @return the xml string for edit page parameters.
	*/
	protected function getEditPageParamXml()
	{
		$var = '';
	
	/*
		this never even really comes into play.... we need to make it someday

		foreach ($this->getDbFields(array(
			'id', 'user_id', 'add_date', 'edit_date', 'privacy', 'status', 'parent_id'
		)) AS $key => $val)
			$var .= "\t\t<param name=\"$key\" />\n";
	*/

		return $var;
	}

	/**
	 * This function determines if a user can view the object or not
	 * @return true if they can view it, false if they cant.
	 */
	public function canView()
	{
		//you cant view a blank content
		if (!$this->id)
			return false;
		return true;
	}

	/**
	 * This function gets an associative array of the object's members most
	 * commonly this will be from a db, but you never know.
	 *
	 * @return an associative array of the objects data.
	 */
	protected function getData($useDb = true)
	{
		if ($useDb)
			return $this->getDbData();
		else
		{
			$data = $this->data;
			$data['id'] = $this->id;

			return $data;
		}
	}

	/**
	 * This function sets all the data for the object.
	 *
	 * @param $data an associative array of data.  The keys must match up with
	 * the object properties.
	 * @param $ignore the fields to ignore and not set
	 */
	//equivalent object members to that (if they exist)
	public function setData($data, $ignore = null)
	{
		//make sure we have an array here =)
		if (is_array($data))
		{
			if (!is_array($ignore))
				$ignore = array($ignore);

			//okay loop thur our data...
			foreach ($data AS $key => $val)
			{
				//if we ignore it... continue
				if (in_array($key, $ignore))
					continue;

				//make sure this key exists for us....
				if ($key === 'id')
					$this->internalId = (int)$val;
				else if ($this->hasField($key))
					$this->data[$key] = $val;
			}
			
			return true;
		}
		else
			return false;

	}

	/**
	 * This function handles saving the data to wherever.
	 *
	 * @return true on success, false on failure
	 */
	protected function saveData()
	{
		return $this->saveDb();
	}

	/**
	 * This function gets all the member information from a database.
	 *
	 * @return an associative array of data or false on failure.
	 */
	private function getDbData()
	{
		//make sure we have an id....
		if ($this->id)
		{
			$result = dbFetchAssoc(dbQuery("
				SELECT *
				FROM $this->tableName
				WHERE id = '$this->id'
			"));

			if (is_array($result))
				return $result;
		}
		
		return false;
	}

	/**
	 * This function saves the object back to the database.  It is a bit
	 * trickier.  It will smartly insert or update depending on if there is an
	 * id or not.  It also only saves the properties of the object that are
	 * named the same as the table fields, all automatically.
	 */
	private function saveDb()
	{
		//format our sql statements w/ the valid fields
		$fields = array();
		$columns = $this->getDbFields();
		
		//loop thru all our dirty fields.
		foreach ($this->dirtyFields AS $key)
		{
			//get our value.
			if ($columns[$key] && isset($this->data[$key]) && $key != 'id')
			{
				$val = $this->data[$key];

				//slashes replacement..
				$val = str_replace("\\\\", "\\", $val);
				$val = str_replace("\'", "'", $val);
				$val = str_replace("\\\"", "\"", $val);

				//add it if we have it...
				$fields[] = "`$key` = '" . addslashes($val) . "'";
			}
		}
		
		//update if we have an id....
		if (count($fields))
		{
			//now make our array
			$sqlFields = implode(",\n", $fields) . "\n";
			
			//update it?
			if ($this->id)
			{
				$sql  = "UPDATE $this->tableName SET\n";
				$sql .= $sqlFields;
				$sql .= "WHERE id = '$this->id'\n";
				$sql .= "LIMIT 1";

				dbExecute($sql);
			}
			//otherwise insert it...
			else
			{
				$sql  = "INSERT INTO $this->tableName SET\n";
				$sql .= $sqlFields;

				$this->id = dbExecute($sql, true);
			}
		}
	}

	/**
	 * This function deletes the object from the database.
	 *
	 * @return true on success, false on failure
	 */
	private function deleteDb()
	{
		//do we have an id?
		if ($this->id)
		{
			dbExecute("
				DELETE FROM $this->tableName
				WHERE id = '$this->id'
			");

			return true;
		}
		return false;
	}

	/**
	 * This function sets up the form and such.  By default it creates a basic
	 * form based on your table.
	 *
	 * @return teh edit form itself.
	 */
	protected function editFormInit()
	{
		//create our form.
		$form = $this->editFormCreate();
		
		//add our fields
		$this->editFormAddFields($form);

		return $form;
	}

	/**
	* This function creates the edit form object... with no fields.
	*
	* @return the form to add objects to
	*/
	protected function editFormCreate()
	{
		//obviously start the form
		$form = new Form();
		$form->action = $this->getUrl(".edit?id=$this->id");
	
		return $form;
	}
	
	/**
	* This function adds the fields to the edit form.  Override this if you want access.
	*
	* @param $form the form to add fields to
	*/
	protected function editFormAddFields($form)
	{
	}

	/**
	 * This function handles the logic of the edit page.
	 */
	protected function editPageLogic()
	{
		//do we have a form?
		$form = $this->editFormInit();
		
		//first we want to check if its submitted 
		if ($form->isSubmitted())
		{
			//validate our form.
			$this->editFormValidate($form);
			
			//then send it off to the handler
			if (!$form->hasError())
			{
				//send them to success...
				$this->editPagePostSaveSync($form);
				$this->editPagePostSuccess($form);
			}
		}
		//no submit... load our initial data...
		else
			$this->editFormLoad($form);
		
		//finally draw the form if needed
		if ($form->needsDrawn())
			$this->editFormDraw($form);
	}

	/**
	* This is where you put any validation code.
	*
	* @param $form teh form to validate
	*/
	protected function editFormValidate($form)
	{
		//do our validation
		$form->validate();
	}

	/**
	 * This function loads our data from the object into the form.
	 *
	 * @param $form teh form to load data into
	 */
	protected function editFormLoad($form)
	{
		if (!$form->isSubmitted())
			$form->setData($this->data);
	}

	/**
	* load some 'dirty' data... that is data that is potentially not the same
	* data and should thusly be saved eventually.
	*
	* @param $data a keyed array of data.
	*/
	public function loadDirtyData($data)
	{
		//load our data back from the form...
		$columns = $this->getDbFields();
		foreach ($columns AS $key => $val)
		{
			//if we got it... load it.
			if (array_key_exists($key, $data))
				$this->$key = $data[$key];
		}
	}

	/**
	* this function gets an array of information on the fields... useful for
	* automatic stuff.
	*
	* @return an array of info about the fields identical to DESCRIBE $table
	*/
	public function getDbFields()
	{
		return BaseJumper::getDbTable($this->tableName);
	}

	/**
	 * This function saves our data after a post and sync's with the object.
	 *
	 * @param $form teh form to get data from
	 */
	protected function editPagePostSaveSync($form)
	{
		global $me;
		
		//load our data back from the form...
		$columns = $this->getDbFields();
		foreach ($columns AS $key => $data)
		{
			if (!$this->id && $key == 'add_date')
				$this->$key = date('Y-m-d H:i:s');
			else if ($key == 'edit_date')
				$this->$key = date('Y-m-d H:i:s');
			else if ($key == 'user_id' && !$this->id)
					$this->$key = $me->id;
			else if ($form->hasField($key))
				$this->$key = $form->getData($key);
		}
		
		//has zip?
		if ($form->hasField('citystzip') && $this->hasField('city') && $this->hasField('state') && $this->hasField('zip'))
		{
			$csz = $form->getData('citystzip');
			$this->city = $csz['city'];
			$this->state = $csz['state'];
			$this->zip = $csz['zip'];
		}

		//do we do location lookup?
		if ($this->useLatLng)
			$this->lookupLatLon();

		//save the info
		$this->save();

		//save our tags..
		if ($this->useTags)
			$this->updateTagsForm($form);
	}

	/**
	 * This function is called on a successful post.  It saves the object then
	 * redirects you back to the homepage.
	 */
	protected function editPagePostSuccess($form)
	{
		Util::redirect($this->getUrl(".view?id=$this->id"));
	}

	/**
	 * This function draws the form.
	 *
	 * @param $form teh form to draw
	 */
	protected function editFormDraw($form)
	{
		$form->drawAll();
	}

	/**
	 * This function draws the delete form.
	 *
	 * @param $body the text to display for the form.
	 * @param $button the text for the submit button.
	 */
	protected function deleteForm($body = null, $button = null)
	{
		//set a default string
		if ($body === null)
			$body = "Are you sure you want to delete this $this->english?";

		if ($button === null)
			$button = 'Delete ' . ucfirst($this->english);
				
		echo "<form method=\"POST\" action=\"$_SERVER[REQUEST_URI]\">\n";
		echo "<input type=\"hidden\" name=\"submit\" value=\"1\">\n";
		echo "<table>";
		echo "<tr><td>$body</td></tr>\n";
		echo "<tr><td align=\"center\"><input type=\"submit\" value=\"$button\"></td></tr>\n";
		echo "</table>";
		echo "</form>";
	}

	/**
	 * This function handles the delete page post.  It deletes the object then
	 * either shows the result or redirects.
	 * @param $body either the text to display or a link.
	 * @param $isUrl true if it is a url.
	 */
	protected function deletePost($body = null, $isUrl = true)
	{
		$this->delete();
		
		if ($isUrl)
		{
			if ($body === null)
				Util::redirect($this->baseUrl);
			else
				Util::redirect($body);
		}
		else
			echo $body;
	}

	/**
	 * This function checks our table to make sure it exists
	 */
	public function checkTable($skip = array())
	{
		if (!BaseJumper::dbHasTable($this->tableName))
			$this->addStatus(
				"Oops, table <b>$this->tableName</b> doesn't exist.  You may
				want to visit the " .  $this->getLink(".createtable", "create
				table") . " page to create it.<br/>Until then, the <b>" .
				get_class($this) . "</b> module may not work correctly."
			);
	}

	/**
	 * This function determines admin status.
	 * @param $acl the acl to authenticate against.
	 * @param $errorFunc the error function to call.  this forces authentication.
	 * @return true if they are admin, false if not.
	 */
	public function isAdmin()
	{
		global $me;
		return $me->isAdmin();
	}
	
	/**
	* This function tells you if an object has a field or not.
	*
	* @param $key the name of the field.
	*
	* @return Boolean if field exists.
	*/
	public function hasField($key)
	{
		if ($this->tableName)
			return BaseJumper::tableHasField($this->tableName, $key);
	
		return false;
	}

	/**
	* this function gets teh create table syntax.
	*
	* @return the string to create the table.
	*/
	public function getCreateTableSql()
	{
		$fields = implode(",\n", $this->getCreateFieldsArray());
		$indexes = implode(",\n", $this->getCreateIndexesArray());

		//add in our fulltext index.
		if (count($this->fullTextFields))
			$indexes .= ",\n FULLTEXT(" . implode(",", $this->fullTextFields) . ")";
		
		return "CREATE TABLE IF NOT EXISTS $this->tableName\n(\n$fields,\n$indexes\n);";
	}

	/**
	* get an array of fields for our object table. extend this to add fields to
	* the table. or remove fields from table
	*
	* @return the keyed array of field name => create syntax
	*/
	public function getCreateFieldsArray()
	{
		$fields = array(
			"id" => "id INT(11) not null auto_increment",
			"add_date" => "add_date datetime not null",
			"edit_date" => "edit_date datetime not null"
		);
			
		if ($this->hasAuthor)
			$fields['user_id'] = "user_id INT(11) default 0 not null";
		
		if ($this->useLatLng)
		{
			$fields['lat'] = "lat double default 0 not null";
			$fields['lon'] = "lon double default 0 not null";
			$fields['street'] = "street varchar(255) default '' not null";
			$fields['city'] = "city varchar(128) default '' not null";
			$fields['state'] = "state char(2) default '' not null";
			$fields['zip'] = "zip char(5) default '' not null";
		}

		if ($this->commentsEnabled)
			$fields['comment_count'] = "comment_count INT(11) default 0 not null";

		return $fields;
	}

	/**
	* get an array of keys for our object table. extend this to add keys to
	* the table. or remove keys from table
	*
	* @return the keyed array of index name => create syntax
	*/
	public function getCreateIndexesArray()
	{
		$fields = array();
		$fields['id'] = "PRIMARY KEY (id)";
		$fields['add_date'] = "KEY (add_date)";
		$fields['edit_date'] = "KEY (edit_date)";
		
		if ($this->hasAuthor)
			$fields['user_id'] = "KEY (user_id)";

		if ($this->useLatLng)
			$fields['latlon'] = "KEY (lat, lon)";
		
		return $fields;
	}

	/**
	* this function creates our key to use with caching.  no need to override
	*
	* @return a key used with CacheBot to cache the object.
	*/
	public function getCacheKey($id = null)
	{
		if ($id === null)
			$id = $this->id;

		return "BaseJumper:object:" . get_class($this) . ":" . $id;
	}

	/**
	* this is the funciton that gets the data we need saved to cache.  by
	* default it saves our data, and will save the comments or tags objects if
	* needed. its recommended to extend this to add data that you'd like cached
	* by the object
	*
	* @return an array of data to cache
	*/
	protected function getDataToCache($deep = true)
	{
		$data = array();
	
		//obviously we want our data.
		$data['id'] = $this->id;
		$data['data'] = $this->data;

/*		//save our comments object..
		if ($this->commentsEnabled)
		{
			$this->threadFactory();
			$data['comments'] = $this->comments;
		}
		
		//what about tags?
		if ($this->useTags)
		{
			$this->tagFactory();
			$data['tags'] = $this->tags;
		}
*/
		if ($deep)
		{
			if ($this->hasAuthor && $this->creator instanceOf BaseUser)
				$data['creator'] = $this->creator->getDataToCache(false);
		}

		return $data;
	}

	/**
	* this function sets the data in the object from the data we retrieved from
	* the cache.  it takes the data from the array and puts it in the object.
	* you'll want to override this one if you added custome data in
	* getDataToCache() and load it into the object.
	*
	* @param $data the data we got from teh cache
	*/
	protected function setDataFromCache($data, $deep = true)
	{
		if (is_array($data))
		{
			//get our id back
			$this->internalId = $data['id'];

			//obviously we want to load our data.
			$this->data = $data['data'];
		
/*			//save our comments object..
			if ($this->commentsEnabled)
				$this->comments = $data['comments'];
			
			//what about tags?
			if ($this->useTags)
				$this->tags = $data['tags'];
*/

			//do we set deep data?
			if ($deep)
			{
				if ($this->hasAuthor)
				{
					$this->creator = new User();
					$this->creator->load($data['creator'], false);
				}
			}
		}
	}

	/**
	* this function gets our data from the cache. no need to override
	*/
	public function getCache()
	{
		return CacheBot::get($this->getCacheKey(), self::$objectCacheLife);
	}

	/**
	* this function saves our data to the cache. no need to override
	*/
	public function setCache()
	{
		return CacheBot::set($this->getCacheKey(), $this->getDataToCache(), self::$objectCacheLife);
	}

	/**
	* this funciton deletes our data from teh cache. no need to override
	*/
	public function deleteCache()
	{
		return CacheBot::delete($this->getCacheKey());
	}
}
?>
