<?
/**
* @ingroup BaseModules
* @brief BaseTags is an object dealing with tags ala Web 2.0.
*
* A class that simplifies the management, adding/editing of tags, and other things dealing with tagging of items.
*/
abstract class BaseTags extends MyModule
{
	/**
	* @b protected  this is the table where we store our tags.
	*/
	protected $tagTable;

	/**
	* @b protected this is the table we use to store relations between tags and objects.
	*/
	protected $joinerTable;
	
	/**
	* plural form of the word of what the tags are...
	*/
	public $plural;

	/**
	* the id of the object we're associating with.
	*/
	public $objectId = 0;

	/**
	* do we cache our array of popular tags?
	*/
	public static $cachePopularTags = true;

	/**
	* how long do we cache our popular tags for?
	*/
	public static $popularTagsCacheLife = 3600;
	
	/**
	* do we cache our array of related tags?
	*/
	public static $cacheRelatedTags = true;

	/**
	* how long do we cache our related tags for?
	*/
	public static $relatedTagsCacheLife = 3600;

	/**
	* our styles for the weighted tags page
	*/
	public static $weightStyles = array(
		'font-size: 35px;',
		'font-size: 25px;',
		'font-size: 20px;', 
		'font-side: 16px;',
		'font-size: 11px;'
	);

	/**
	* our constructor
	*
	* @param $id the id of the BaseObject we're associating with.
	* @param $tagTable the table to store our tags in.
	* @param $joinerTable optional table to join with whatever object we're using.
	*/
	public function __construct($id, $tagTable, $joinerTable)
	{
		$this->english = 'tag';
		$this->plural = 'tags';
		
		$this->tagTable = $tagTable;
		$this->joinerTable = $joinerTable;
		$this->objectId = (int)$id;
	} 

	/**
	* get our xml... see BaseModule::getPagesXml() for more info.
	*
	* @return the xml page string.
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();
		$xml .= <<<XML
			<page name="view">
				<param name="tag" regex="/^\w*$/" required="1" />
			</page>
XML;
		return $xml;
	 }

	/**
	* this is our view page... view info on a tag.
	*/
	public function initViewPage()
	{
		$tag = $this->params('tag');

		if (!$this->findTag($tag))
			throw new PageError("The tag $tag does not exist.");

		$this->pageTitle = self::prettyTag($tag);

		//we wanna cache the view page
		$this->doCachePage = true;
	}

	/**
	* here we draw it.  provided by default is the list of related tags.
	*/
	public function drawViewPage()
	{
		$this->drawRelatedTags();
	}

	/**
	* draws the tags related to a tag
	*/
	public function drawRelatedTags()
	{
		echo "<h3>Related " . ucfirst($this->plural) . "</h3>\n";
		echo "<p>" . $this->getRelatedTagsList($this->params('tag')) . "</p>";
	}

	/**
	* our main page is a cloud of all the cool tags we have.
	*/
	public function initMainPage()
	{
		$this->pageTitle = 'Popular ' . ucfirst($this->plural);

		$this->doCachePage = true;
	}

	/**
	* this draws a paragraph of the popular tags.
	*/
	public function drawMainPage()
	{
		echo "<p>" . $this->getWeightedPopularTagsList() . "</p>";
	}

	/**
	* this function cleans a tag for insertion into a database. spaces and
	* whitespace will be replaced with underscores, and any non-alphanumeric
	* characters will be removed.
	*
	* @param the tag to be cleaned.
	*
	* @return the tag cleaned
	*/
	public static function cleanTag($tag)
	{
		//trip it!
		$tag = trim($tag);

		//make all whitespace be underscores.
		$tag = preg_replace("/\s/", "_", $tag);
		
		//anything not a 'word' character (alphanumeric, underscore) gets removed.
		$tag = preg_replace("/\W/", "", $tag);

		return $tag;
	}

	/**
	* this function takes a database tag and makes it pretty.  aka replaces
	* underscores with spaces
	*
	* @param $tag the tag to format
	* @return the prettified tag
	*/
	public static function prettyTag($tag)
	{
		return str_replace("_", " ", $tag);
	}

	/**
	* this function adds the tag to the database.  if it exists, then the id is
	* returned.
	*
	* @param $tag the tag to be added
	*
	* @return the id of the tag.
	*/
	public function addTag($tag)
	{
		//clean it.
		$tag = self::cleanTag($tag);

		//look for it.
		$id = $this->findTag($tag);
		
		//if we didnt get it...
		if (!$id)
			$id = $this->insertTag($tag);

		//give them the id.
		return $id;
	}

	/**
	* this function inserts the tag into the database.  it does the actual insertion.
	*
	* @param $tag the tag to be inserted.  make sure its clean before its called.
	* @return the id of the new tag.
	*/
	protected function insertTag($tag)
	{
		$tag = self::cleanTag($tag);

		//insert it into the db.
		$id = dbExecute("
			INSERT INTO $this->tagTable
				(tag)
			VALUES
				('$tag')
		", true);

		return $id;
	}

	/**
	* this function deletes a tag from the database
	*
	* @param the tag to delete
	*
	* @return the number of rows affected
	*/
	public function deleteTag($tag)
	{
		//clean it...
		$tag = self::cleanTag($tag);

		//delete it.
		$rows = dbExecute("
			DELETE FROM $this->tagTable
			WHERE tag = '$tag'
			LIMIT 1
		", -1);

		return $rows;
	}

	/**
	* here we are looking to see if a tag exists.
	*
	* @param the tag to search for.
	*
	* @return the id of the tag if found, false if not.
	*/
	public function findTag($tag)
	{
		//clean it...
		$tag = self::cleanTag($tag);
	
		//search it.
		$rs = dbQuery("
			SELECT id
			FROM $this->tagTable
			WHERE tag = '$tag'
		");

		//get it.
		if (dbGetNumRows($rs))
		{
			$ar = dbFetchAssoc($rs);
			return $ar['id'];
		}
			
		//oops, no id!
		return false;
	}

	/**
	* this function gets a list of tags that a our object is associated with
	*
	* @return an array of tag strings
	*/
	public function getTags()
	{
		$tags = array();

		//only do it if we have an id.
		if ($this->objectId)
		{
			$where = $this->getTagsWhere();
			
			//do our query.
			$rs = dbQuery("
				SELECT t.id, t.tag
				FROM $this->tagTable t
				INNER JOIN $this->joinerTable j
					ON t.id = j.tag_id
				$where
				ORDER BY t.tag
			");

			//fetch it!
			while ($ar = dbFetchAssoc($rs))
				$tags[$ar['id']] = $ar['tag'];
		}

		//send them home.
		return $tags;
	}

	/**
	* this gets sql code to get tags from the db for a certain object.
	*
	* @return the sql WHERE code to use to get the tags for an object
	*/
	protected function getTagsWhere()
	{
		return "WHERE j.object_id = '$this->objectId'";
	}

	/**
	* gets an array of tags for our object, except formated for humans
	*
	* @return an array of pretty tags
	*/
	public function getPrettyTags()
	{
		$tags = $this->getTags();

		foreach ($tags AS &$tag)
			$tag = self::prettyTag($tag);

		return $tags;
	}

	/**
	* this function gets the ids of all the objects with a certain tag.
	*
	* @param $tagId the tag id to search on
	*
	* @return the string used to get all our objects.
	*/
	public function getObjectsSql($tagId)
	{
		//our query
		return "
			SELECT distinct(z.object_id)
			FROM $this->joinerTable z
			WHERE z.tag_id = '$tagId'
		";
	}

	/**
	* this function gets a list of popular tags for use in a tag cloud or whatnot.
	*
	* @param $count the number of tags to get. default 30
	*
	* @return an array with keys that are our tags, and values which are the
	* counts of tag occurrence
	*/
	public function getPopularTags($count = 30)
	{
		//heres our lookup sql.
		$popularSql = "
			SELECT t.tag, count(j.object_id) AS popularity
			FROM $this->tagTable t
			INNER JOIN $this->joinerTable j
				ON t.id = j.tag_id
			GROUP BY t.tag
			ORDER BY popularity DESC
			LIMIT $count
		";
		$key = "BaseJumper:tags:popular:" . sha1($popularSql);

		//try the cache.
		if (self::$cachePopularTags)
			$tags = CacheBot::get($key, self::$popularTagsCacheLife);
	
		//did we get them?
		if (!$tags)
		{
			$tags = array();
			
			//do our query...
			$rs = dbQuery($popularSql);
			
			//get our array.
			while ($ar = dbFetchAssoc($rs))
				$tags[] = $ar['tag'];

			//save it back to the cache.
			if (self::$cachePopularTags)
				CacheBot::set($key, $tags, self::$popularTagsCacheLife);
		}

		//send it home.
		return $tags;
	}

	/**
	* this gets a list of all the popular tags.
	*
	* @param $count the number of tags to get
	* @return an html string with tags linked to their view pages.
	*/
	public function getPopularTagsList($count = 60)
	{
		//inital stuff.
		$tags = $this->getPopularTags($count);

		//link up!
		$links = $this->getLinkedTags($tags);

		//get 'er done.
		return implode(", ", $links);
	}
	
	/**
	* this gets a weighted list of all the popular tags.
	*
	* @param $count the number of tags to get
	* @return an html string with tags linked to their view pages.
	*/
	public function getWeightedPopularTagsList($count = 100)
	{
		//inital stuff.
		$tags = $this->getPopularTags($count);

		//link up!
		$links = $this->getLinkedTags($tags);

		//loop thru and get the weighted stuff
		$i = 0;
		foreach ($links AS $link)
		{
			if ($i < 5)
			{
				$class = "WeightedTagHighest";
				$style = self::$weightStyles[0];
			}
			else if ($i < 20)
			{
				$class = "WeightedTagHigh";
				$style = self::$weightStyles[1];
			}
			else if ($i < 40)
			{
				$class = "WeightedTagMedium";
				$style = self::$weightStyles[2];
			}
			else if ($i < 65)
			{
				$class = "WeightedTagLow";
				$style = self::$weightStyles[3];
			}
			else
			{
				$class = "WeightedTagLowest";
				$style = self::$weightStyles[4];
			}
			
			$weighted[] = "<span class=\"$class\" style=\"$style\">$link</span>";

			$i++;
		}

		//get 'er done.
		return implode(" ", $weighted);
	}

	/**
	* this function gets the parameters for our tag entry field.
	*
	* @return an array of parameters for the TextAreaField
	*/
	protected function getTagFieldParams()
	{
		return array(
			'id' => strtolower(get_class($this)),
			'value' => implode(", ", $this->getPrettyTags()),
			'title' => "Enter $this->plural<br/>(separated by commas)",
			'class' => 'BaseTags',
			'width' => '95%',
			'height' => '100px',
		);
	}
	
	/**
	* this function gets the parameters for our popular tag field.  it is a
	* labelfield that has our popular tags as javascript that adds them to the
	* tag field.
	*
	* @return an array of parameters for the LabelField
	*/
	protected function getPopularFieldParams()
	{
		return array(
			'title' => "Popular $this->plural<br/>(click to add)",
			'text' => $this->getFormPopularCloud()
		);
	}

	/**
	* update our object's tags from a form object
	*
	* @param $form the form with our new tag data in it.
	*/
	public function updateTagsForm($form)
	{
		//get our data.
		$data = $form->getData(strtolower(get_class($this)));
		$dataRs = explode(",", $data);

		//pass it off to our updater.
		$this->updateTags($dataRs);
	}

	/**
	* update our object's tags in the database.
	*
	* @param $data an array of tags to update.
	*/
	public function updateTags($data)
	{
		//initialization
		$old = $this->getTags();
		$newTags = array();
		$oldTags = array();
		
		//clean them.
		if (count($old))
			foreach ($old AS $tag)
				$oldTags[] = self::cleanTag($tag);
		if (count($data))
			foreach ($data AS $tag)
				$newTags[] = self::cleanTag($tag);

		//add the news ones!
		if (count($newTags))
		{
			foreach ($newTags AS $tag)
			{
				if (!in_array($tag, $oldTags))
					$this->addTagLink($tag);
			}
		}
		
		//delete the old tags...
		if (count($oldTags))
		{
			foreach ($oldTags AS $tag)
			{
				if (!in_array($tag, $newTags))
					$this->removeTagLink($tag);
			}
		}
	}

	/**
	* this function links a tag to our object.  it will also create the tag.
	*
	* @param $tag.  the tag to be linked.
	*/
	public function addTagLink($tag)
	{
		$tag = self::cleanTag($tag);

		//is it a tag?
		if ($tag != '')
		{
			//find/create tag.
			$tagId = $this->addTag($tag);
			
			//insert it.
			if ($tagId)
				$this->addTagLinkDb($tagId);
		}
	}

	/**
	* add a link in the db between our object and this tag
	*
	* @param $tagId the id of the tag in the tag table
	*/
	protected function addTagLinkDb($tagId)
	{
		dbExecute("
			INSERT INTO $this->joinerTable
				(tag_id, object_id)
			VALUES
				('$tagId', '$this->objectId')
		");
	}

	/**
	* this function removes a tag from our object. 
	*
	* @param $tag.  the tag to be unlinked.
	*/
	public function removeTagLink($tag)
	{
		//get our tag id.
		$tagId = $this->findTag($tag);
		
		//delete it.
		if ($tagId)
			$this->removeTagLinkDb($tagId);
	}
	
	/**
	* removes the link between our tag and the object in the db
	*
	* @param $tagId the id of the tag in the db.
	*/
	public function removeTagLinkDb($tagId)
	{
		dbExecute("
			DELETE FROM $this->joinerTable
			WHERE tag_id = '$tagId'
				AND object_id = '$this->objectId'
		");
	}

	/**
	* this function gets tags that are related to a tag.  that is, other tags
	* that objects with this tag are related to.
	*
	* @param $tag the tag to search on.
	*
	* @return an array of tags.
	*/
	public function getRelatedTags($tag)
	{
		//get our tag and object.
		$tagId = $this->findTag($tag);

		$sql = "
			SELECT t.tag, count(t.tag) AS popularity
			FROM $this->tagTable t
			INNER JOIN $this->joinerTable j
				ON t.id = j.tag_id
			WHERE j.object_id IN (" . $this->getObjectsSql($tagId) . ")
				AND j.tag_id != '$tagId'
			GROUP BY t.tag
			ORDER BY popularity DESC, t.tag
			LIMIT 30
		";
		$key = "BaseJumper:tags:popular:" . sha1($sql);

		//try the cache.
		if (self::$cacheRelatedTags)
			$tags = CacheBot::get($key, self::$relatedTagsCacheLife);
	
		//did we get them?
		if (!$tags)
		{
			//init.
			$tags = array();
		
			//did we get it?
			if ($tagId)
			{
				//do our query to find what tags the other objects that use our tag
				$tagRs = dbQuery($sql);

				//get the tags.
				while ($tagAr = dbFetchAssoc($tagRs))
					$tags[] = $tagAr['tag'];
				
				//save it!
				if (self::$cacheRelatedTags)
					CacheBot::set($key, $tags, self::$relatedTagsCacheLife);
			}
		}

		return $tags;
	}

	/**
	* this function links up our tags to their view pages.
	*
	* @param $tags an array of tags
	*
	* @return an array of tags linked to their view page.
	*/
	public function getLinkedTags($tags)
	{
		$links = array();

		if (count($tags))
		{
			//create links.
			foreach ($tags AS $tag)
				$links[] = $this->getLink(".view?tag=$tag", self::prettyTag($tag));
		}

		return $links;
	}

	/**
	* this function gets a list of our object's tags.
	*
	* @param $tags an array of tags to link.  defaults to getTags()
	* @return a html string of tags each of which link to their view page.
	*/
	public function getTagList($tags = null)
	{
		//inital stuff.
		if ($tags === null)
			$tags = $this->getTags();

		//link up!
		$links = $this->getLinkedTags($tags);

		//get 'er done.
		return implode(", ", $links);
	}
	
	/**
	* this function gets a list of related tags. see getRelatedTags() for more info.
	*
	* @param $tag the tag to search on.
	*
	* @return an html string of related tags.  linked to their view pages.
	*/
	public function getRelatedTagsList($tag)
	{
		$tags = $this->getRelatedTags($tag);

		//link up!
		$links = $this->getLinkedTags($tags);

		//get 'er done.
		return implode(", ", $links);
	}

	/**
	* this function is for our tag form.  it shows popular tags that are clickable and when clicked will add them to the form. 
	*
	* @return a html string of tags that add themselves to the tag form when clicked.
	*/
	protected function getFormPopularCloud()
	{
		//get our tags.
		$tags = $this->getTags();
		$popular = $this->getPopularTags();

		//get the ones that we arent using already.
		$usable = array_diff($popular, $tags);

		//create our links..
		if (count($usable))
		{
			foreach ($usable AS $tag)
				$links[] = "<a href=\"#\" onClick=\"return addTagToForm('" . strtolower(get_class($this)) . "', '" .
					self::prettyTag($tag) . "');\">" . self::prettyTag($tag) . "</a>";
		
			return implode(", ", $links);
		}

		return "No popular $this->plural found.";
	}
	
	/**
	* this function adds our tag field to our form.
	*
	* @param $form the form object to add the field to
	*/
	public function addTagField($form)
	{
		$form->add('TextAreaField', strtolower(get_class($this)), $this->getTagFieldParams()); 
	}

	/**
	* this function adds our popular tags field to our object.
	*
	* @param $form the form object to add the field to
	*/
	public function addPopularTagField($form)
	{
		$form->add('LabelField', 'popular_' . strtolower(get_class($this)), $this->getPopularFieldParams());
	}

	/**
	* this function adds both the tag area and the popular tags field to our
	* form.
	*
	* @param $form the form object to add the fields to
	*/
	public function addTagFields($form)
	{
		$this->addTagField($form);
		$this->addPopularTagField($form);
	}
	
	/**
	* gets the table creation sql for the tags
	*
	* @return the create table sql statement(s)
	*/
	public function getCreateTableSql()
	{
		$tagFields = implode(",\n", $this->getTagFieldsArray());
		$tagIndexes = implode(",\n", $this->getTagIndexesArray());
		$sql = "CREATE TABLE IF NOT EXISTS $this->tagTable\n(\n$tagFields,\n$tagIndexes\n);\n";
		
		$joinerFields = implode(",\n", $this->getJoinerFieldsArray());
		$joinerIndexes = implode(",\n", $this->getJoinerIndexesArray());
		$sql .= "CREATE TABLE IF NOT EXISTS $this->joinerTable\n(\n$joinerFields,\n$joinerIndexes\n);\n";

		return $sql;
	}

	/**
	* get an array of fields in the table that stores the tags.
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getTagFieldsArray()
	{
		$fields = array(
			"id" => "id INT(11) not null auto_increment",
			"tag" => "tag varchar(64) default '' not null"
		);

		return $fields;
	}

	/**
	* get an array of indexes on fields in the tag table
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getTagIndexesArray()
	{
		$fields = array(
			"id" => "PRIMARY KEY(id)",
			"tag" => "KEY (tag)"
		);

		return $fields;
	}
	
	/**
	* get an array of fields in the table that joins tags to objects
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getJoinerFieldsArray()
	{
		$fields = array(
			"tag_id" => "tag_id INT(11) not null default 0",
			"object_id" => "object_id INT(11) not null default 0",
		);

		return $fields;
	}

	/**
	* get an array of indexes on fields in the table that joins tags to objects
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getJoinerIndexesArray()
	{
		$fields = array(
			"tag_id" => "KEY(tag_id)",
			"object_id" => "KEY(object_id)",
		);

		return $fields;
	}
}
?>
