<?
/**
* @ingroup ObjectManagers
* @brief Handles drawing a comment thrad for a BaseObject.
*
* The thread object handles drawing comment threads for the associate
* BaseObject.  RSS, threading, etc are all supported.
*/

abstract class BaseThread extends MyManager
{
	/**
	* @b private The id of the content the thread is for.
	*/
	private $contentId;

	/**
	* @b private The name of the class that we're doing comments for.  if this is an imagethread, it would be 'image'
	*/
	private $class;
	
	/**
	* @b public the count of comments in teh thread.
	*/
	public $count = null;
	
	/**
	* this is our construtor!
	*
	* @param $contentId teh id of the content the thread is on
	* @param $class the name of the class that our comments are on... see description of private variable class.
	* @param $table optional parameter for table... defaults to comments_{$class}
	* @param $objectType optional parameter for name of class for comments we're managing... defaults to {$class}Comment
	*/
	function __construct($contentId, $class, $table = null, $objectType = null)
	{
		//who are we on?
		$this->contentId = $contentId;
		$this->class = strtolower($class);

		//default table..
		if ($table === null)
			$table = 'comments_' . $this->class;

		//just use base comments...
		if ($objectType === null)
			$objectType = $this->class . 'Comment';

		//construct
		parent::__construct($objectType, $table);
	}

	/**
	* Creates a new comment of our type...
	*
	* @param $data the data to load the class with.
	*/
	function factory($data = null)
	{
		return new $this->objectType($data, $this->class);
	}

	/**
	* deletes our comment thread.
	*/
	function delete()
	{
		$rs = $this->search(array(
			'content_id' => $this->contentId
		), -1);

		parent::delete($rs);
	}

	/**
	* gets the count of comments in this thread or multiple threads.
	*
	* @param $ids either an id or an array of ids to fetch counts for.
	* @param if $ids is an array, it will return an associative array of counts
	* with the key being the content id, otherwise it will return a count for
	* that content id.
	*/
	function getCommentCount($ids = null)
	{
		if ($this->count !== null)
			return $this->count;

		if (is_array($ids))
		{
			$comRs = dbQuery("
				SELECT content_id, count(content_id) AS count
				FROM {$this->object->tableName}
				WHERE content_id IN (" . implode(',', $ids) . ")
				GROUP BY content_id
			");
			while ($comAr = dbFetchAssoc($comRs))
				$ret[$comAr['content_id']] = $comAr['count'];
			return $ret;
		}
		else
		{
			$comRs = dbQuery("
				SELECT id
				FROM {$this->object->tableName}
				WHERE content_id = '$this->contentId'
			");
			return dbGetNumRows($comRs);
		}
	}

	/**
	* draws a summary of the comments... basically how many comments with a
	* link to view them.  wrapped in a span with class of 'commentSummary'
	*/
	function drawCommentSummary()
	{
		$comCount = $this->getCommentCount();
		
		echo "<span class=\"commentSummary\">\n";
		$url = $this->getUrl(array('module' => $this->class, 'page' => 'view', 'id' => $this->contentId)) . "#comments";
		echo "<a href=\"$url\"><b>$comCount</b> " . Util::pluralize('comment', $comCount) . "</a>";
		echo "</span>\n";
	}

	/**
	* This draws teh comment table. It's wrapped in a div with id of
	* 'commentContainer'  if useRss == true, it will also draw a comment rss
	* feed link.
	*/
	function drawCommentTable()
	{
		echo "<div id=\"commentContainer\">\n";

		echo "<p>\n";
		$form = $this->object->createReplyForm($this->contentId);
		$form->drawAll();
		echo "</p>\n";

		echo "<a name=\"comments\">&nbsp;</a>";
		$this->drawComments();
		
		//do we want rss?
		if ($this->useRss)
			echo $this->getRssLink(".search?output=rss&content_id=$this->contentId", "Subscribe to comment RSS feed.");
		
		echo "\n</div>\n";
	}

	/**
	* This gets all the comments for this content id.
	*
	* @return an array of comment objects
	*/
	function getComments()
	{
		return $this->search(array(
			'content_id' => $this->contentId
		), -1);
	}

	/**
	* this draws all the comments in teh thread.
	*/
	function drawComments()
	{
		$coms = $this->getComments();
		$this->object->drawChildComments($coms, 0);
	}

	/**
	* this initializes teh rss page.
	*/
	function initSearchPage()
	{
		$this->contentId = $this->params('content_id');
		
		parent::initSearchPage();
	}

	/**
	* this genearates the rss object.  The link goes to the content view page
	* and the title is the Config::get('site_name') $class Comment Thread
	*/
	function generateRss($params = array())
	{
		$rss = parent::generateRss($params);
		$rss->link = $this->getUrl(strtolower($this->class) . ".view?id=" . $this->contentId, null, true);
		$rss->title = Config::get('site_name') . " " . ucfirst($this->class) . " Comment Thread";

		return $rss;
	}

	/**
	* add content id as required parameter.
	*
	* this means you can only get comments for an individual object
	*
	* @see ObjectManager::getSearchParamXml()
	*/
	public function getSearchParamXml()
	{
		$xml = '<param name="content_id" desc="the id of the content the comment thread is on" required="1" is_searchable="true" />';
		$xml .= parent::getSearchParamXml();

		return $xml;
	}
}
?>
