<?
/**
* @ingroup BaseModules
* @brief A class to handle automated logins via special links called 'tickets'.
* 
* BaseTicket handles teh creation, validation, and deletion of 'tickets' which
* are basically specially generated links that allow a user to automatically be
* logged in once.  These are for sending emails and such where it is handy for
* a user not to have to remember a password.  They are designed to be secure,
* but may not be entirely secure.  Use them at your own risk.
*/
abstract class BaseTicket extends MyModule
{
	/**
	* The hash of the ticket... basically the ticket id.
	*/
	private $hash;

	/**
	* The url to send the user to.
	*/
	private $url;

	/**
	* A user object for our ticket
	*/
	private $user;
	
	/**
	* The table to use.
	*/
	private $table;
	
	/**
	* our constructor
	*
	* @param $hash the hash we're working with.
	*/
	function __construct($hash = null, $table = 'tickets')
	{
		$this->hash = $hash;
		$this->table = $table;
		
		parent::__construct();
	}

	/**
	* our pages xml function... adds a 'gate' page which processes tickets.
	*
	* @return see BaseModule::getPagesXml()
	*/
	function getPagesXml()
	{
		$xml = parent::getPagesXml();
		$xml .= "<page name=\"gate\">\n";
		$xml .= "\t<param name=\"ticket\" required=\"1\" regex=\"/^[0-9a-f]{16}$/\" />\n";
		$xml .= "\t<param name=\"url\" />\n";
		$xml .= "</page>";

		return $xml;
	}

	function getCreateTableSql()
	{
		return <<<SQL
CREATE TABLE `$this->table` (
  `hash` varchar(16) NOT NULL default '',
  `user_id` int(11) NOT NULL default '0',
  `url` varchar(255) NOT NULL default '',
  `expire` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`hash`),
  KEY `user_id` (`user_id`)
) ENGINE=InnoDB;
SQL;
		return $sql;
	}

	/**
	* initializes teh gate page.  processes teh ticket.  redirects them if its
	* legit, checks for a url and redirects to that (pages should be asserting
	* login if they need users logged in.), otherwise it errors out if there is
	* no url parameter
	*/
	public function initGatePage()
	{
		$this->hash = $this->params('ticket');
	
		if (!$this->process())
		{
			if ($this->params('url'))
				Util::redirect(base64_decode($this->params('url')));
			else
				throw new PageError("No fallback url found.  You must use the entire link contained in the email.");
		}
	}

	/**
	* another placeholder
	*/
	public function drawMainPage()
	{
		global $me;
	}

	/**
	* verify that the ticket is legitimate. (ie it exists in teh database.)
	*
	* @return the db result row on success, false on failure.
	*/
	public function verify()
	{
		$hashRs = dbQuery("
			SELECT *
			FROM $this->table
			WHERE hash = '$this->hash'
		");
		if (dbGetNumRows($hashRs))
			return dbFetchAssoc($hashRs);
		return false;
	}

	/**
	* generates a 16 letter hash based on sha1 of rand.
	*
	* @param $url the url to send the user to.
	* @param $user the user object we want to make a ticket for.
	* @param $expire when to expire... generated via date("Y-m-d H:i:s", foo).  Optional... defaults to 2 months
	*
	* @return the hash key of the ticket.
	*/
	public function generate($url, $user, $expire = null)
	{
		//nice big hash.
		$this->hash = substr(sha1(mt_rand()), 0, 16);
		$this->url = $url;
		$this->user = $user;
		
		//default 2 months
		if ($expire === null)
			$this->expire = date('Y-m-d H:i:s', strtotime('+2 month'));

		//create it.
		dbExecute("
			INSERT INTO $this->table
				(user_id, hash, url, expire)
			VALUES
				('$user->id', '$this->hash', '$this->url', '$this->expire')
		");
		
		return $hash;
	}

	/**
	* gets teh url to use.
	*
	* @return a full url for use in emails, etc.
	*/
	public function url()
	{
		//return $this->getUrl(array('module' => 'ticket', 'page' => 'gate', 'ticket' => $this->hash), null, true);
		return $this->getUrl(".gate?ticket=$this->hash&url=" . base64_encode($this->url), null, true);
	}

	/**
	* process our hash.  if we can verify it, then we create the user, destroy the hash, and redirect them.
	*
	* @return true on success, false on failure
	*/
	public function process()
	{
		if ($hash = $this->verify())
		{
			//if we verify... create it!
			$user = new User($hash['user_id']);
			$user->doLogin();
		
			//destroy!
			$this->destroy();
			
			//redirect..
			Util::redirect($hash['url']);

			return true;
		}
		
		return false;
	}

	/**
	* destroy our hash from the database.
	*/
	public function destroy()
	{
		dbExecute("
			DELETE FROM $this->table
			WHERE hash = '$this->hash'
		");
	}
}
?>
