<?php
/**
* @ingroup ObjectManagers
* @brief A class to handle searching/sorting/pagination of BaseObjects
*
* The parent class that implements all the searching / sorting of all the objects. This class provides outputs on the searched data in a variety of formats:
* html - default, used for showing a website
* xml - XML output of the public data on the objects
* json - JSON output of the public data on the objects.  good for ajax
* php - PHP serialized() output of the public data on the objects.  good for exporting to other PHP scripts
* rss - RSS formatted data on the object.  very handy for creating RSS feeds on anything and everything on your site.
*/
abstract class ObjectManager extends MyModule
{
	/**
	 * @b protected.  The name of the database table its linked to.
	 */
	protected $table;

	/**
	 * @b public.  The name of the class of the object we're working on.
	 */
	public $objectType;
	
	/**
	 * @b public.  Our object that we're working on.
	 */
	public $object;
	
	/**
	* @b public  Bool that determines if we use rss or not
	*/
	public $useRss = true;

	/**
	* @b public  The total number of results we got.
	*/
	public $numResults;

	/**
	* @b public The number of results per page (default: 15)
	*/
	public $numRows;
	
	/**
	* @b private The query string to use for getting objects
	*/
	private $query;

	/**
	* @b private what result page we're on. (default: 0)
	*/
	private $resultPage = 0;

	/**
	* @b private the row to start on.  based on resultPage and numRows
	*/
	private $startRow = 0;

	/**
	* @b private the row to end on.  based on resultPage and numRows
	*/
	private $endRow = 0;

	/**
	* @b private the array of parameters used in the last search
	*/
	private $searchParams = array();

	/**
	* do we cache the search query count?
	*/
	public static $cacheSearchCount = true;

	/**
	* do we cache search query results?
	*/
	public static $cacheSearchQuery = false;

	/**
	* how long to cache the results of our search querys (count and the actual select)
	*/
	public static $searchCacheLife = 300;
	
	/**
	* boolean of whether we use fulltext search or not.
	*/
	public $useFullTextSearch = true;
	
	/**
	* @overload PagingTable::__constructor
	* @param $objectType is the type of object to create for each list item
	* @param $table is the table for the BaseObject
	* @param $numRows the number of rows per page.  default: 15
	*/
	function __construct($objectType, $table = null, $numRows = 15)
	{
		//give it to me, daddy.
		parent::__construct();

		//set and create our object.
		$this->objectType = $objectType;
		$this->object = $this->factory();
		
		//if we didnt get a table, get it from the object class
		if ($table === null)
		{
			$this->table = $object->tableName;
			$this->english = $this->object->english;
		}
		else
			$this->table = $table;

		//set our numrows.
		$this->numRows = $numRows;
	}

	/**
	* create a new object of the type we need.
	*
	* @param $data optional data to pass to the object.
	* @return the newly created object.
	*/
	function factory($data = null)
	{
		return new $this->objectType($data);
	}

	/**
	* get our option xml string for what we can sort on.
	*
	* @return the xml string of sort options
	*/
	function getSortOptionXml()
	{
		$rval = "";

		//get our array and make the xml;
		$sort = $this->getSortFieldsArray();
		foreach ($sort AS $key => $val)
			$rval .= "<option value=\"$key\" desc=\"$val\"/>\n";

		return $rval;
	}

	/**
	* this function gets all the possible sort values.  override this to add
	* sortable fields.  name, birthday, etc.
	*
	* @return an array with keys as the sortable fields, and values of the description
	*/
	protected function getSortFieldsArray()
	{
		$sort = array();
		
		if ($this->object->hasField('add_date'))
			$sort['add_date'] = 'sort by date added';
			
		if ($this->object->hasField('edit_date'))
			$sort['edit_date'] = 'sort by date last edited/modified';
		
		if ($this->object->hasField('comment_count'))
			$sort['comment_count'] = 'sort by number of comments';
		
		if ($this->object->hasField('user_id'))
			$sort['user_id'] = 'sort by user id';
		
		$sort['id'] = 'sort the results by id';
		$sort['random'] = 'sort the results randomly';

		//do we want to sort by fulltext?
		if ($this->useFullTextSearch)
			$sort['relevancy'] = 'sort by fulltext relevancy';
	
		return $sort;
	}

	/**
	* delete a set of objects.
	*
	* @param $rs an array of objects to delete.
	*/
	public function delete($rs)
	{
		if (count($rs))
			foreach ($rs AS $ob)
				$ob->delete();
	}

	/**
	* get our pages... default + php, rss, json, and search.
	*
	* @return the xml string of pages.
	*/
	public function getPagesXml()
	{
		//obviously we want the base xml.
		$xml = parent::getPagesXml();

		//main pages are rss and search.
		$xml .= $this->getSearchPageXml();
		
		return $xml;
	}
	
	/**
	* our search page xml configuration.
	*
	* @return the xml string of search page & params.
	*/
	public function getSearchPageXml()
	{
		$xml = "\t<page name=\"search\" desc=\"search for {$this->english}s from the manager.\">\n";
		$xml .= $this->getSearchParamXml();
		$xml .= "\t</page>\n";

		return $xml;
	}
	
	/**
	* our search page parameter xml configuration.
	*
	* @return the xml string of search params.
	*/
	public function getSearchParamXml()
	{
		//init
		$xml = '';
		
		//just get a set of objects
		$xml .= "\t\t<param name=\"ids\" desc=\"a comma separated list of ids you want.\" />\n";

		if ($this->useFullTextSearch)
			$xml .= "\t\t<param name=\"query\" desc=\"a string to query the database against.\" />\n";
		
		//any field specific stuff...
		$fields = $this->object->getDbFields();
		
		//allow to search for things with this parent.
		if (is_array($fields['parent_id']))
			$xml .= "\t\t<param name=\"parent_id\" desc=\"the id of the	parent.\" is_searchable=\"true\" type=\"int\"/>\n";
		
		//or search on a user.
		if (isset($fields['user_id']))
			$xml .= "\t\t<param name=\"user_id\" desc=\"the id of the user who created the {$this->object->english}\" is_searchable=\"true\" type=\"int\"/>\n";
		
		//do we give them lat/lon access?
		if (isset($this->object->useLatLng))
		{
			if (isset($fields['lat']))
			{
				$xml .= "\t\t<param name=\"northlat\" type=\"float\" is_searchable=\"true\"/>\n";
				$xml .= "\t\t<param name=\"southlat\" type=\"float\" is_searchable=\"true\"/>\n";
			}

			if (isset($fields['lon']))
			{
				$xml .= "\t\t<param name=\"eastlon\" type=\"float\" is_searchable=\"true\"/>\n";
				$xml .= "\t\t<param name=\"westlon\" type=\"float\" is_searchable=\"true\"/>\n";
			}
		}

		//add date is good.
		if (isset($fields['add_date']))
		{
			$xml .= "\t\t<param name=\"add_date_start\" desc=\"start date for your query, eg 10-22-1983. it will get all the {$this->object->english}s added on or after it\" regex=\"/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}(isset([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}))?$/\"/>\n";
			$xml .= "\t\t<param name=\"add_date_stop\" desc=\"end date for your query. it will get all the {$this->object->english}s added on or before it.  used with start date you can specify a range.\"  regex=\"/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}(isset([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}))?$/\"/>\n";
		}

		//as is edit date
		if (isset($fields['edit_date']))
		{
			$xml .= "\t\t<param name=\"edit_date_start\" desc=\"start date for your query, eg 10-22-1983. it will get all the {$this->object->english}s edited on or after it.\" regex=\"/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}(isset([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}))?$/\"/>\n";
			$xml .= "\t\t<param name=\"edit_date_stop\" desc=\"end date for your query. it will get all the {$this->object->english} edited on or before it.  used with start date you can specify a range\" regex=\"/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}(isset([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}))?$/\"/>\n";
		}
		
		//heres our sorting type stuff
		$xml .= "\t\t<param name=\"result_page\" desc=\"the page number you want. defaults to 0\" type=\"int\"/>\n";
		$xml .= "\t\t<param name=\"num_results\" desc=\"the number of results per page\" type=\"int\"/>\n";
		$xml .= "\t\t<param name=\"sort\" desc=\"the parameter you want to sort by.\">\n";
		$xml .= $this->getSortOptionXml();
		$xml .= "\t\t</param>\n";
		$xml .= "\t\t<param name=\"order\" desc=\"how you want to order the objects\">\n";
		$xml .= "\t\t\t<option value=\"asc\" desc=\"descending order... highest to lowest\" />\n";
		$xml .= "\t\t\t<option value=\"desc\" desc=\"ascending order... lowest to highest\" />\n";
		$xml .= "\t\t</param>\n";

		//what type of output do we want?
		$xml .= $this->getSearchOutputXml();

		return $xml;
	}

	/**
	* get the xml for our search output types
	*
	* @return the output types in xml format 
	*/
	protected function getSearchOutputXml()
	{
		return <<<XML
			<param name="output">
				<option value="html" description="plain HTML output"/>
				<option value="xml" description="data as structured xml"/>
				<option value="rss" description="data as an rss feed"/>
				<option value="json" description="data in JSON format"/>
				<option value="php" description="data in PHP serialized() format"/>
			</param>
XML;
	}
		
	/**
	* initialize our main page... basically redirect to search page. you'll lose your parameters... dont link to main page.
	*/
	function initMainPage()
	{
		Util::redirect($this->getUrl(".search"));
	}

	/**
	* initialize our search page.  pass off to initSearch()
	*/
	function initSearchPage()
	{
		//set our max results param
		if((int)$this->params('num_results') > 200)
			$this->setParam('num_results', 200);

		//make sure we can do fulltext search...
		if ($this->params('query') && !count($this->object->fullTextFields))
			throw new PageError('Oops, there are no fulltext fields to search in ' . get_class($this->object));
	
		$this->initSearch($this->params());

		//titles are nice =)
		$this->pageTitle = ucfirst($this->object->english) . " Search";
	
		//what output do we want?
		$this->initSearchPageOutput();
	}

	/**
	* initialize the search page output type.  passes it off to each initFooOutput() function
	*/
	protected function initSearchPageOutput()
	{
		//how do you want it?  how does it feel?
		$output = $this->params('output');
		switch ($output)
		{
			case 'rss':
				$this->initRSSOutput();
				break;
				
			case 'json':
				$this->initJSONOutput();
				break;
				
			case 'php':
				$this->initPHPOutput();
				break;

			case 'xml':
				$this->initXMLOutput();
				break;

			case 'html':
			default:
				$this->initHTMLOutput();
				break;
		}
	}
	
	/**
	* init the page for PHP output: blank template
	*/
	protected function initPHPOutput()
	{
		$this->setTemplate(new BlankTemplate());
	}
	
	/**
	* init the page for JSON output: blank template
	*/
	protected function initJSONOutput()
	{
		$this->setTemplate(new BlankTemplate());
	}
	
	/**
	* init the page for RSS output: blank template
	*/
	protected function initRSSOutput()
	{
		$this->setTemplate(new BlankTemplate());
	}

	/**
	* init page for XML output:  XMLTemplate and checks to see if needed classes exist
	*/
	protected function initXMLOutput()
	{
		//we want the xml template
		$this->setTemplate(new XMLTemplate());
	
		//make sure we have the serializer class
		if (!class_exists('XML_Serializer'))
			throw new PageError('BaseJumper xml data export depends on Pear::XML_Serializer to function.');
	}

	/**
	* init page for HTML output:  nothing to do
	*/
	protected function initHTMLOutput()
	{
	}

	/**
	* draw our search page!
	*/
	public function drawSearchPage()
	{
		//what is our output type?
		$output = $this->params('output');
		if (!$output)
			$output = 'html';

		//now, how do we draw it.
		switch ($output)
		{
			case 'rss':
				$this->drawRSSOutput();	
				break;

			case 'json':
				$this->drawJSONOutput();	

				break;

			case 'php':
				$this->drawPHPOutput();	
				break;

			case 'xml':
				$this->drawXMLOutput();	
				break;

			case 'html':
			default:
				$this->drawHTMLOutput();	
				break;
		}
	}

	/**
	* draw our HTML output:  draw a search form, and the results
	*/
	protected function drawHTMLOutput()
	{
		//get our results
		$rs = $this->getSearchResult();
		
		//draw our search form.
		if ($this->useFullTextSearch)
			$this->drawSearchForm();

		//now draw them.
		if (count($rs))
			$this->drawResults($rs);
		else
			$this->drawNoResults();
	}
	
	/**
	* draw our rss page.  generate, draw, and save.
	*/
	protected function drawRssOutput()
	{
		$feed = $this->generateRss($this->params());
		$feed->saveFeed('RSS2.0', '', true);
	}


	/**
	* draw our json page.  get public data and encode.
	*/
	protected function drawJSONOutput()
	{
		$data = $this->getPublicData($this->params());

		echo JSON::encode($data);
	}
	
	/**
	* draw our xml page.  get public data and encode.
	*/
	protected function drawXMLOutput()
	{
		$options = array(
			"indent"         => "\t",
			"linebreak"      => "\n",
			"typeHints"      => false,
			"addDecl"        => false,
			"defaultTagName" => get_class($this->object),
			"rootName" => "BaseJumper"
		);
		//setup our serializer
		$serial = new XML_Serializer($options);
		
		//serialize our data.
		$data = $this->getPublicData($this->params());
		if ($serial->serialize($data))
			echo $serial->getSerializedData();
	}
	
	/**
	* draw our php serialize page... echo the serialized() public data.
	*/
	protected function drawPHPOutput()
	{
		echo serialize($this->getPublicData($this->params()));
	}

	
	/**
	* initialize our search.  sets up the query string.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	function initSearch($params)
	{
		//fix slashies!
		foreach ($params AS $key => $val)
			$params[$key] = dbEscape($val);
		
		//save our parameters.
		$this->searchParams = $params;
		
		//did we have a parameter for number of results.
		if ($params['num_results'])
			$this->numRows = (int)$params['num_results'];
		
		//finally set the query.
		$this->query = $this->getSearchQuery($params);
	}

	/**
	* get teh entire search query.
	*
	* @param $params a keyed array of parameters.  same as search page
	*
	* @return the entire search wuere, limit and all
	*/
	public function getSearchQuery($params)
	{
		//our basic search
		$select = $this->getSearchSelect($params);
		$countSel = $this->getSearchCountSelect($params);
		$from = $this->getSearchFrom($params);
		$where = $this->getSearchWhere($params);
		$group = $this->getSearchGroupBy($params);
		$order = $this->getSearchOrder($params);

		//find out how many results we got.
		$countSql = $countSel . $from . $where;
		
		//do we look it up?
		if (self::$cacheSearchCount)
			$countAr = CacheBot::get('BaseJumper:count:' . sha1($countSql), self::$searchCacheLife);

		//no cache?  no problem.
		if (!$countAr)
		{
			$countAr = dbFetchAssoc(dbQuery($countSql));

			//do we cache it?
			if (self::$cacheSearchCount)
				CacheBot::set('BaseJumper:count:' . sha1($countSql), $countAr, self::$searchCacheLife);
		}
		
		//get the count.
		$this->numResults = $countAr['count'];

		//get our limit stuff...
		$this->resultPage = abs((int)$params['result_page']);

		//do a bit of error checking on the bounds of the result.
		if ($this->resultPage * $this->numRows > $this->numResults)
			throw new PageError('We do not have that many ' . $this->english . "s.");
	
		//add our limit in here...
		$limit = " LIMIT " . $this->resultPage * $this->numRows . ", $this->numRows";

		//give it back to the man.
		return $select . $from . $where . $group . $order . $limit;
	}
	
	/**
	* get the select part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchSelect($params = array())
	{
		$sql = "SELECT o.id\n";

		//add our fulltext relevancy in.
		if ($this->useFullTextSearch && $params['query'])
			$sql .= ", " . $this->getFullTextSearch($params['query']) . " AS relevancy\n";

		return $sql;
	}
	
	/**
	* get the select count part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchCountSelect($params = array())
	{
		return "SELECT count(distinct(o.id)) AS count\n";
	}

	/**
	* get the from/join part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchFrom($params = array())
	{
		$from = "FROM {$this->object->tableName} o \n";

		return $from; 
	}

	/**
	* get the where part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchWhere($params = array())
	{
		global $me;

		$where = "WHERE 1 ";

		//our xml configuration.
		$xml = $this->parseModuleXml();
		$rs = $xml->xpath("//page[@name='search']");

		//get our where stuff...
		if (is_object($rs[0]->param[0]))
		{
			//loop thru all our params in our object.
			foreach ($rs[0]->param AS $param)
			{
				//get that param.
				$pName = (string)$param['name'];
				$pVal = $params[$pName];

				//did we get one?
				if ($pVal)
				{
					//listing of ids?
					if ($pName == 'ids')
						$where .= " AND o.id IN ($pVal) ";

					//by user id.
					if ($pName == 'user_id')
						$where .= " AND o.user_id = '$pVal' ";
					
					//heres our date stuff...
					else if ($pName == 'add_date_start')
						$where .= " AND o.add_date >= '$pVal' ";
					else if ($pName == 'add_date_stop')
						$where .= " AND o.add_date <= '$pVal' ";
					else if ($pName == 'edit_date_start')
						$where .= " AND o.edit_date >= '$pVal' ";
					else if ($pName == 'edit_date_stop')
						$where .= " AND o.edit_date <= '$pVal' ";
					
					//how about location?
					else if ($pName == 'northlat')
						$where .= " AND o.lat <= '$pVal' AND o.lat != 0";
					else if ($pName == 'southlat')
						$where .= " AND o.lat >= '$pVal'";
					else if ($pName == 'eastlon')
						$where .= " AND o.lon <= '$pVal' AND o.lon != 0";
					else if ($pName == 'westlon')
						$where .= " AND o.lon >= '$pVal'";
					
					//if it was passed and is searchable... try it.
					else if ($pVal && (string)$param['is_searchable'] == 'true')
						$where .= " AND o.$pName = '$pVal'";
				}
			}
		}

		//add our privacy where.
		$where .= $this->getPrivacyWhere($params);
		
		//what about our fulltext search
		if ($this->useFullTextSearch && $params['query'] && count($this->object->fullTextFields))
			$where .= " AND " . $this->getFullTextSearch($params['query']);
		
		return $where;
	}

	/**
	* get our fulltext search MATCH string.
	*
	* @param the string to match against.
	*
	* @return the match string to use in sql.
	*/
	protected function getFullTextSearch($query)
	{
		//if we can use it, do it.
		if ($this->useFullTextSearch && count($this->object->fullTextFields))
			return "MATCH (" . implode(',', $this->object->fullTextFields) . ") AGAINST ('$query')";
	
		return '';
	}

	/**
	* get the group by part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchGroupBy($params = array())
	{
			return '';
	}

	/**
	* get the where by part of our search query.  this is specifically for privacy options.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getPrivacyWhere($params = array())
	{
	}

	/**
	* get the order part of our search query.
	*
	* @param $params a keyed array of parameters.  same as search page.
	*/
	protected function getSearchOrder($params = array())
	{
		//get our sort...
		$sortVar = $params['sort'];
		$order = $params['order'];

		//if we dont have order and sort... add date desc!
		if (!$order && !$sortVar)
			$order = 'DESC';
		
		if ($sortVar == 'random')
			$sort = " ORDER BY rand() ";
		else if ($sortVar)
		{
			if ($this->object->hasField($sortVar))
				$sort = " ORDER BY o.$sortVar $order";
			else
				$sort = " ORDER BY $sortVar $order";
		}
		//add our fulltext order if we can
		else if ($this->useFullTextSearch && $params['query'])
			$sort = " ORDER BY relevancy $order";
		else if ($this->object->hasField('add_date'))
			$sort = " ORDER BY o.add_date $order";
		else
			$sort = '';

		return $sort;
	}

	/**
	* draws our pagination info wrapped in a div with id of 'pagination'
	*/
	function drawPagination()
	{
		echo "<div id=\"pagination\">\n";
		echo $this->getPaginationStats();
		echo $this->getPaginationNav();
		echo "</div>\n";
	}

	/**
	* get a string about our pagination.  a-b of x, etc.
	*
	* @return the string ready to use
	*/
	function getPaginationStats()
	{
		return "Showing $this->startRow - $this->endRow of $this->numResults {$this->english}s. ";
	} 
	
	/**
	* get our pagination nav.  pages, prev/next, etc.
	*
	* @return our string ready to use.
	*/
	function getPaginationNav()
	{
		$s = '';
		//do we have any pages to show?
		if ($this->hasPage($this->resultPage-1) || $this->hasPage($this->resultPage+1))
		{
			$s = "Page: ";
		
			//get our previous
			if ($this->resultPage >= 1)
			{
				$s .= $this->getPaginationFirst();
				$s .= " ";
				$s .= $this->getPaginationPrev();
			}

			//show the nearest 5 pages.
			for ($i = $this->resultPage - 5; $i <= $this->resultPage + 5; $i++)
			{
				if ($i == $this->resultPage)
					$s .= " <b>" . ($this->resultPage + 1) . "</b> ";
				else if ($this->hasPage($i))
					$s .= " " . $this->getPaginationLink($i) . " ";
			}
			
			//show our next link
			if ($this->endRow < $this->numResults)
			{
				$s .= $this->getPaginationNext();
				$s .= " ";
				$s .= $this->getPaginationLast();
			}
		}

		return $s;
	}

	/**
	* get a pagination url
	*
	* @param $page the page to use
	*
	* @return the url of that page
	*/
	function getPaginationUrl($page)
	{
		//update our parameters
		$params = $this->searchParams;
		$params['page'] = 'search';
		$params['result_page'] = $page;
		
		//get the link.
		return $this->getUrl($params);
	}

	/**
	* get a pagination link
	*
	* @param $page the page to link to.
	* @param $link the text to link.  default null = number of the page.
	*
	* @return a html link.
	*/
	function getPaginationLink($page, $link = null)
	{
		$url = $this->getPaginationUrl($page);

		//link to our page if none.
		if ($link === null)
			$link = $page + 1;
		
		//get our link
		return "<a href=\"$url\">$link</a>";
	}

	/**
	* get our first page link.
	*
	* @return the markup string.
	*/
	function getPaginationFirst()
	{
		return $this->getPaginationLink(0, '<< first');
	}
	
	/**
	* get our previous page link.
	*
	* @return the markup string.
	*/
	function getPaginationPrev()
	{
		return $this->getPaginationLink($this->resultPage - 1, '< prev');
	}

	/**
	* get our next page link.
	*
	* @return the markup string.
	*/
	function getPaginationNext()
	{ 
		return $this->getPaginationLink($this->resultPage + 1, 'next >');
	}
	
	/**
	* get our last page link.
	*
	* @return the markup string.
	*/
	function getPaginationLast()
	{ 
		$page = floor($this->numResults / $this->numRows); 

		return $this->getPaginationLink($page, 'last >>');
	}
	
	/**
	* determines if a page exists or not
	*
	* @param $page the number of the page.
	*
	* @return boolean of its status
	*/
	function hasPage($page)
	{
		//negative pages == barf
		if ($page < 0)
			return false;
		
		//if we show are page, are there still results?
		return (($this->numResults - ($page * $this->numRows)) > 0);
	}
	
	/**
	* calculate our pagination stuff... what we start on, end on, etc.
	*/
	function calculatePagination()
	{
		$this->startRow = ($this->resultPage * $this->numRows) + 1;
		$this->endRow = ($this->resultPage + 1) * $this->numRows;

		if ($this->endRow > $this->numResults)
			$this->endRow = $this->numResults;

		if ($this->endRow == 0)
			$this->startRow = 0;
	}

	/**
	* get our search result.  used to do query, setup array of objects, preload
	* secondary objects to save queries (users, comments, etc..), also
	* calculates pagination.
	*
	* @return the array of result objects
	*/
	function getSearchResult()
	{
		//do our search.
		$data = $this->doObjectSearch();
		
		//load up our objects
		$ret = $this->loadObjects($data);

		//set our pagination
		$this->calculatePagination();
		
		return $ret;
	}

	/**
	* do the search for the objects
	*
	* if ObjectManager::cacheSearchQuery is true, it will try to load it from
	* the cache.  otherwise it will do the query itself.
	*
	* @return an array of  arrays of object data from the DB
	*/
	protected function doObjectSearch()
	{
		//get our objects
		if (self::$cacheSearchQuery)
			$data = CacheBot::get('BaseJumper:search:' . sha1($this->query), self::$searchCacheLife);

		//no cache?  no problem.
		if (!is_array($data))
		{
			//init so the cache save works
			$data = array();
			
			//do our query.
			$rs = dbQuery($this->query);
			while ($ar = dbFetchAssoc($rs))
				$data[] = $ar;

			//save it.
			if (self::$cacheSearchQuery)
				CacheBot::set('BaseJumper:search:' . sha1($this->query), $data, self::$searchCacheLife);
		}

		//send it back
		return $data;
	}

	/**
	* load the database data into an array of objects
	*
	* @todo make this much more efficient and not so shitty v2.2 baby!
	*
	* @param the data from the db in an array of arrays
	* @return an array of objects to use
	*/
	protected function loadObjects($data)
	{
		$objs = array();
		
		//did we get any ids?
		if (count($data))
		{
			//get all our cache keys
			$keys = array();
			foreach ($data AS $row)
				$keys[] = $this->object->getCacheKey($row['id']);
			//get our data back.
			$data = CacheBot::get($keys, BaseObject::$objectCacheLife);
			foreach ($data AS $key => $val)
			{
				$info = explode(":", $key);
				$id = $info[3];

				//was the cache lookup successful?
				if ($val)
					$objs[$id] = $this->factory($val);
				//nope, have it load itself
				else
					$objs[$id] = $this->factory($id);
	 		}
	 	}

		return $objs;
	}

	/**
	* draws our result set.  pagination, rows, rss link.
	*
	* @param $rs an array of objects.
	*/
	function drawResults($rs)
	{
		$this->drawPagination();
		$this->drawRows($rs);
		$this->drawPagination();

		if ($this->useRss)
			$this->drawRssLink();
	}

	/**
	* draw our rss link.
	*
	* @param $params parameters to append to the rss link.
	*/
	function drawRssLink($params = null)
	{
		$link = ".search?output=rss";
		
		//get some params
		if ($params === null)
			$params = $this->params();
		
		//add them in.
		$get = array();
		foreach ($params AS $key => $param)
		{
			if ($key != 'result_page')
				$get[] = "$key=$param";
		}
		if (count($get))
			$link .= "&" . implode("&", $get);

		echo $this->getRssLink($link, "Subscribe to this {$this->object->english} feed.");
	}

	/**
	* This gets a link to the rss feed!  Its super easy. Plus it includes the rss icon and a help link.
	*
	* @param $link the link to the rss page.  generally its '.rss?param=value'
	* @param $text the text to include in the link describing it.
	* @param $addFeed boolean to add a feed link in the header... saves lots of work, dont it?
	*/
	public function getRssLink($link, $text, $addFeed = true)
	{
		$txt = "<div id=\"commentRss\"><img src=\"lib/img/rss-16x16.png\"/><span>" . $this->getPlainRssLink($link, $text, $addFeed) . "</span>";
		$txt .= '<a href="http://news.bbc.co.uk/2/hi/help/3223484.stm"><img src="lib/img/question.gif" width="17" height="17"/></a></div>';

		return $txt;
	}

	/**
	* get a plain text link to the rss feed
	*
	* @param $link the feed to link to
	* @param $text the text to link
	* @param $addFeed boolean if we should add feed to header or not
	* @return the html for the rss link
	*/
	public function getPlainRssLink($link, $text, $addFeed = true)
	{
		if ($addFeed)
			$this->addFeed($this->getUrl($link), $text);

		return $this->getLink($link, $text);
	}

	/**
	* this function is called on a search page when there are no results.
	*/
	function drawNoResults()
	{
		echo "<p>No {$this->english}s found.</p>\n";
	}

	/**
	* this draws the rows for each object. calls drawHeaderRow() on teh object,
	* and then drawRow() on each object in teh result set. each object is
	* wrapped in a div of class $class.
	*
	* @param $rs an array of objects to draw.
	* @param $class the css class to wrap the objects in.
	*/
	function drawRows($rs, $class = null)
	{
		//init our flag
		$flag = true;
		
		//what class is it?
		if ($class === null)
			$class = "contentRow " . strtolower($this->objectType);

		//did we get anything?
		if (count($rs))
		{
			//draw out header and loop
			$this->object->drawHeaderRow();
			foreach ($rs AS $ar)
			{
				//can we view it?
				$this->object = $ar;
				if ($this->object->canView())
				{
					//our row flags
					if ($flag)
						$classRow = "onRow";
					else
						$classRow = "offRow";
				
					//draw our object
					echo "\n<div class=\"$class\">\n";
					echo "\n<div class=\"$classRow\">\n";
					$this->object->drawRow();
					echo "</div>\n";
					echo "</div>\n";

					//change our flag
					$flag = !$flag;
				}
			}
		}
	}
	
	/**
	* This draws all the objects in one-line form.  they are wrapped in a div of objectType + Line
	*
	* @param $rs an array of objects to draw.
	* @param $class the css class to wrap the objects in.
	*/
	function drawLines($rs, $class = null)
	{
		if ($class === null)
			$class = strtolower($this->objectType) . "Line";
		

		if (count($rs))
		{
			echo "<div class=\"ObjectLines\">";
			$this->object->drawHeaderLine();
			foreach ($rs AS $ar)
			{
				$id = strtolower($this->objectType) . "Line-$ar->id";
				$this->object = $ar;
				if ($this->object->canView())
				{
					echo "\n<div id=\"$id\" class='$class'>\n";
					$this->object->drawLine();
					echo "</div>\n";
				}
			}
			echo "</div>";
		}
	}

	/**
	* search for a user's content..
	*
	* @param $userId the id of the user.
	* @param $limit max results to get.
	*
	* @return the result set.
	*/
	function userSearch($userId, $limit = null)
	{
		return $this->search(array(
			'user_id' => $userId
		), $limit);
	}

	/**
	* search for objects.  you can call this to do custom searches.
	*
	* @param $params a keyed array of parameters to search on.
	* @param $limit the maximum results to return.  defaults to numRows, pass -1 for unlimited.
	*
	* @return the result set.
	*/
	public function search($params = array(), $limit = null)
	{
		//know your limit!
		if ($limit == -1)
			$limit = 99999999;

		//set it!
		if ($limit !== null)
			$params['num_results'] = $limit;

		//prep it!
		$this->initSearch($params);

		//now actually search
		return $this->getSearchResult();
	} 
	
	/**
	* just like search, except all you get is a count of objects.  use this
	* when you only want the count (a cheap query) and not an array of the
	* results.
	*
	* @param $params a keyed array of parameters to search on.
	* @param $cache do we use the cache of teh count or start fresh?
	*
	* @return the count of objects that match.
	*/
	public function searchCount($params = array(), $cache = true)
	{
		//save our old cache setting.
		$oldCache = self::$cacheSearchCount;

		//set our new one.
		self::$cacheSearchCount = $cache;
		
		//prep it!
		$this->initSearch($params);

		//save it back in now
		self::$cacheSearchCount = $oldCache;

		//now actually search
		return $this->numResults;
	} 
	
	/**
	* generate our rss. uses the feedcreator class.
	*
	* @param $params a keyed array of parameters as for search used to search for rss items.
	* @param $useCached boolean of whether we want to cache. default: true
	*
	* @return the UniversialFeedCreator object.
	*/
	function generateRss($params = array(), $useCached = true)
	{
		require_once("lib/feedcreator.class.php");

		//create the object
		$rss = new UniversalFeedCreator();

		//do we use cached?
		if ($useCached)
			$rss->useCached(); // use cached version if age<1 hour
		
		$rss->title = $this->pageTitle . " - " . Config::get('site_name');
		$rss->description = $this->object->english . " feed"; 
		$item->descriptionHtmlSyndicated = true;
		
		//get our links...
		$rss->link = "http://" . Config::get('site_hostname');
		$rss->syndicationURL = "http://" . Config::get('site_hostname') . $_SERVER["PHP_SELF"];

		//do our search
		$rs = $this->search($params);
		if (count($rs))
		{
			foreach ($rs AS $ob)
			{
				$item = $ob->getRssItem();
				$rss->addItem($item);
			}
		}

		return $rss;
	}

	/**
	* get our public data.  actually its an array of the object's public data... but same diff.
	*
	* @param $params a keyed array of parameters to search on.  same as search()
	*
	* @return an array of public data.  woot!
	*/
	function getPublicData($params)
	{
		//get our search results.
		$results = array();
		$rs = $this->search($params);
		if (count($rs))
			foreach ($rs AS $ob)
				$results[] = $ob->getPublicData();
	
		//format our data.
		$data = array(
			'runtime' => '%%BJ_RUNTIME%%',
			'queries' => '%%BJ_QUERIES%%',
			'cachehits' => '%%BJ_CACHE_HITS%%',
			'page' => $this->resultPage,
			'startrow' => $this->startRow,
			'endrow' => $this->endRow,
			'totalresults' => $this->numResults,
			'resultsperpage' => $this->numRows 
		);
		$data['Results'] = $results;

		return $data;
	}

	/**
	 * This function determines admin status. (passes off to object->isAdmin())
	 * @return true if they are admin, false if not.
	 */
	public function isAdmin()
	{
		return (bool)$this->object->isAdmin();
	}
	
	/**
	* create our search form for searching.
	*
	* @return returns a Form object that does searching.
	*/
	public function createSearchForm($query = null)
	{
		$form = new Form(new OneLineLayout());
		$form->action = $this->getUrl(".search");
		$form->onSubmit = "return objectManagerSearch('$form->name')";

		$form->add('TextField', 'query', array(
			'title' => 'Search ' . ucfirst(Util::pluralize($this->object->english, 2)) . ' ',
			'size' => 50,
			'required' => true
		));
		
		$form->addSubmit('Search ');

		//if null.. default to our param
		if ($query === null)
			$query = $this->params('query');

		//set our data.
		if ($query)
			$form->setData(array(
				'query' => $query
			));

		return $form;
	}

	/**
	* draw our search form for doing fulltext searches
	*/
	public function drawSearchForm()
	{
		$this->needsJs('lib/js/basejumper.js');

		echo "<div class=\"ObjectSearch\" id=\"" . get_class($this->object) . "Search\">\n";
		$form = $this->createSearchForm();
		$form->draw();
		echo "</div>";
	}
}
